@@ -19,7 +19,6 @@ package org.apache.kudu.backup
 import java.nio.file.Files
 import java.nio.file.Path
 import java.util
-
 import com.google.common.base.Objects
 import org.apache.commons.io.FileUtils
 import org.apache.kudu.client.PartitionSchema.HashBucketSchema
@@ -83,6 +82,64 @@ class TestKuduBackup extends KuduTestSuite {
     validateTablesMatch(tableName, s"$tableName-restore")
   }
 
+  @Test
+  def testAutoIncrementingColumnBackupAndRestore() {
+    val rowCount = 100
+    val expectedRowCount = 200
+    val simpleAutoIncrementingTableOptions = new CreateTableOptions()
+      .setRangePartitionColumns(List("key").asJava)
+      .setNumReplicas(1)
+    val AutoIncrementingTable = kuduClient.createTable(
+      simpleAutoIncrementingTableName,
+      simpleAutoIncrementingSchema,
+      simpleAutoIncrementingTableOptions)
+
+    val session = kuduClient.newSession()
+
+    // Insert some rows.
+    Range(0, rowCount).foreach { i =>
+      val insert = AutoIncrementingTable.newInsert
+      val row = insert.getRow
+      row.addInt("key", i)
+      row.addString("val", s"a$i")
+      session.apply(insert)
+    }
+
+    // Perform a full backup.
+    backupAndValidateTable(simpleAutoIncrementingTableName, rowCount, false)
+    // Insert some more rows.
+    Range(rowCount, 2 * rowCount).foreach { i =>
+      val insert = AutoIncrementingTable.newInsert
+      val row = insert.getRow
+      row.addInt("key", i)
+      row.addString("val", s"a$i")
+      session.apply(insert)
+    }
+    // Perform an incremental backup.
+    backupAndValidateTable(simpleAutoIncrementingTableName, rowCount, true)
+
+    // Restore the table.
+    restoreAndValidateTable(simpleAutoIncrementingTableName, expectedRowCount)
+    // Validate the table schemas match.
+    validateTablesMatch(
+      simpleAutoIncrementingTableName,
+      s"$simpleAutoIncrementingTableName-restore")
+
+    // Validate the data written in the restored table.
+    val restoreTable = kuduClient.openTable(s"$simpleAutoIncrementingTableName-restore")
+    val scanner = kuduClient.newScannerBuilder(restoreTable).build()
+    val rows = scanner.asScala.toList
+    assertEquals(expectedRowCount, rows.length)
+    var i = 0
+    rows.foreach { row =>
+      assertEquals(i, row.getInt("key"))
+      assertEquals(i + 1, row.getLong(Schema.getAutoIncrementingColumnName))
+      assertEquals(s"a$i", row.getString("val"))
+      i += 1
+    }
+    assertEquals(expectedRowCount, rows.length)
+  }
+
   @Test
   def testSimpleIncrementalBackupAndRestore() {
     insertRows(table, 100) // Insert data into the default test table.
@@ -590,6 +647,119 @@ class TestKuduBackup extends KuduTestSuite {
     restoreAndValidateTable(tableName, 50)
   }
 
+  @Test
+  def testTableWithOnlyCustomHashSchemas(): Unit = {
+    // Create the initial table and load it with data.
+    val tableName = "testTableWithOnlyCustomHashSchemas"
+    val table = kuduClient.createTable(tableName, schema, tableOptionsWithCustomHashSchema)
+    insertRows(table, 100)
+
+    // Run and validate initial backup.
+    backupAndValidateTable(tableName, 100, false)
+
+    // Insert rows then run and validate an incremental backup.
+    insertRows(table, 100, 100)
+    backupAndValidateTable(tableName, 100, true)
+
+    // Restore the table and check the row count.
+    restoreAndValidateTable(tableName, 200)
+
+    // Check the range bounds and the hash schema of each range of the restored table.
+    val restoredTable = kuduClient.openTable(s"$tableName-restore")
+    assertEquals(
+      "[0 <= VALUES < 100 HASH(key) PARTITIONS 2, " +
+        "100 <= VALUES < 200 HASH(key) PARTITIONS 3]",
+      restoredTable.getFormattedRangePartitionsWithHashSchema(10000).toString
+    )
+  }
+
+  @Test
+  def testTableWithTableAndCustomHashSchemas(): Unit = {
+    // Create the initial table and load it with data.
+    val tableName = "testTableWithTableAndCustomHashSchemas"
+    val table = kuduClient.createTable(tableName, schema, tableOptionsWithTableAndCustomHashSchema)
+    insertRows(table, 100)
+
+    // Run and validate initial backup.
+    backupAndValidateTable(tableName, 100, false)
+
+    // Insert rows then run and validate an incremental backup.
+    insertRows(table, 200, 100)
+    backupAndValidateTable(tableName, 200, true)
+
+    // Restore the table and check the row count.
+    restoreAndValidateTable(tableName, 300)
+
+    // Check the range bounds and the hash schema of each range of the restored table.
+    val restoredTable = kuduClient.openTable(s"$tableName-restore")
+    assertEquals(
+      "[0 <= VALUES < 100 HASH(key) PARTITIONS 2, " +
+        "100 <= VALUES < 200 HASH(key) PARTITIONS 3, " +
+        "200 <= VALUES < 300 HASH(key) PARTITIONS 4]",
+      restoredTable.getFormattedRangePartitionsWithHashSchema(10000).toString
+    )
+  }
+
+  @Test
+  def testTableAlterWithTableAndCustomHashSchemas(): Unit = {
+    // Create the initial table and load it with data.
+    val tableName = "testTableAlterWithTableAndCustomHashSchemas"
+    var table = kuduClient.createTable(tableName, schema, tableOptionsWithTableAndCustomHashSchema)
+    insertRows(table, 100)
+
+    // Run and validate initial backup.
+    backupAndValidateTable(tableName, 100, false)
+
+    // Insert rows then run and validate an incremental backup.
+    insertRows(table, 200, 100)
+    backupAndValidateTable(tableName, 200, true)
+
+    // Drops range partition with table wide hash schema and re-adds same range partition with
+    // custom hash schema, also adds another range partition with custom hash schema through alter.
+    val twoHundred = createPartitionRow(200)
+    val threeHundred = createPartitionRow(300)
+    val fourHundred = createPartitionRow(400)
+    val newPartition = new RangePartitionWithCustomHashSchema(
+      twoHundred,
+      threeHundred,
+      RangePartitionBound.INCLUSIVE_BOUND,
+      RangePartitionBound.EXCLUSIVE_BOUND)
+    newPartition.addHashPartitions(List("key").asJava, 5, 0)
+    val newPartition1 = new RangePartitionWithCustomHashSchema(
+      threeHundred,
+      fourHundred,
+      RangePartitionBound.INCLUSIVE_BOUND,
+      RangePartitionBound.EXCLUSIVE_BOUND)
+    newPartition1.addHashPartitions(List("key").asJava, 6, 0)
+    kuduClient.alterTable(
+      tableName,
+      new AlterTableOptions()
+        .dropRangePartition(twoHundred, threeHundred)
+        .addRangePartition(newPartition)
+        .addRangePartition(newPartition1))
+
+    // TODO: Avoid this table refresh by updating partition schema after alter table calls.
+    // See https://issues.apache.org/jira/browse/KUDU-3388 for more details.
+    table = kuduClient.openTable(tableName)
+
+    // Insert rows then run and validate an incremental backup.
+    insertRows(table, 100, 300)
+    backupAndValidateTable(tableName, 100, true)
+
+    // Restore the table and validate.
+    assertTrue(runRestore(createRestoreOptions(Seq(tableName))))
+
+    // Check the range bounds and the hash schema of each range of the restored table.
+    val restoredTable = kuduClient.openTable(s"$tableName-restore")
+    assertEquals(
+      "[0 <= VALUES < 100 HASH(key) PARTITIONS 2, " +
+        "100 <= VALUES < 200 HASH(key) PARTITIONS 3, " +
+        "200 <= VALUES < 300 HASH(key) PARTITIONS 5, " +
+        "300 <= VALUES < 400 HASH(key) PARTITIONS 6]",
+      restoredTable.getFormattedRangePartitionsWithHashSchema(10000).toString
+    )
+  }
+
   @Test
   def testTableNameChangeFlags() {
     // Create four tables and load data
@@ -893,6 +1063,21 @@ class TestKuduBackup extends KuduTestSuite {
     val hashBucketsMatch = (0 until beforeBuckets.size).forall { i =>
       HashBucketSchemasMatch(beforeBuckets(i), afterBuckets(i))
     }
+    val beforeRangeHashSchemas = before.getRangesWithHashSchemas.asScala
+    val afterRangeHashSchemas = after.getRangesWithHashSchemas.asScala
+    if (beforeRangeHashSchemas.size != afterRangeHashSchemas.size) return false
+    for (i <- 0 until beforeRangeHashSchemas.size) {
+      val beforeHashSchemas = beforeRangeHashSchemas(i).hashSchemas.asScala
+      val afterHashSchemas = afterRangeHashSchemas(i).hashSchemas.asScala
+      if (beforeHashSchemas.size != afterHashSchemas.size) return false
+      for (j <- 0 until beforeHashSchemas.size) {
+        if (!HashBucketSchemasMatch(beforeHashSchemas(j), afterHashSchemas(j))) return false
+      }
+      if (!Objects.equal(beforeRangeHashSchemas(i).lowerBound, afterRangeHashSchemas(i).lowerBound)
+        || !Objects
+          .equal(beforeRangeHashSchemas(i).upperBound, afterRangeHashSchemas(i).upperBound))
+        return false
+    }
     hashBucketsMatch &&
     Objects.equal(before.getRangeSchema.getColumnIds, after.getRangeSchema.getColumnIds)
   }
