@@ -284,6 +284,12 @@ public final class AsyncKuduScanner {
 
   final long scanRequestTimeout;
 
+  /**
+   * UUID of the tserver which the scanner is bound with. The following scans of
+   * this scanner will be sent to the tserver.
+   */
+  private String tsUUID;
+
   /**
    * The prefetching result is cached in memory. This atomic reference is used to avoid
    * two concurrent prefetchings occur and the latest one overrides the previous one.
@@ -370,6 +376,8 @@ public final class AsyncKuduScanner {
         columns.add(getStrippedColumnSchema(originalColumn));
       }
     } else {
+      // By default, a scanner is created with all columns including auto-incrementing
+      // column if projected columns are not specified.
       columns.addAll(table.getSchema().getColumns());
     }
     // This is a diff scan so add the IS_DELETED column.
@@ -539,6 +547,10 @@ public final class AsyncKuduScanner {
     this.rowDataFormat = rowDataFormat;
   }
 
+  public String getTsUUID() {
+    return tsUUID;
+  }
+
   /**
    * Scans a number of rows.
    * <p>
@@ -597,12 +609,14 @@ public final class AsyncKuduScanner {
           }
 
           if (!resp.more || resp.scannerId == null) {
+            tsUUID = resp.data.getTsUUID();
             scanFinished();
             return Deferred.fromResult(resp.data); // there might be data to return
           }
           scannerId = resp.scannerId;
           sequenceId++;
           canRequestMore = resp.more;
+          tsUUID = resp.data.getTsUUID();
           if (LOG.isDebugEnabled()) {
             LOG.debug("Scanner {} opened on {}", Bytes.pretty(scannerId), tablet);
           }
@@ -689,7 +703,31 @@ public final class AsyncKuduScanner {
       new Callback<RowResultIterator, Response>() {
         @Override
         public RowResultIterator call(final Response resp) {
+          long lastPropagatedTimestamp = AsyncKuduClient.NO_TIMESTAMP;
+          if (readMode == ReadMode.READ_YOUR_WRITES &&
+              resp.scanTimestamp != AsyncKuduClient.NO_TIMESTAMP) {
+            // For READ_YOUR_WRITES mode, update the latest propagated timestamp
+            // with the chosen snapshot timestamp sent back from the server, to
+            // avoid unnecessarily wait for subsequent reads. Since as long as
+            // the chosen snapshot timestamp of the next read is greater than
+            // the previous one, the scan does not violate READ_YOUR_WRITES
+            // session guarantees.
+            lastPropagatedTimestamp = resp.scanTimestamp;
+          } else if (resp.propagatedTimestamp != AsyncKuduClient.NO_TIMESTAMP) {
+            // Otherwise we just use the propagated timestamp returned from
+            // the server as the latest propagated timestamp.
+            lastPropagatedTimestamp = resp.propagatedTimestamp;
+          }
+          if (lastPropagatedTimestamp != AsyncKuduClient.NO_TIMESTAMP) {
+            client.updateLastPropagatedTimestamp(lastPropagatedTimestamp);
+          }
           numRowsReturned += resp.data.getNumRows();
+          if (isFaultTolerant && resp.lastPrimaryKey != null) {
+            lastPrimaryKey = resp.lastPrimaryKey;
+          }
+          if (resp.resourceMetricsPb != null) {
+            resourceMetrics.update(resp.resourceMetricsPb);
+          }
           if (!resp.more) {  // We're done scanning this tablet.
             scanFinished();
             return resp.data;
@@ -818,6 +856,11 @@ public final class AsyncKuduScanner {
     } else {
       buf.append(", endPrimaryKey=<end>");
     }
+    if (lastPrimaryKey.length > 0) {
+      buf.append(", lastPrimaryKey=").append(Bytes.hex(lastPrimaryKey));
+    } else {
+      buf.append(", lastPrimaryKey=<last>");
+    }
     buf.append(')');
     return buf.toString();
   }
