@@ -17,11 +17,13 @@
 
 package org.apache.kudu.client;
 
+import java.security.cert.CertificateException;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.concurrent.Executor;
 
 import com.google.common.base.Preconditions;
+import com.google.protobuf.ByteString;
 import com.stumbleupon.async.Deferred;
 import org.apache.yetus.audience.InterfaceAudience;
 import org.apache.yetus.audience.InterfaceStability;
@@ -163,6 +165,9 @@ public class KuduClient implements AutoCloseable {
 
   /**
    * Delete a table on the cluster with the specified name.
+   * The deleted table may turn to soft-deleted status with the flag
+   * default_deleted_table_reserve_seconds set to nonzero on the master side.
+   *
    * @param name the table's name
    * @return an rpc response object
    * @throws KuduException if anything went wrong
@@ -172,6 +177,46 @@ public class KuduClient implements AutoCloseable {
     return joinAndHandleException(d);
   }
 
+  /**
+   * SoftDelete a table on the cluster with the specified name, the table will be
+   * reserved for reserveSeconds before being purged.
+   * @param name the table's name
+   * @param reserveSeconds the soft deleted table to be alive time
+   * @return an rpc response object
+   * @throws KuduException if anything went wrong
+   */
+  public DeleteTableResponse deleteTable(String name,
+                                         int reserveSeconds) throws KuduException {
+    Deferred<DeleteTableResponse> d = asyncClient.deleteTable(name, reserveSeconds);
+    return joinAndHandleException(d);
+  }
+
+  /**
+   * Recall a deleted table on the cluster with the specified table id
+   * @param id the table's id
+   * @return an rpc response object
+   * @throws KuduException if anything went wrong
+   */
+  public RecallDeletedTableResponse recallDeletedTable(String id) throws KuduException {
+    Deferred<RecallDeletedTableResponse> d = asyncClient.recallDeletedTable(id);
+    return joinAndHandleException(d);
+  }
+
+  /**
+   * Recall a deleted table on the cluster with the specified table id
+   * and give the recalled table the new table name
+   * @param id the table's id
+   * @param newTableName the recalled table's new name
+   * @return an rpc response object
+   * @throws KuduException if anything went wrong
+   */
+  public RecallDeletedTableResponse recallDeletedTable(String id,
+                                                       String newTableName) throws KuduException {
+    Deferred<RecallDeletedTableResponse> d = asyncClient.recallDeletedTable(id,
+                                                         newTableName);
+    return joinAndHandleException(d);
+  }
+
   /**
    * Alter a table on the cluster as specified by the builder.
    * @param name the table's name (old name if the table is being renamed)
@@ -219,8 +264,8 @@ public class KuduClient implements AutoCloseable {
   }
 
   /**
-   * Get the list of all the tables.
-   * @return a list of all the tables
+   * Get the list of all the regular tables.
+   * @return a list of all the regular tables
    * @throws KuduException if anything went wrong
    */
   public ListTablesResponse getTablesList() throws KuduException {
@@ -228,14 +273,35 @@ public class KuduClient implements AutoCloseable {
   }
 
   /**
-   * Get a list of table names. Passing a null filter returns all the tables. When a filter is
-   * specified, it only returns tables that satisfy a substring match.
+   * Get a list of regular table names. Passing a null filter returns all the tables.
+   * When a filter is specified, it only returns tables that satisfy a substring match.
    * @param nameFilter an optional table name filter
    * @return a deferred that contains the list of table names
    * @throws KuduException if anything went wrong
    */
   public ListTablesResponse getTablesList(String nameFilter) throws KuduException {
-    Deferred<ListTablesResponse> d = asyncClient.getTablesList(nameFilter);
+    Deferred<ListTablesResponse> d = asyncClient.getTablesList(nameFilter, false);
+    return joinAndHandleException(d);
+  }
+
+  /**
+   * Get the list of all the soft deleted tables.
+   * @return a list of all the soft deleted tables
+   * @throws KuduException if anything went wrong
+   */
+  public ListTablesResponse getSoftDeletedTablesList() throws KuduException {
+    return getSoftDeletedTablesList(null);
+  }
+
+  /**
+   * Get list of soft deleted table names. Passing a null filter returns all the tables.
+   * When a filter is specified, it only returns tables that satisfy a substring match.
+   * @param nameFilter an optional table name filter
+   * @return a deferred that contains the list of table names
+   * @throws KuduException if anything went wrong
+   */
+  public ListTablesResponse getSoftDeletedTablesList(String nameFilter) throws KuduException {
+    Deferred<ListTablesResponse> d = asyncClient.getTablesList(nameFilter, true);
     return joinAndHandleException(d);
   }
 
@@ -402,6 +468,33 @@ public class KuduClient implements AutoCloseable {
     asyncClient.importAuthenticationCredentials(authnData);
   }
 
+  /**
+   * Mark the given CA certificates (in DER format) as the trusted ones for the
+   * client. The provided list of certificates replaces any previously set ones.
+   *
+   * @param certificates list of certificates to trust (in DER format)
+   * @throws CertificateException if any of the specified certificates were invalid
+   */
+  @InterfaceStability.Unstable
+  public void trustedCertificates(List<ByteString> certificates) throws CertificateException {
+    asyncClient.trustedCertificates(certificates);
+  }
+
+  /**
+   * Set JWT (JSON Web Token) to authenticate the client to a server.
+   * <p>
+   * @note If {@link #importAuthenticationCredentials(byte[] authnData)} and
+   * this method are called on the same object, the JWT provided with this call
+   * overrides the corresponding JWT that comes as a part of the imported
+   * authentication credentials (if present).
+   *
+   * @param jwt The JSON web token to set.
+   */
+  @InterfaceStability.Unstable
+  public void jwt(String jwt) {
+    asyncClient.jwt(jwt);
+  }
+
   /**
    * Get the timeout used for operations on sessions and scanners.
    * @return a timeout in milliseconds
@@ -427,7 +520,8 @@ public class KuduClient implements AutoCloseable {
   }
 
   /**
-   * Sends a request to the master to check if the cluster supports ignore operations.
+   * Sends a request to the master to check if the cluster supports ignore operations, including
+   * InsertIgnore, UpdateIgnore and DeleteIgnore operations.
    * @return true if the cluster supports ignore operations
    */
   @InterfaceAudience.Private
