@@ -82,6 +82,7 @@ import org.apache.kudu.rpc.RpcHeader.AuthenticationTypePB;
 import org.apache.kudu.rpc.RpcHeader.NegotiatePB;
 import org.apache.kudu.rpc.RpcHeader.NegotiatePB.NegotiateStep;
 import org.apache.kudu.rpc.RpcHeader.RpcFeatureFlag;
+import org.apache.kudu.security.Token.JwtRawPB;
 import org.apache.kudu.security.Token.SignedTokenPB;
 import org.apache.kudu.util.SecurityUtil;
 
@@ -115,29 +116,37 @@ public class Negotiator extends SimpleChannelInboundHandler<CallResponse> {
 
   /**
    * The cipher suites, in order of our preference.
+   *
    * This list is based on the kDefaultTls13Ciphers and kDefaultTlsCiphers lists
    * in security_flags.cc: see that file for details on how it was derived.
+   *
+   * For the mapping between IANA and OpenSSL cipher names, run
+   * `openssl ciphers -stdname` on OpenSSL 1.1.1 (and newer) or see
+   *   https://www.openssl.org/docs/man1.1.1/man1/ciphers.html
+   *   https://wiki.mozilla.org/Security/Cipher_Suites
+   *
+   * For information on TLSv1.3 (JEP 332) and appropriate ciphers in Java 8
+   * updates, see
+   *   https://www.oracle.com/java/technologies/javase/8all-relnotes.html
    */
   static final String[] PREFERRED_CIPHER_SUITES = new String[] {
-      "TLS_AES_128_GCM_SHA256",                   // Java 8 (updated), Java 11 (TLS 1.3+ only)
-      "TLS_AES_256_GCM_SHA384",                   // Java 8 (updated), Java 11 (TLS 1.3+ only)
-      "TLS_CHACHA20_POLY1305_SHA256",             // Java 8 (updated), Java 11 (TLS 1.3+ only)
-      "TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384",  // Java 8
-      "TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384",    // Java 8
-      "TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256",  // Java 8
-      "TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256",    // Java 8
-      "TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA384",  // Java 7 (TLS 1.2+ only)
-      "TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA384",    // Java 7 (TLS 1.2+ only)
-      "TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256",  // Java 7 (TLS 1.2+ only)
-      "TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256",    // Java 7 (TLS 1.2+ only)
-      "TLS_RSA_WITH_AES_256_GCM_SHA384",          // Java 8
-      "TLS_RSA_WITH_AES_128_GCM_SHA256",          // Java 8
-      "TLS_RSA_WITH_AES_256_CBC_SHA256",          // Java 7 (TLS 1.2+ only)
-      "TLS_RSA_WITH_AES_128_CBC_SHA256",          // Java 7 (TLS 1.2+ only)
-      // The following two are critical to allow the client to connect to
-      // servers running versions of OpenSSL that don't support TLS 1.2.
-      "TLS_RSA_WITH_AES_256_CBC_SHA",             // All Java versions
-      "TLS_RSA_WITH_AES_128_CBC_SHA"              // All Java versions
+      "TLS_AES_128_GCM_SHA256",                 // TLSv1.3: Java 8 updates (8u261), Java 11
+      "TLS_AES_256_GCM_SHA384",                 // TLSv1.3: Java 8 updates (8u261), Java 11
+      "TLS_CHACHA20_POLY1305_SHA256",           // TLSv1.3: Java 12
+      "TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256",// TLSv1.2: Java 8
+      "TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256",  // TLSv1.2: Java 8
+      "TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384",// TLSv1.2: Java 8
+      "TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384",  // TLSv1.2: Java 8
+      "TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256",  // TLSv1.2: Java 12
+      "TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256",    // TLSv1.2: Java 12
+      // The following AES CBC ciphers are present to be around when no AES GCM
+      // ciphers are available (that's so for some FIPS 140-2 environments).
+      "TLS_ECDHE_ECDSA_WITH_AES_128_CCM",       // TLSv1.2: custom JSSE providers
+      "TLS_ECDHE_ECDSA_WITH_AES_256_CCM",       // TLSv1.2: custom JSSE providers
+      "TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256",// TLSv1.2: Java 7
+      "TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256",  // TLSv1.2: Java 7
+      "TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA384",// TLSv1.2: Java 7
+      "TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA384",  // TLSv1.2: Java 7
   };
 
   /**
@@ -145,17 +154,16 @@ public class Negotiator extends SimpleChannelInboundHandler<CallResponse> {
    * This list is based on the kDefaultTlsMinVersion in security_flags.cc.
    */
   static final String[] PREFERRED_PROTOCOLS = new String[]{
-      "TLSv1.3",
-      "TLSv1.2",
-      "TLSv1.1",
-      "TLSv1",
+      "TLSv1.3",  // Java 8 updates (8u261), Java 11
+      "TLSv1.2",  // Java 8
   };
 
   private enum State {
     INITIAL,
     AWAIT_NEGOTIATE,
     AWAIT_TLS_HANDSHAKE,
-    AWAIT_TOKEN_EXCHANGE,
+    AWAIT_AUTHN_TOKEN_EXCHANGE,
+    AWAIT_JWT_EXCHANGE,
     AWAIT_SASL,
     FINISHED
   }
@@ -168,8 +176,18 @@ public class Negotiator extends SimpleChannelInboundHandler<CallResponse> {
    * The authentication token we'll try to connect with, maybe null.
    * This is fetched from {@link #securityContext} in the constructor to
    * ensure that it doesn't change over the course of a negotiation attempt.
+   * An authentication token is used as secondary credentials.
    */
   private final SignedTokenPB authnToken;
+  /**
+   * A JSON Web Token (JWT) to authenticate this client/actor to the server
+   * that we'll try to connect with. Similar to {@link #authnToken}, this token
+   * may be null, and it's fetched from {@link #securityContext} as well.
+   * Cannot change over the course of an RPC connection negotiation attempt.
+   *
+   * @note unlike {@link #authnToken}, {@link #jsonWebToken} is used as primary credentials
+   */
+  private final JwtRawPB jsonWebToken;
 
   private enum AuthnTokenNotUsedReason {
     NONE_AVAILABLE("no token is available"),
@@ -246,6 +264,7 @@ public class Negotiator extends SimpleChannelInboundHandler<CallResponse> {
     this.requireAuthentication = requireAuthentication;
     this.requireEncryption = requireEncryption;
     this.encryptLoopback = encryptLoopback;
+
     SignedTokenPB token = securityContext.getAuthenticationToken();
     if (token != null) {
       if (ignoreAuthnToken) {
@@ -261,6 +280,13 @@ public class Negotiator extends SimpleChannelInboundHandler<CallResponse> {
       this.authnToken = null;
       this.authnTokenNotUsedReason = AuthnTokenNotUsedReason.NONE_AVAILABLE;
     }
+
+    JwtRawPB jwt = securityContext.getJsonWebToken();
+    if (jwt != null && securityContext.hasTrustedCerts()) {
+      this.jsonWebToken = jwt;
+    } else {
+      this.jsonWebToken = null;
+    }
   }
 
   public void sendHello(ChannelHandlerContext ctx) {
@@ -293,6 +319,15 @@ public class Negotiator extends SimpleChannelInboundHandler<CallResponse> {
           AuthenticationTypePB.Token.getDefaultInstance());
     }
 
+    // We may also have a JSON Web token, but it can be sent to the server
+    // only if we can verify the server's authenticity and the channel between
+    // this client and the server is confidential, i.e. it's protected by
+    // authenticated TLS.
+    if (jsonWebToken != null) {
+      builder.addAuthnTypesBuilder().setJwt(
+          AuthenticationTypePB.Jwt.getDefaultInstance());
+    }
+
     // We currently don't support client-certificate authentication from the
     // Java client.
 
@@ -334,8 +369,11 @@ public class Negotiator extends SimpleChannelInboundHandler<CallResponse> {
       case AWAIT_SASL:
         handleSaslMessage(ctx, response);
         break;
-      case AWAIT_TOKEN_EXCHANGE:
-        handleTokenExchangeResponse(ctx, response);
+      case AWAIT_AUTHN_TOKEN_EXCHANGE:
+        handleAuthnTokenExchangeResponse(ctx, response);
+        break;
+      case AWAIT_JWT_EXCHANGE:
+        handleJwtExchangeResponse(ctx, response);
         break;
       case AWAIT_TLS_HANDSHAKE:
         handleTlsMessage(ctx, response);
@@ -552,6 +590,12 @@ public class Negotiator extends SimpleChannelInboundHandler<CallResponse> {
               "but client had no valid token");
         }
         break;
+      case JWT:
+        if (jsonWebToken == null) {
+          throw new IllegalArgumentException("server chose JWT authentication " +
+              "but client had no valid JWT");
+        }
+        break;
       default:
         throw new IllegalArgumentException("server chose bad authn type " + chosenAuthnType);
     }
@@ -578,6 +622,7 @@ public class Negotiator extends SimpleChannelInboundHandler<CallResponse> {
         engine = securityContext.createSSLEngineTrustAll();
         break;
       case TOKEN:
+      case JWT:
         engine = securityContext.createSSLEngine();
         break;
       default:
@@ -739,13 +784,16 @@ public class Negotiator extends SimpleChannelInboundHandler<CallResponse> {
       case TOKEN:
         sendTokenExchange(ctx);
         break;
+      case JWT:
+        sendJwtExchange(ctx);
+        break;
       default:
         throw new AssertionError("unreachable");
     }
   }
 
   private void sendTokenExchange(ChannelHandlerContext ctx) {
-    // We must not send a token unless we have successfully finished
+    // We must not send authn token unless we have successfully finished
     // authenticating via TLS.
     Preconditions.checkNotNull(authnToken);
     Preconditions.checkNotNull(sslHandshakeFuture);
@@ -754,16 +802,39 @@ public class Negotiator extends SimpleChannelInboundHandler<CallResponse> {
     RpcHeader.NegotiatePB.Builder builder = RpcHeader.NegotiatePB.newBuilder()
         .setStep(NegotiateStep.TOKEN_EXCHANGE)
         .setAuthnToken(authnToken);
-    state = State.AWAIT_TOKEN_EXCHANGE;
+    state = State.AWAIT_AUTHN_TOKEN_EXCHANGE;
     sendSaslMessage(ctx, builder.build());
   }
 
-  private void handleTokenExchangeResponse(ChannelHandlerContext ctx, NegotiatePB response)
+  private void sendJwtExchange(ChannelHandlerContext ctx) {
+    // We must not send JWT unless we have successfully finished
+    // authenticating via TLS.
+    Preconditions.checkNotNull(jsonWebToken);
+    Preconditions.checkNotNull(sslHandshakeFuture);
+    Preconditions.checkState(sslHandshakeFuture.isSuccess());
+
+    RpcHeader.NegotiatePB.Builder builder = RpcHeader.NegotiatePB.newBuilder()
+        .setStep(NegotiateStep.JWT_EXCHANGE)
+        .setJwtRaw(jsonWebToken);
+    state = State.AWAIT_JWT_EXCHANGE;
+    sendSaslMessage(ctx, builder.build());
+  }
+
+  private void handleAuthnTokenExchangeResponse(ChannelHandlerContext ctx, NegotiatePB response)
       throws SaslException {
     Preconditions.checkArgument(response.getStep() == NegotiateStep.TOKEN_EXCHANGE,
         "expected TOKEN_EXCHANGE, got step: {}", response.getStep());
 
-    // The token response doesn't have any actual data in it, so we can just move on.
+    // The authn token response doesn't have any actual data in it, so we can just move on.
+    finish(ctx);
+  }
+
+  private void handleJwtExchangeResponse(ChannelHandlerContext ctx, NegotiatePB response)
+      throws SaslException {
+    Preconditions.checkArgument(response.getStep() == NegotiateStep.JWT_EXCHANGE,
+        "expected JWT_EXCHANGE, got step: {}", response.getStep());
+
+    // The JWT response doesn't have any actual data in it, so we can just move on.
     finish(ctx);
   }
 
