@@ -29,6 +29,7 @@ import static org.apache.kudu.test.ClientTestUtil.createManyVarcharsSchema;
 import static org.apache.kudu.test.ClientTestUtil.createSchemaWithBinaryColumns;
 import static org.apache.kudu.test.ClientTestUtil.createSchemaWithDateColumns;
 import static org.apache.kudu.test.ClientTestUtil.createSchemaWithDecimalColumns;
+import static org.apache.kudu.test.ClientTestUtil.createSchemaWithNonUniqueKey;
 import static org.apache.kudu.test.ClientTestUtil.createSchemaWithTimestampColumns;
 import static org.apache.kudu.test.ClientTestUtil.getBasicCreateTableOptions;
 import static org.apache.kudu.test.ClientTestUtil.getBasicTableOptionsWithNonCoveredRange;
@@ -41,6 +42,7 @@ import static org.junit.Assert.assertNotEquals;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertThat;
+import static org.junit.Assert.assertThrows;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 import static org.junit.Assume.assumeTrue;
@@ -49,8 +51,10 @@ import java.io.Closeable;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
 import java.math.BigDecimal;
+import java.security.cert.CertificateException;
 import java.sql.Date;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
@@ -64,6 +68,8 @@ import java.util.concurrent.Future;
 import java.util.concurrent.TimeUnit;
 
 import com.google.common.collect.ImmutableList;
+import com.google.common.collect.Lists;
+import com.google.protobuf.ByteString;
 import com.stumbleupon.async.Deferred;
 import org.junit.Before;
 import org.junit.Rule;
@@ -172,6 +178,47 @@ public class TestKuduClient {
                  newSchema.getColumn("column3_s").getCompressionAlgorithm());
   }
 
+  /**
+   * Test recalling a soft deleted table through a KuduClient.
+   */
+  @Test(timeout = 100000)
+  public void testRecallDeletedTable() throws Exception {
+    // Check that we can create a table.
+    assertTrue(client.getTablesList().getTablesList().isEmpty());
+    final KuduTable table = client.createTable(TABLE_NAME, basicSchema,
+        getBasicCreateTableOptions());
+    final String tableId = table.getTableId();
+    assertEquals(1, client.getTablesList().getTablesList().size());
+    assertEquals(TABLE_NAME, client.getTablesList().getTablesList().get(0));
+
+    // Check that we can delete it.
+    client.deleteTable(TABLE_NAME, 600);
+    List<String> tables = client.getTablesList().getTablesList();
+    assertEquals(0, tables.size());
+    tables = client.getSoftDeletedTablesList().getTablesList();
+    assertEquals(1, tables.size());
+    String softDeletedTable = tables.get(0);
+    assertEquals(TABLE_NAME, softDeletedTable);
+    // Check that we can recall the soft_deleted table.
+    client.recallDeletedTable(tableId);
+    assertEquals(1, client.getTablesList().getTablesList().size());
+    assertEquals(TABLE_NAME, client.getTablesList().getTablesList().get(0));
+
+    // Check that we can delete it.
+    client.deleteTable(TABLE_NAME, 600);
+    tables = client.getTablesList().getTablesList();
+    assertEquals(0, tables.size());
+    tables = client.getSoftDeletedTablesList().getTablesList();
+    assertEquals(1, tables.size());
+    softDeletedTable = tables.get(0);
+    assertEquals(TABLE_NAME, softDeletedTable);
+    // Check we can recall soft deleted table with new table name.
+    final String newTableName = "NewTable";
+    client.recallDeletedTable(tableId, newTableName);
+    assertEquals(1, client.getTablesList().getTablesList().size());
+    assertEquals(newTableName, client.getTablesList().getTablesList().get(0));
+  }
+
   /**
    * Test creating a table with various invalid schema cases.
    */
@@ -741,6 +788,371 @@ public class TestKuduClient {
     }
   }
 
+  /**
+   * Test creating a table with non unique primary key in the table schema.
+   */
+  @Test(timeout = 100000)
+  public void testCreateTableWithNonUniquePrimaryKeys() throws Exception {
+    // Create a schema with non unique primary key column
+    Schema schema = createSchemaWithNonUniqueKey();
+    assertFalse(schema.isPrimaryKeyUnique());
+    // Verify auto-incrementing column is in the schema
+    assertTrue(schema.hasAutoIncrementingColumn());
+    assertEquals(3, schema.getColumnCount());
+    assertEquals(2, schema.getPrimaryKeyColumnCount());
+    assertEquals(1, schema.getColumnIndex(Schema.getAutoIncrementingColumnName()));
+    // Create a table
+    client.createTable(TABLE_NAME, schema, getBasicCreateTableOptions());
+
+    KuduSession session = client.newSession();
+    KuduTable table = client.openTable(TABLE_NAME);
+
+    // Verify that the primary key is not unique, and an auto-incrementing column is
+    // added as key column in the position after all key columns.
+    schema = table.getSchema();
+    assertFalse(schema.isPrimaryKeyUnique());
+    assertTrue(schema.hasAutoIncrementingColumn());
+    assertEquals(3, schema.getColumnCount());
+    assertEquals(2, schema.getPrimaryKeyColumnCount());
+    assertEquals(1, schema.getColumnIndex(Schema.getAutoIncrementingColumnName()));
+    assertTrue(schema.getColumn(Schema.getAutoIncrementingColumnName()).isKey());
+    assertTrue(schema.getColumn(
+        Schema.getAutoIncrementingColumnName()).isAutoIncrementing());
+
+    // Insert rows into the table without assigning values for the auto-incrementing
+    // column.
+    for (int i = 0; i < 3; i++) {
+      Insert insert = table.newInsert();
+      PartialRow row = insert.getRow();
+      row.addInt("key", i);
+      row.addInt("c1", i * 10);
+      session.apply(insert);
+    }
+    session.flush();
+
+    // Scan all the rows in the table with all columns.
+    // Verify that the auto-incrementing column is included in the rows.
+    List<String> rowStrings = scanTableToStrings(table);
+    assertEquals(3, rowStrings.size());
+    for (int i = 0; i < rowStrings.size(); i++) {
+      StringBuilder expectedRow = new StringBuilder();
+      expectedRow.append(String.format("INT32 key=%d, INT64 %s=%d, INT32 c1=%d",
+          i, Schema.getAutoIncrementingColumnName(), i + 1, i * 10));
+      assertEquals(expectedRow.toString(), rowStrings.get(i));
+    }
+
+    // Update "c1" column of the first row with "key" and auto-incrementing columns.
+    Update update = table.newUpdate();
+    PartialRow row = update.getRow();
+    row.addInt(schema.getColumnByIndex(0).getName(), 0);
+    row.addLong(schema.getColumnByIndex(1).getName(), 1);
+    row.addInt(schema.getColumnByIndex(2).getName(), 100);
+    session.apply(update);
+    session.flush();
+
+    // Scan all the rows in the table without the auto-incrementing column.
+    // Verify that "c1" column of the first row is updated.
+    KuduScanner.KuduScannerBuilder scanBuilder = client.newScannerBuilder(table);
+    KuduScanner scanner =
+        scanBuilder.setProjectedColumnNames(Lists.newArrayList("key", "c1")).build();
+    rowStrings.clear();
+    for (RowResult r : scanner) {
+      rowStrings.add(r.rowToString());
+    }
+    Collections.sort(rowStrings);
+    assertEquals(3, rowStrings.size());
+    for (int i = 0; i < rowStrings.size(); i++) {
+      StringBuilder expectedRow = new StringBuilder();
+      if (i == 0) {
+        expectedRow.append(String.format("INT32 key=0, INT32 c1=100"));
+      } else {
+        expectedRow.append(String.format("INT32 key=%d, INT32 c1=%d", i, i * 10));
+      }
+      assertEquals(expectedRow.toString(), rowStrings.get(i));
+    }
+
+    // Upsert rows into the table after assigning values for the auto-incrementing
+    // column. The first three rows will be applied as updates and the next three as
+    // inserts.
+    for (int i = 0; i < 6; i++) {
+      Upsert upsert = table.newUpsert();
+      row = upsert.getRow();
+      row.addInt("key", i);
+      row.addLong(Schema.getAutoIncrementingColumnName(), i + 1);
+      row.addInt("c1", i * 20);
+      session.apply(upsert);
+    }
+    session.flush();
+
+    // Scan all the rows in the table with all columns.
+    // Verify that the auto-incrementing column is included in the rows.
+    rowStrings = scanTableToStrings(table);
+    assertEquals(6, rowStrings.size());
+    for (int i = 0; i < rowStrings.size(); i++) {
+      String expectedRow = String.format("INT32 key=%d, INT64 %s=%d, INT32 c1=%d",
+              i, Schema.getAutoIncrementingColumnName(), i + 1, i * 20);
+      assertEquals(expectedRow, rowStrings.get(i));
+    }
+
+    // Delete the first row with "key" and auto-incrementing columns.
+    // Verify that number of rows is decreased by 1.
+    Delete delete = table.newDelete();
+    row = delete.getRow();
+    row.addInt(schema.getColumnByIndex(0).getName(), 0);
+    row.addLong(schema.getColumnByIndex(1).getName(), 1);
+    session.apply(delete);
+    session.flush();
+    assertEquals(5, countRowsInScan(client.newScannerBuilder(table).build()));
+
+    // Check that we can delete the table.
+    client.deleteTable(TABLE_NAME);
+  }
+
+  /**
+   * Test operations for table with auto-incrementing column.
+   */
+  @Test(timeout = 100000)
+  public void testTableWithAutoIncrementingColumn() throws Exception {
+    // Create a schema with non unique primary key column
+    Schema schema = createSchemaWithNonUniqueKey();
+    assertFalse(schema.isPrimaryKeyUnique());
+    // Verify auto-incrementing column is in the schema
+    assertTrue(schema.hasAutoIncrementingColumn());
+    assertEquals(3, schema.getColumnCount());
+    assertEquals(2, schema.getPrimaryKeyColumnCount());
+    // Create a table
+    client.createTable(TABLE_NAME, schema, getBasicCreateTableOptions());
+
+    final KuduSession session = client.newSession();
+    KuduTable table = client.openTable(TABLE_NAME);
+    schema = table.getSchema();
+    assertTrue(schema.hasAutoIncrementingColumn());
+
+    // Verify that UPSERT is allowed for table with auto-incrementing column
+    Upsert upsert = table.newUpsert();
+    PartialRow rowUpsert = upsert.getRow();
+    rowUpsert.addInt("key", 0);
+    rowUpsert.addLong(Schema.getAutoIncrementingColumnName(), 1);
+    rowUpsert.addInt("c1", 10);
+    session.apply(upsert);
+
+    // Verify that UPSERT_IGNORE is allowed for table with auto-incrementing column
+    UpsertIgnore upsertIgnore = table.newUpsertIgnore();
+    PartialRow rowUpsertIgnore = upsertIgnore.getRow();
+    rowUpsertIgnore.addInt("key", 1);
+    rowUpsertIgnore.addLong(Schema.getAutoIncrementingColumnName(), 2);
+    rowUpsertIgnore.addInt("c1", 20);
+    session.apply(upsertIgnore);
+
+    // Change desired block size for auto-incrementing column
+    client.alterTable(TABLE_NAME, new AlterTableOptions().changeDesiredBlockSize(
+        Schema.getAutoIncrementingColumnName(), 1));
+    // Change encoding for auto-incrementing column
+    client.alterTable(TABLE_NAME, new AlterTableOptions().changeEncoding(
+        Schema.getAutoIncrementingColumnName(), ColumnSchema.Encoding.PLAIN_ENCODING));
+    // Change compression algorithm for auto-incrementing column
+    client.alterTable(TABLE_NAME, new AlterTableOptions().changeCompressionAlgorithm(
+        Schema.getAutoIncrementingColumnName(), ColumnSchema.CompressionAlgorithm.NO_COMPRESSION));
+    session.flush();
+
+    // Verify that auto-incrementing column value cannot be specified in an INSERT operation.
+    try {
+      Insert insert = table.newInsert();
+      PartialRow row = insert.getRow();
+      row.addInt("key", 1);
+      row.addLong(Schema.getAutoIncrementingColumnName(), 1);
+      row.addInt("c1", 10);
+      session.apply(insert);
+      fail("INSERT on table with auto-incrementing column set");
+    } catch (KuduException ex) {
+      assertTrue(ex.getMessage().contains("Auto-Incrementing column should not " +
+          "be specified for INSERT operation"));
+    }
+
+    // Verify that auto-incrementing column value cannot be specified in an INSERT_IGNORE operation.
+    try {
+      InsertIgnore insertIgnore = table.newInsertIgnore();
+      PartialRow row = insertIgnore.getRow();
+      row.addInt("key", 1);
+      row.addLong(Schema.getAutoIncrementingColumnName(), 1);
+      row.addInt("c1", 10);
+      session.apply(insertIgnore);
+      fail("INSERT on table with auto-incrementing column set");
+    } catch (KuduException ex) {
+      assertTrue(ex.getMessage().contains("Auto-Incrementing column should not " +
+          "be specified for INSERT operation"));
+    }
+    // Verify that auto-incrementing column cannot be added
+    try {
+      client.alterTable(TABLE_NAME, new AlterTableOptions().addColumn(
+          Schema.getAutoIncrementingColumnName(), Schema.getAutoIncrementingColumnType(), 0));
+      fail("Add auto-incrementing column");
+    } catch (IllegalArgumentException e) {
+      assertTrue(e.getMessage().contains("Column name " +
+          Schema.getAutoIncrementingColumnName() + " is reserved by Kudu engine"));
+    }
+    try {
+      client.alterTable(TABLE_NAME, new AlterTableOptions().addColumn(
+          new ColumnSchema.AutoIncrementingColumnSchemaBuilder().build()));
+      fail("Add auto-incrementing column");
+    } catch (IllegalArgumentException e) {
+      assertTrue(e.getMessage().contains("Column name " +
+          Schema.getAutoIncrementingColumnName() + " is reserved by Kudu engine"));
+    }
+
+    // Verify that auto-incrementing column cannot be removed
+    try {
+      client.alterTable(TABLE_NAME, new AlterTableOptions().dropColumn(
+          Schema.getAutoIncrementingColumnName()));
+      fail("Drop auto-incrementing column");
+    } catch (IllegalArgumentException e) {
+      assertTrue(e.getMessage().contains("Cannot remove auto-incrementing column " +
+          Schema.getAutoIncrementingColumnName()));
+    }
+
+    // Verify that auto-incrementing column cannot be renamed
+    try {
+      client.alterTable(TABLE_NAME, new AlterTableOptions().renameColumn(
+          Schema.getAutoIncrementingColumnName(), "new_auto_incrementing"));
+      fail("Rename auto-incrementing column");
+    } catch (IllegalArgumentException e) {
+      assertTrue(e.getMessage().contains("Cannot rename auto-incrementing column " +
+          Schema.getAutoIncrementingColumnName()));
+    }
+
+    // Verify that auto-incrementing column cannot be changed by removing default
+    try {
+      client.alterTable(TABLE_NAME, new AlterTableOptions().removeDefault(
+          Schema.getAutoIncrementingColumnName()));
+      fail("Remove default value for auto-incrementing column");
+    } catch (IllegalArgumentException e) {
+      assertTrue(e.getMessage().contains("Auto-incrementing column " +
+          Schema.getAutoIncrementingColumnName() + " does not have default value"));
+    }
+
+    // Verify that auto-incrementing column cannot be changed with default value
+    try {
+      client.alterTable(TABLE_NAME, new AlterTableOptions().changeDefault(
+          Schema.getAutoIncrementingColumnName(), 0));
+      fail("Change default value for auto-incrementing column");
+    } catch (IllegalArgumentException e) {
+      assertTrue(e.getMessage().contains("Cannot set default value for " +
+          "auto-incrementing column " + Schema.getAutoIncrementingColumnName()));
+    }
+
+    // Verify that auto-incrementing column cannot be changed for its immutable
+    try {
+      client.alterTable(TABLE_NAME, new AlterTableOptions().changeImmutable(
+          Schema.getAutoIncrementingColumnName(), true));
+      fail("Change immutable for auto-incrementing column");
+    } catch (IllegalArgumentException e) {
+      assertTrue(e.getMessage().contains("Cannot change immutable for " +
+          "auto-incrementing column " + Schema.getAutoIncrementingColumnName()));
+    }
+
+    client.deleteTable(TABLE_NAME);
+  }
+
+  /**
+   * Test inserting and retrieving rows from a table that has a range partition
+   * with custom hash schema.
+   */
+  @Test(timeout = 100000)
+  public void testRangeWithCustomHashSchema() throws Exception {
+    List<ColumnSchema> cols = new ArrayList<>();
+    cols.add(new ColumnSchema.ColumnSchemaBuilder("c0", Type.INT64).key(true).build());
+    cols.add(new ColumnSchema.ColumnSchemaBuilder("c1", Type.INT32).nullable(true).build());
+    Schema schema = new Schema(cols);
+
+    CreateTableOptions options = new CreateTableOptions();
+    options.setRangePartitionColumns(ImmutableList.of("c0"));
+    options.addHashPartitions(ImmutableList.of("c0"), 2);
+
+    // Add range partition with table-wide hash schema.
+    {
+      PartialRow lower = schema.newPartialRow();
+      lower.addLong("c0", -100);
+      PartialRow upper = schema.newPartialRow();
+      upper.addLong("c0", 100);
+      options.addRangePartition(lower, upper);
+    }
+
+    // Add a partition with custom hash schema.
+    {
+      PartialRow lower = schema.newPartialRow();
+      lower.addLong("c0", 100);
+      PartialRow upper = schema.newPartialRow();
+      upper.addLong("c0", 200);
+
+      RangePartitionWithCustomHashSchema rangePartition =
+          new RangePartitionWithCustomHashSchema(
+              lower,
+              upper,
+              RangePartitionBound.INCLUSIVE_BOUND,
+              RangePartitionBound.EXCLUSIVE_BOUND);
+      rangePartition.addHashPartitions(ImmutableList.of("c0"), 5, 0);
+      options.addRangePartition(rangePartition);
+    }
+
+    client.createTable(TABLE_NAME, schema, options);
+
+    KuduSession session = client.newSession();
+    session.setFlushMode(SessionConfiguration.FlushMode.AUTO_FLUSH_SYNC);
+    KuduTable table = client.openTable(TABLE_NAME);
+
+    // Check the range with the table-wide hash schema.
+    {
+      for (int i = 0; i < 10; ++i) {
+        Insert insert = table.newInsert();
+        PartialRow row = insert.getRow();
+        row.addLong("c0", i);
+        row.addInt("c1", 1000 * i);
+        session.apply(insert);
+      }
+
+      // Scan all the rows in the table.
+      List<String> rowStringsAll = scanTableToStrings(table);
+      assertEquals(10, rowStringsAll.size());
+
+      // Now scan the rows that are in the range with the table-wide hash schema.
+      List<String> rowStrings = scanTableToStrings(table,
+          KuduPredicate.newComparisonPredicate(schema.getColumn("c0"), GREATER_EQUAL, 0),
+          KuduPredicate.newComparisonPredicate(schema.getColumn("c0"), LESS, 100));
+      assertEquals(10, rowStrings.size());
+      for (int i = 0; i < rowStrings.size(); ++i) {
+        StringBuilder expectedRow = new StringBuilder();
+        expectedRow.append(String.format("INT64 c0=%d, INT32 c1=%d", i, 1000 * i));
+        assertEquals(expectedRow.toString(), rowStrings.get(i));
+      }
+    }
+
+    // Check the range with the custom hash schema.
+    {
+      for (int i = 100; i < 110; ++i) {
+        Insert insert = table.newInsert();
+        PartialRow row = insert.getRow();
+        row.addLong("c0", i);
+        row.addInt("c1", 2 * i);
+        session.apply(insert);
+      }
+
+      // Scan all the rows in the table.
+      List<String> rowStringsAll = scanTableToStrings(table);
+      assertEquals(20, rowStringsAll.size());
+
+      // Now scan the rows that are in the range with the custom hash schema.
+      List<String> rowStrings = scanTableToStrings(table,
+          KuduPredicate.newComparisonPredicate(schema.getColumn("c0"), GREATER_EQUAL, 100));
+      assertEquals(10, rowStrings.size());
+      for (int i = 0; i < rowStrings.size(); ++i) {
+        StringBuilder expectedRow = new StringBuilder();
+        expectedRow.append(String.format("INT64 c0=%d, INT32 c1=%d",
+            i + 100, 2 * (i + 100)));
+        assertEquals(expectedRow.toString(), rowStrings.get(i));
+      }
+    }
+  }
+
   /**
   * Test scanning with limits.
   */
@@ -1551,4 +1963,14 @@ public class TestKuduClient {
       AsyncKuduSession.injectLatencyBufferFlushCb(false);
     }
   }
+
+  @Test(timeout = 50000)
+  public void testImportInvalidCert() throws Exception {
+    // An empty certificate to import.
+    byte[] caCert = new byte[0];
+    CertificateException e = assertThrows(CertificateException.class, () -> {
+      client.trustedCertificates(Arrays.asList(ByteString.copyFrom(caCert)));
+    });
+    assertTrue(e.getMessage().contains("Could not parse certificate"));
+  }
 }
