@@ -25,7 +25,7 @@
 // Strings:
 //   raw strings that were written
 // Offsets:  [pointed to by offsets_pos]
-//   gvint-encoded offsets pointing to the beginning of each string
+//   gvint-encoded offsets pointing to the beginning of each string.
 #ifndef KUDU_CFILE_BINARY_PLAIN_BLOCK_H
 #define KUDU_CFILE_BINARY_PLAIN_BLOCK_H
 
@@ -93,7 +93,7 @@ class BinaryPlainBlockBuilder final : public BlockBuilder {
   size_t end_of_data_offset_;
   size_t size_estimate_;
 
-  // Offsets of each entry, relative to the start of the block
+  // Offsets of each entry, relative to the start of the block.
   std::vector<uint32_t> offsets_;
 
   bool finished_;
@@ -105,8 +105,23 @@ class BinaryPlainBlockBuilder final : public BlockBuilder {
 class BinaryPlainBlockDecoder final : public BlockDecoder {
  public:
   explicit BinaryPlainBlockDecoder(scoped_refptr<BlockHandle> block);
+  BinaryPlainBlockDecoder(BinaryPlainBlockDecoder&& other) noexcept {
+    *this = std::move(other);
+  }
   virtual ~BinaryPlainBlockDecoder();
 
+  BinaryPlainBlockDecoder& operator=(BinaryPlainBlockDecoder&& other) noexcept {
+    block_ = std::move(other.block_);
+    data_ = other.data_;
+    parsed_ = other.parsed_;
+    offsets_buf_ = std::move(other.offsets_buf_);
+    num_elems_ = other.num_elems_;
+    ordinal_pos_base_ = other.ordinal_pos_base_;
+    cur_idx_ = other.cur_idx_;
+
+    return *this;
+  }
+
   virtual Status ParseHeader() OVERRIDE;
   virtual void SeekToPositionInBlock(uint pos) OVERRIDE;
   virtual Status SeekAtOrAfterValue(const void *value,
