@@ -20,20 +20,21 @@
 #include <memory>
 #include <unordered_map>
 #include <utility>
-#include <vector>
 
-#include "kudu/cfile/binary_dict_block.h"
-#include "kudu/cfile/binary_plain_block.h"
-#include "kudu/cfile/binary_prefix_block.h"
-#include "kudu/cfile/bshuf_block.h"
-#include "kudu/cfile/plain_bitmap_block.h"
-#include "kudu/cfile/plain_block.h"
-#include "kudu/cfile/rle_block.h"
+#include "kudu/cfile/binary_dict_block.h" // IWYU pragma: keep
+#include "kudu/cfile/binary_plain_block.h" // IWYU pragma: keep
+#include "kudu/cfile/binary_prefix_block.h" // IWYU pragma: keep
+#include "kudu/cfile/block_encodings.h"
+#include "kudu/cfile/block_handle.h"
+#include "kudu/cfile/bshuf_block.h" // IWYU pragma: keep
+#include "kudu/cfile/plain_bitmap_block.h" // IWYU pragma: keep
+#include "kudu/cfile/plain_block.h" // IWYU pragma: keep
+#include "kudu/cfile/rle_block.h" // IWYU pragma: keep
 #include "kudu/common/types.h"
 #include "kudu/gutil/port.h"
+#include "kudu/gutil/ref_counted.h"
 #include "kudu/gutil/singleton.h"
 #include "kudu/gutil/strings/substitute.h"
-#include "kudu/util/slice.h"
 
 using std::make_pair;
 using std::pair;
@@ -43,6 +44,25 @@ using std::unordered_map;
 namespace kudu {
 namespace cfile {
 
+// Base template class to help instantiate classes with specific BlockBuilder and BlockDecoder
+// classes.
+template<class Builder, class Decoder>
+struct EncodingTraits {
+  static Status CreateBlockBuilder(unique_ptr<BlockBuilder>* bb, const WriterOptions* options) {
+    bb->reset(new Builder(options));
+    return Status::OK();
+  }
+
+  static Status CreateBlockDecoder(unique_ptr<BlockDecoder>* bd,
+                                   // https://bugs.llvm.org/show_bug.cgi?id=44598
+                                   // NOLINTNEXTLINE(performance-unnecessary-value-param)
+                                   scoped_refptr<BlockHandle> block,
+                                   CFileIterator* /*parent_cfile_iter*/) {
+    bd->reset(new Decoder(std::move(block)));
+    return Status::OK();
+  }
+};
+
 template<DataType Type, EncodingType Encoding>
 struct DataTypeEncodingTraits {};
 
@@ -56,151 +76,68 @@ template<DataType Type, EncodingType Encoding> struct TypeEncodingTraits
 // Generic, fallback, partial specialization that should work for all
 // fixed size types.
 template<DataType Type>
-struct DataTypeEncodingTraits<Type, PLAIN_ENCODING> {
-
-  static Status CreateBlockBuilder(BlockBuilder **bb, const WriterOptions *options) {
-    *bb = new PlainBlockBuilder<Type>(options);
-    return Status::OK();
-  }
-
-  static Status CreateBlockDecoder(BlockDecoder **bd, const Slice &slice,
-                                   CFileIterator *iter) {
-    *bd = new PlainBlockDecoder<Type>(slice);
-    return Status::OK();
-  }
-};
+struct DataTypeEncodingTraits<Type, PLAIN_ENCODING>
+    : public EncodingTraits<PlainBlockBuilder<Type>, PlainBlockDecoder<Type>> {};
 
 // Generic, fallback, partial specialization that should work for all
 // fixed size types.
 template<DataType Type>
-struct DataTypeEncodingTraits<Type, BIT_SHUFFLE> {
-
-  static Status CreateBlockBuilder(BlockBuilder **bb, const WriterOptions *options) {
-    *bb = new BShufBlockBuilder<Type>(options);
-    return Status::OK();
-  }
-
-  static Status CreateBlockDecoder(BlockDecoder **bd, const Slice &slice,
-                                   CFileIterator *iter) {
-    *bd = new BShufBlockDecoder<Type>(slice);
-    return Status::OK();
-  }
-};
+struct DataTypeEncodingTraits<Type, BIT_SHUFFLE>
+    : public EncodingTraits<BShufBlockBuilder<Type>, BShufBlockDecoder<Type>> {};
 
 // Template specialization for plain encoded string as they require a
 // specific encoder/decoder.
 template<>
-struct DataTypeEncodingTraits<BINARY, PLAIN_ENCODING> {
-
-  static Status CreateBlockBuilder(BlockBuilder **bb, const WriterOptions *options) {
-    *bb = new BinaryPlainBlockBuilder(options);
-    return Status::OK();
-  }
-
-  static Status CreateBlockDecoder(BlockDecoder **bd, const Slice &slice,
-                                   CFileIterator *iter) {
-    *bd = new BinaryPlainBlockDecoder(slice);
-    return Status::OK();
-  }
-};
+struct DataTypeEncodingTraits<BINARY, PLAIN_ENCODING>
+    : public EncodingTraits<BinaryPlainBlockBuilder, BinaryPlainBlockDecoder> {};
 
 // Template specialization for packed bitmaps
 template<>
-struct DataTypeEncodingTraits<BOOL, PLAIN_ENCODING> {
-
-  static Status CreateBlockBuilder(BlockBuilder **bb, const WriterOptions *options) {
-    *bb = new PlainBitMapBlockBuilder(options);
-    return Status::OK();
-  }
-
-  static Status CreateBlockDecoder(BlockDecoder **bd, const Slice &slice,
-                                   CFileIterator *iter) {
-    *bd = new PlainBitMapBlockDecoder(slice);
-    return Status::OK();
-  }
-};
-
+struct DataTypeEncodingTraits<BOOL, PLAIN_ENCODING>
+    : public EncodingTraits<PlainBitMapBlockBuilder, PlainBitMapBlockDecoder> {};
 
 // Template specialization for RLE encoded bitmaps
 template<>
-struct DataTypeEncodingTraits<BOOL, RLE> {
-
-  static Status CreateBlockBuilder(BlockBuilder** bb, const WriterOptions *options) {
-    *bb = new RleBitMapBlockBuilder(options);
-    return Status::OK();
-  }
-
-  static Status CreateBlockDecoder(BlockDecoder **bd, const Slice &slice,
-                                   CFileIterator *iter) {
-    *bd = new RleBitMapBlockDecoder(slice);
-    return Status::OK();
-  }
-};
+struct DataTypeEncodingTraits<BOOL, RLE>
+    : public EncodingTraits<RleBitMapBlockBuilder, RleBitMapBlockDecoder> {};
 
 // Template specialization for plain encoded string as they require a
 // specific encoder \/decoder.
 template<>
-struct DataTypeEncodingTraits<BINARY, PREFIX_ENCODING> {
-
-  static Status CreateBlockBuilder(BlockBuilder **bb, const WriterOptions *options) {
-    *bb = new BinaryPrefixBlockBuilder(options);
-    return Status::OK();
-  }
-
-  static Status CreateBlockDecoder(BlockDecoder **bd, const Slice &slice,
-                                   CFileIterator *iter) {
-    *bd = new BinaryPrefixBlockDecoder(slice);
-    return Status::OK();
-  }
-};
+struct DataTypeEncodingTraits<BINARY, PREFIX_ENCODING>
+    : public EncodingTraits<BinaryPrefixBlockBuilder, BinaryPrefixBlockDecoder> {};
 
 // Template for dictionary encoding
 template<>
-struct DataTypeEncodingTraits<BINARY, DICT_ENCODING> {
-
-  static Status CreateBlockBuilder(BlockBuilder **bb, const WriterOptions *options) {
-    *bb = new BinaryDictBlockBuilder(options);
-    return Status::OK();
-  }
-
-  static Status CreateBlockDecoder(BlockDecoder **bd, const Slice &slice,
-                                   CFileIterator *iter) {
-    *bd = new BinaryDictBlockDecoder(slice, iter);
+struct DataTypeEncodingTraits<BINARY, DICT_ENCODING>
+    : public EncodingTraits<BinaryDictBlockBuilder, BinaryDictBlockDecoder> {
+  static Status CreateBlockDecoder(unique_ptr<BlockDecoder>* bd,
+                                   scoped_refptr<BlockHandle> block,
+                                   CFileIterator* parent_cfile_iter) {
+    bd->reset(new BinaryDictBlockDecoder(std::move(block), parent_cfile_iter));
     return Status::OK();
   }
 };
 
 template<DataType IntType>
-struct DataTypeEncodingTraits<IntType, RLE> {
-
-  static Status CreateBlockBuilder(BlockBuilder** bb, const WriterOptions *options) {
-    *bb = new RleIntBlockBuilder<IntType>(options);
-    return Status::OK();
-  }
-
-  static Status CreateBlockDecoder(BlockDecoder** bd, const Slice& slice,
-                                   CFileIterator *iter) {
-    *bd = new RleIntBlockDecoder<IntType>(slice);
-    return Status::OK();
-  }
-};
-
+struct DataTypeEncodingTraits<IntType, RLE>
+    : public EncodingTraits<RleIntBlockBuilder<IntType>, RleIntBlockDecoder<IntType>> {};
 
 template<typename TypeEncodingTraitsClass>
-TypeEncodingInfo::TypeEncodingInfo(TypeEncodingTraitsClass t)
+TypeEncodingInfo::TypeEncodingInfo(TypeEncodingTraitsClass /*t*/)
     : encoding_type_(TypeEncodingTraitsClass::kEncodingType),
       create_builder_func_(TypeEncodingTraitsClass::CreateBlockBuilder),
       create_decoder_func_(TypeEncodingTraitsClass::CreateBlockDecoder) {
 }
 
-Status TypeEncodingInfo::CreateBlockDecoder(BlockDecoder **bd,
-                                            const Slice &slice,
-                                            CFileIterator *iter) const {
-  return create_decoder_func_(bd, slice, iter);
+Status TypeEncodingInfo::CreateBlockDecoder(unique_ptr<BlockDecoder>* bd,
+                                            scoped_refptr<BlockHandle> block,
+                                            CFileIterator* parent_cfile_iter) const {
+  return create_decoder_func_(bd, std::move(block), parent_cfile_iter);
 }
 
 Status TypeEncodingInfo::CreateBlockBuilder(
-    BlockBuilder **bb, const WriterOptions *options) const {
+    unique_ptr<BlockBuilder>* bb, const WriterOptions* options) const {
   return create_builder_func_(bb, options);
 }
 
