@@ -24,12 +24,13 @@
 #include <map>
 #include <memory>
 #include <mutex>
+#include <optional>
 #include <ostream>
 #include <set>
 #include <string>
+#include <type_traits>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <glog/logging.h>
 #include <google/protobuf/stubs/common.h>
 
@@ -50,6 +51,7 @@
 #include "kudu/client/scan_predicate-internal.h"
 #include "kudu/client/scan_token-internal.h"
 #include "kudu/client/scanner-internal.h"
+#include "kudu/client/schema-internal.h"
 #include "kudu/client/session-internal.h"
 #include "kudu/client/table-internal.h"
 #include "kudu/client/table_alterer-internal.h"
@@ -81,7 +83,6 @@
 #include "kudu/master/master.proxy.h"
 #include "kudu/rpc/messenger.h"
 #include "kudu/rpc/request_tracker.h"
-#include "kudu/rpc/response_callback.h"
 #include "kudu/rpc/rpc.h"
 #include "kudu/rpc/rpc_controller.h"
 #include "kudu/rpc/sasl_common.h"
@@ -90,7 +91,7 @@
 #include "kudu/security/tls_context.h"
 #include "kudu/security/token.pb.h"
 #include "kudu/tserver/tserver.pb.h"
-#include "kudu/tserver/tserver_service.proxy.h"
+#include "kudu/tserver/tserver_service.proxy.h" // IWYU pragma: keep
 #include "kudu/util/async_util.h"
 #include "kudu/util/debug-util.h"
 #include "kudu/util/init.h"
@@ -127,8 +128,12 @@ using kudu::rpc::Messenger;
 using kudu::rpc::MessengerBuilder;
 using kudu::rpc::RpcController;
 using kudu::rpc::UserCredentials;
+using kudu::security::JwtRawPB;
 using kudu::tserver::ScanResponsePB;
 using std::map;
+using std::make_optional;
+using std::nullopt;
+using std::optional;
 using std::set;
 using std::string;
 using std::unique_ptr;
@@ -304,11 +309,21 @@ KuduClientBuilder& KuduClientBuilder::connection_negotiation_timeout(
   return *this;
 }
 
+KuduClientBuilder& KuduClientBuilder::jwt(const string& jwt) {
+  data_->jwt_ = jwt;
+  return *this;
+}
+
 KuduClientBuilder& KuduClientBuilder::import_authentication_credentials(string authn_creds) {
   data_->authn_creds_ = std::move(authn_creds);
   return *this;
 }
 
+KuduClientBuilder& KuduClientBuilder::trusted_certificate(const string& cert_pem) {
+  data_->trusted_certs_pem_.emplace_back(cert_pem);
+  return *this;
+}
+
 KuduClientBuilder& KuduClientBuilder::num_reactors(int num_reactors) {
   data_->num_reactors_ = num_reactors;
   return *this;
@@ -346,6 +361,9 @@ Status ImportAuthnCreds(const string& authn_creds,
     }
     messenger->set_authn_token(tok);
   }
+  if (pb.has_jwt()) {
+    messenger->set_jwt(pb.jwt());
+  }
   if (pb.has_real_user()) {
     user_credentials->set_real_user(pb.real_user());
   }
@@ -370,7 +388,7 @@ Status KuduClientBuilder::Build(shared_ptr<KuduClient>* client) {
         data_->connection_negotiation_timeout_.ToMilliseconds());
   }
   if (data_->num_reactors_) {
-    builder.set_num_reactors(data_->num_reactors_.get());
+    builder.set_num_reactors(*data_->num_reactors_);
   }
   if (!data_->sasl_protocol_name_.empty()) {
     builder.set_sasl_proto_name(data_->sasl_protocol_name_);
@@ -388,15 +406,55 @@ Status KuduClientBuilder::Build(shared_ptr<KuduClient>* client) {
   RETURN_NOT_OK(builder.Build(&messenger));
   UserCredentials user_credentials;
 
-  // Parse and import the provided authn data, if any.
-  if (!data_->authn_creds_.empty()) {
-    RETURN_NOT_OK(ImportAuthnCreds(data_->authn_creds_, messenger.get(), &user_credentials));
+  // Parse and import the provided authn data, if any. Override the JWT portion
+  // of the imported authn credentials if the JWT was specified as well.
+  // If no serialized authn data was provided, use JWT if it's available.
+  if (!data_->authn_creds_.empty() || !data_->jwt_.empty()) {
+    string authn_creds;
+    if (!data_->authn_creds_.empty() && !data_->jwt_.empty()) {
+      // Merge the imported authn creds and the JWT: use data_->authn_creds_,
+      // but override its JWT portion with data_->jwt_.
+      AuthenticationCredentialsPB pb;
+      if (!pb.ParseFromString(data_->authn_creds_)) {
+        return Status::InvalidArgument("invalid authentication data");
+      }
+      if (pb.has_jwt()) {
+        JwtRawPB jwt_pb;
+        // Copying, not moving: Build() is supposed to be re-enterable.
+        *jwt_pb.mutable_jwt_data() = data_->jwt_;
+        *pb.mutable_jwt() = std::move(jwt_pb);
+      }
+      string creds;
+      if (!pb.SerializeToString(&creds)) {
+        return Status::RuntimeError("could not serialize authentication data");
+      }
+      authn_creds = std::move(creds);
+    } else if (!data_->jwt_.empty()) {
+      // Build authn creds with just the provided JWT.
+      JwtRawPB jwt_pb;
+      // Copying, not moving: Build() is supposed to be re-enterable.
+      *jwt_pb.mutable_jwt_data() = data_->jwt_;
+      AuthenticationCredentialsPB pb;
+      *pb.mutable_jwt() = std::move(jwt_pb);
+      if (!pb.SerializeToString(&authn_creds)) {
+        return Status::RuntimeError("could not serialize authentication data");
+      }
+    }
+    RETURN_NOT_OK(ImportAuthnCreds(
+        !authn_creds.empty() ? authn_creds : data_->authn_creds_,
+        messenger.get(),
+        &user_credentials));
   }
   if (!user_credentials.has_real_user()) {
     // If there are no authentication credentials, then set the real user to the
     // currently logged-in user.
     RETURN_NOT_OK(user_credentials.SetLoggedInRealUser());
   }
+  for (const auto& cert_str : data_->trusted_certs_pem_) {
+    security::Cert cert;
+    RETURN_NOT_OK(cert.FromString(cert_str, security::DataFormat::PEM));
+    RETURN_NOT_OK(messenger->mutable_tls_context()->AddTrustedCertificate(cert));
+  }
 
   shared_ptr<KuduClient> c(new KuduClient);
   c->data_->messenger_ = std::move(messenger);
@@ -506,13 +564,29 @@ Status KuduClient::IsCreateTableInProgress(const string& table_name,
 }
 
 Status KuduClient::DeleteTable(const string& table_name) {
+  // The default param 'reserve_seconds' not be set means the behavior of DeleteRPC is
+  // controlled by the 'default_deleted_table_reserve_seconds' flag.
   return DeleteTableInCatalogs(table_name, true);
 }
 
+Status KuduClient::SoftDeleteTable(const string& table_name,
+                                   uint32_t reserve_seconds) {
+  return DeleteTableInCatalogs(table_name, true, reserve_seconds);
+}
+
 Status KuduClient::DeleteTableInCatalogs(const string& table_name,
-                                         bool modify_external_catalogs) {
+                                         bool modify_external_catalogs,
+                                         int32_t reserve_seconds) {
   MonoTime deadline = MonoTime::Now() + default_admin_operation_timeout();
-  return data_->DeleteTable(this, table_name, deadline, modify_external_catalogs);
+  std::optional<uint32_t> reserve_time = reserve_seconds >= 0 ?
+      std::make_optional(reserve_seconds) : std::nullopt;
+  return  KuduClient::Data::DeleteTable(this, table_name, deadline, modify_external_catalogs,
+                                        reserve_time);
+}
+
+Status KuduClient::RecallTable(const string& table_id, const string& new_table_name) {
+  MonoTime deadline = MonoTime::Now() + default_admin_operation_timeout();
+  return KuduClient::Data::RecallTable(this, table_id, deadline, new_table_name);
 }
 
 KuduTableAlterer* KuduClient::NewTableAlterer(const string& table_name) {
@@ -563,7 +637,21 @@ Status KuduClient::ListTabletServers(vector<KuduTabletServer*>* tablet_servers)
 
 Status KuduClient::ListTables(vector<string>* tables, const string& filter) {
   vector<Data::TableInfo> tables_info;
-  RETURN_NOT_OK(data_->ListTablesWithInfo(this, &tables_info, filter));
+  RETURN_NOT_OK(data_->ListTablesWithInfo(this, &tables_info, filter, false));
+  tables->clear();
+  tables->reserve(tables_info.size());
+  for (auto& info : tables_info) {
+    tables->emplace_back(std::move(info.table_name));
+  }
+  return Status::OK();
+}
+
+Status KuduClient::ListSoftDeletedTables(vector<string>* tables, const string& filter) {
+  vector<Data::TableInfo> tables_info;
+  RETURN_NOT_OK(data_->ListTablesWithInfo(this, &tables_info, filter,
+      /*list_tablet_with_partition=*/ true, /*show_soft_deleted=*/ true));
+  tables->clear();
+  tables->reserve(tables_info.size());
   for (auto& info : tables_info) {
     tables->emplace_back(std::move(info.table_name));
   }
@@ -693,10 +781,10 @@ Status KuduClient::GetTableStatistics(const string& table_name,
   }
   unique_ptr<KuduTableStatistics> table_statistics(new KuduTableStatistics);
   table_statistics->data_ = new KuduTableStatistics::Data(
-      resp.has_on_disk_size() ? boost::optional<int64_t>(resp.on_disk_size()) : boost::none,
-      resp.has_live_row_count() ? boost::optional<int64_t>(resp.live_row_count()) : boost::none,
-      resp.has_disk_size_limit() ? boost::optional<int64_t>(resp.disk_size_limit()) : boost::none,
-      resp.has_row_count_limit() ? boost::optional<int64_t>(resp.row_count_limit()) : boost::none);
+      resp.has_on_disk_size() ? optional<int64_t>(resp.on_disk_size()) : nullopt,
+      resp.has_live_row_count() ? optional<int64_t>(resp.live_row_count()) : nullopt,
+      resp.has_disk_size_limit() ? optional<int64_t>(resp.disk_size_limit()) : nullopt,
+      resp.has_row_count_limit() ? optional<int64_t>(resp.row_count_limit()) : nullopt);
 
   *statistics = table_statistics.release();
   return Status::OK();
@@ -737,10 +825,12 @@ void KuduClient::SetLatestObservedTimestamp(uint64_t ht_timestamp) {
 Status KuduClient::ExportAuthenticationCredentials(string* authn_creds) const {
   AuthenticationCredentialsPB pb;
 
-  boost::optional<security::SignedTokenPB> tok = data_->messenger_->authn_token();
-  if (tok) {
+  if (auto tok = data_->messenger_->authn_token(); tok) {
     pb.mutable_authn_token()->CopyFrom(*tok);
   }
+  if (auto jwt = data_->messenger_->jwt(); jwt) {
+    pb.mutable_jwt()->CopyFrom(*jwt);
+  }
   pb.set_real_user(data_->user_credentials_.real_user());
 
   vector<string> cert_ders;
@@ -915,21 +1005,21 @@ Status KuduTableCreator::Create() {
   // Build request.
   CreateTableRequestPB req;
   req.set_name(data_->table_name_);
-  if (data_->num_replicas_ != boost::none) {
-    req.set_num_replicas(data_->num_replicas_.get());
+  if (data_->num_replicas_) {
+    req.set_num_replicas(*data_->num_replicas_);
   }
   if (data_->dimension_label_) {
-    req.set_dimension_label(data_->dimension_label_.get());
+    req.set_dimension_label(*data_->dimension_label_);
   }
   if (data_->extra_configs_) {
     req.mutable_extra_configs()->insert(data_->extra_configs_->begin(),
                                         data_->extra_configs_->end());
   }
-  if (data_->owner_ != boost::none) {
-    req.set_owner(data_->owner_.get());
+  if (data_->owner_) {
+    req.set_owner(*data_->owner_);
   }
-  if (data_->comment_ != boost::none) {
-    req.set_comment(data_->comment_.get());
+  if (data_->comment_) {
+    req.set_comment(*data_->comment_);
   }
   RETURN_NOT_OK_PREPEND(SchemaToPB(*data_->schema_->schema_, req.mutable_schema(),
                                    SCHEMA_PB_WITHOUT_WRITE_DEFAULT),
@@ -970,24 +1060,26 @@ Status KuduTableCreator::Create() {
       return Status::InvalidArgument("range bounds must not be null");
     }
 
-    RowOperationsPB_Type lower_bound_type =
+    const RowOperationsPB_Type lower_bound_type =
         range->lower_bound_type_ == KuduTableCreator::INCLUSIVE_BOUND
         ? RowOperationsPB::RANGE_LOWER_BOUND
         : RowOperationsPB::EXCLUSIVE_RANGE_LOWER_BOUND;
 
-    RowOperationsPB_Type upper_bound_type =
+    const RowOperationsPB_Type upper_bound_type =
         range->upper_bound_type_ == KuduTableCreator::EXCLUSIVE_BOUND
         ? RowOperationsPB::RANGE_UPPER_BOUND
         : RowOperationsPB::INCLUSIVE_RANGE_UPPER_BOUND;
 
-    splits_encoder.Add(lower_bound_type, *range->lower_bound_);
-    splits_encoder.Add(upper_bound_type, *range->upper_bound_);
-
-    if (has_range_with_custom_hash_schema) {
+    if (!has_range_with_custom_hash_schema) {
+      splits_encoder.Add(lower_bound_type, *range->lower_bound_);
+      splits_encoder.Add(upper_bound_type, *range->upper_bound_);
+    } else {
       auto* range_pb = partition_schema->add_custom_hash_schema_ranges();
       RowOperationsPBEncoder encoder(range_pb->mutable_range_bounds());
       encoder.Add(lower_bound_type, *range->lower_bound_);
       encoder.Add(upper_bound_type, *range->upper_bound_);
+      // Now, after adding the information range bounds, add the information
+      // on hash schema for the range.
       if (range->is_table_wide_hash_schema_) {
         // With the presence of a range with custom hash schema when the
         // table-wide hash schema is used for this particular range, also add an
@@ -1010,6 +1102,16 @@ Status KuduTableCreator::Create() {
     }
   }
 
+  bool has_immutable_column_schema = false;
+  for (size_t i = 0; i < data_->schema_->num_columns(); i++) {
+    const auto& col_schema = data_->schema_->Column(i);
+    if (col_schema.is_immutable()) {
+      has_immutable_column_schema = true;
+      break;
+    }
+  }
+  bool has_auto_incrementing_column = data_->schema_->schema_->has_auto_incrementing();
+
   if (data_->table_type_) {
     req.set_table_type(*data_->table_type_);
   }
@@ -1022,10 +1124,16 @@ Status KuduTableCreator::Create() {
   }
 
   CreateTableResponsePB resp;
-  RETURN_NOT_OK_PREPEND(data_->client_->data_->CreateTable(
-      data_->client_, req, &resp, deadline, !data_->range_partitions_.empty()),
-                        Substitute("Error creating table $0 on the master",
-                                   data_->table_name_));
+  RETURN_NOT_OK_PREPEND(
+      data_->client_->data_->CreateTable(data_->client_,
+                                         req,
+                                         &resp,
+                                         deadline,
+                                         !data_->range_partitions_.empty(),
+                                         has_range_with_custom_hash_schema,
+                                         has_immutable_column_schema,
+                                         has_auto_incrementing_column),
+      Substitute("Error creating table $0 on the master", data_->table_name_));
   // Spin until the table is fully created, if requested.
   if (data_->wait_) {
     TableIdentifierPB table;
@@ -1037,28 +1145,26 @@ Status KuduTableCreator::Create() {
   return Status::OK();
 }
 
-KuduTableCreator::KuduRangePartition::KuduRangePartition(
+KuduRangePartition::KuduRangePartition(
     KuduPartialRow* lower_bound,
     KuduPartialRow* upper_bound,
-    RangePartitionBound lower_bound_type,
-    RangePartitionBound upper_bound_type)
+    KuduTableCreator::RangePartitionBound lower_bound_type,
+    KuduTableCreator::RangePartitionBound upper_bound_type)
     : data_(new Data(lower_bound, upper_bound, lower_bound_type, upper_bound_type)) {
 }
 
-KuduTableCreator::KuduRangePartition::~KuduRangePartition() {
+KuduRangePartition::~KuduRangePartition() {
   delete data_;
 }
 
-Status KuduTableCreator::KuduRangePartition::add_hash_partitions(
+Status KuduRangePartition::add_hash_partitions(
     const vector<string>& columns,
     int32_t num_buckets,
     int32_t seed) {
   if (seed < 0) {
-    // TODO(aserbin): change the signature of
-    //                KuduRangePartition::add_hash_partitions() to use uint32_t
-    //                for the 'seed' parameter while it's still possible since
-    //                the client API hasn't been released yet
-    return Status::InvalidArgument("hash seed must non-negative");
+    // int32_t, not uint32_t for seed is used to be "compatible" with the type
+    // of the 'seed' parameter for KuduTableCreator::add_hash_partitions().
+    return Status::InvalidArgument("hash seed must be non-negative");
   }
   return data_->add_hash_partitions(columns, num_buckets, seed);
 }
@@ -1151,6 +1257,10 @@ KuduUpsert* KuduTable::NewUpsert() {
   return new KuduUpsert(shared_from_this());
 }
 
+KuduUpsertIgnore* KuduTable::NewUpsertIgnore() {
+  return new KuduUpsertIgnore(shared_from_this());
+}
+
 KuduUpdate* KuduTable::NewUpdate() {
   return new KuduUpdate(shared_from_this());
 }
@@ -1446,6 +1556,10 @@ KuduClient* KuduSession::client() const {
   return data_->client_.get();
 }
 
+const ResourceMetrics& KuduSession::GetWriteOpMetrics() const {
+  return data_->write_op_metrics_;
+}
+
 ////////////////////////////////////////////////////////////
 // KuduTableAlterer
 ////////////////////////////////////////////////////////////
@@ -1474,7 +1588,7 @@ KuduTableAlterer* KuduTableAlterer::SetComment(const string& new_comment) {
 
 KuduColumnSpec* KuduTableAlterer::AddColumn(const string& name) {
   Data::Step s = { AlterTableRequestPB::ADD_COLUMN,
-                   new KuduColumnSpec(name), nullptr, nullptr };
+                   new KuduColumnSpec(name), nullptr };
   auto* spec = s.spec;
   data_->steps_.emplace_back(std::move(s));
   return spec;
@@ -1482,7 +1596,7 @@ KuduColumnSpec* KuduTableAlterer::AddColumn(const string& name) {
 
 KuduColumnSpec* KuduTableAlterer::AlterColumn(const string& name) {
   Data::Step s = { AlterTableRequestPB::ALTER_COLUMN,
-                   new KuduColumnSpec(name), nullptr, nullptr };
+                   new KuduColumnSpec(name), nullptr };
   auto* spec = s.spec;
   data_->steps_.emplace_back(std::move(s));
   return spec;
@@ -1490,7 +1604,7 @@ KuduColumnSpec* KuduTableAlterer::AlterColumn(const string& name) {
 
 KuduTableAlterer* KuduTableAlterer::DropColumn(const string& name) {
   Data::Step s = { AlterTableRequestPB::DROP_COLUMN,
-                   new KuduColumnSpec(name), nullptr, nullptr };
+                   new KuduColumnSpec(name), nullptr };
   data_->steps_.emplace_back(std::move(s));
   return this;
 }
@@ -1515,29 +1629,57 @@ KuduTableAlterer* KuduTableAlterer::AddRangePartitionWithDimension(
     data_->status_ = Status::InvalidArgument("range partition bounds may not be null");
     return this;
   }
-  if (!lower_bound->schema()->Equals(*upper_bound->schema())) {
+  if (*lower_bound->schema() != *upper_bound->schema()) {
     data_->status_ = Status::InvalidArgument("range partition bounds must have matching schemas");
     return this;
   }
   if (data_->schema_ == nullptr) {
     data_->schema_ = lower_bound->schema();
-  } else if (!lower_bound->schema()->Equals(*data_->schema_)) {
+  } else if (*lower_bound->schema() != *data_->schema_) {
     data_->status_ = Status::InvalidArgument("range partition bounds must have matching schemas");
     return this;
   }
 
   Data::Step s { AlterTableRequestPB::ADD_RANGE_PARTITION,
                  nullptr,
-                 unique_ptr<KuduPartialRow>(lower_bound),
-                 unique_ptr<KuduPartialRow>(upper_bound),
-                 lower_bound_type,
-                 upper_bound_type,
-                 dimension_label.empty() ? boost::none : boost::make_optional(dimension_label) };
+                 std::unique_ptr<KuduRangePartition>(new KuduRangePartition(
+                     lower_bound, upper_bound, lower_bound_type, upper_bound_type)),
+                 dimension_label.empty() ? nullopt : make_optional(dimension_label) };
   data_->steps_.emplace_back(std::move(s));
   data_->has_alter_partitioning_steps = true;
   return this;
 }
 
+KuduTableAlterer* KuduTableAlterer::AddRangePartition(
+    KuduRangePartition* partition) {
+  CHECK(partition);
+  if (partition->data_->lower_bound_ == nullptr || partition->data_->upper_bound_  == nullptr) {
+    data_->status_ = Status::InvalidArgument("range partition bounds may not be null");
+    return this;
+  }
+  if (partition->data_->lower_bound_->schema() != partition->data_->upper_bound_->schema()) {
+    data_->status_ = Status::InvalidArgument("range partition bounds must have matching schemas");
+    return this;
+  }
+  if (data_->schema_ == nullptr) {
+    data_->schema_ = partition->data_->lower_bound_->schema();
+  } else if (partition->data_->lower_bound_->schema() != data_->schema_) {
+    data_->status_ = Status::InvalidArgument("range partition bounds must have matching schemas");
+    return this;
+  }
+
+  Data::Step s { AlterTableRequestPB::ADD_RANGE_PARTITION,
+                 nullptr,
+                 std::unique_ptr<KuduRangePartition>(partition),
+                 nullopt };
+  data_->steps_.emplace_back(std::move(s));
+  data_->has_alter_partitioning_steps = true;
+  if (!data_->steps_.back().range_partition->data_->is_table_wide_hash_schema_) {
+    data_->adding_range_with_custom_hash_schema = true;
+  }
+  return this;
+}
+
 KuduTableAlterer* KuduTableAlterer::DropRangePartition(
     KuduPartialRow* lower_bound,
     KuduPartialRow* upper_bound,
@@ -1547,23 +1689,21 @@ KuduTableAlterer* KuduTableAlterer::DropRangePartition(
     data_->status_ = Status::InvalidArgument("range partition bounds may not be null");
     return this;
   }
-  if (!lower_bound->schema()->Equals(*upper_bound->schema())) {
+  if (*lower_bound->schema() != *upper_bound->schema()) {
     data_->status_ = Status::InvalidArgument("range partition bounds must have matching schemas");
     return this;
   }
   if (data_->schema_ == nullptr) {
     data_->schema_ = lower_bound->schema();
-  } else if (!lower_bound->schema()->Equals(*data_->schema_)) {
+  } else if (*lower_bound->schema() != *data_->schema_) {
     data_->status_ = Status::InvalidArgument("range partition bounds must have matching schemas");
     return this;
   }
 
   Data::Step s { AlterTableRequestPB::DROP_RANGE_PARTITION,
                  nullptr,
-                 unique_ptr<KuduPartialRow>(lower_bound),
-                 unique_ptr<KuduPartialRow>(upper_bound),
-                 lower_bound_type,
-                 upper_bound_type };
+                 std::unique_ptr<KuduRangePartition>(new KuduRangePartition(
+                     lower_bound, upper_bound, lower_bound_type, upper_bound_type)) };
   data_->steps_.emplace_back(std::move(s));
   data_->has_alter_partitioning_steps = true;
   return this;
@@ -1605,12 +1745,25 @@ Status KuduTableAlterer::Alter() {
   AlterTableResponsePB resp;
   RETURN_NOT_OK(data_->ToRequest(&req));
 
+  bool has_immutable_column_schema = false;
+  for (const auto& step : data_->steps_) {
+    if ((step.step_type == AlterTableRequestPB::ADD_COLUMN ||
+         step.step_type == AlterTableRequestPB::ALTER_COLUMN) &&
+        step.spec->data_->immutable) {
+      has_immutable_column_schema = true;
+      break;
+    }
+  }
+
   MonoDelta timeout = data_->timeout_.Initialized() ?
     data_->timeout_ :
     data_->client_->default_admin_operation_timeout();
   MonoTime deadline = MonoTime::Now() + timeout;
-  RETURN_NOT_OK(data_->client_->data_->AlterTable(data_->client_, req, &resp, deadline,
-                                                  data_->has_alter_partitioning_steps));
+  RETURN_NOT_OK(data_->client_->data_->AlterTable(
+      data_->client_, req, &resp, deadline,
+      data_->has_alter_partitioning_steps,
+      data_->adding_range_with_custom_hash_schema,
+      has_immutable_column_schema));
 
   if (data_->has_alter_partitioning_steps) {
     // If the table partitions change, clear the local meta cache so that the
@@ -1776,30 +1929,33 @@ Status KuduScanner::AddExclusiveUpperBoundRaw(const Slice& key) {
 }
 
 Status KuduScanner::AddLowerBoundPartitionKeyRaw(const Slice& partition_key) {
-  // TODO(aserbin): use move semantics to pass PartitionKey arguments
-  if (const auto& table = GetKuduTable();
-      table->data_->partition_schema_.HasCustomHashSchemas()) {
-    return Status::InvalidArgument(Substitute(
-        "$0: cannot use AddLowerBoundPartitionKeyRaw() because "
-        "the table has custom per-range hash schemas", table->name()));
-  }
+  // The number of hash dimensions in all hash schemas of a table is an
+  // invariant and checked throughout the code. With that, the table-wide hash
+  // schema is used as a proxy to find the number of hash dimensions to separate
+  // the hash-related prefix from the rest of the encoded partition key in the
+  // code below.
+  //
+  // TODO(KUDU-2671) update this code if allowing for different number of
+  //                 dimensions in range-specific hash schemas
   const auto& hash_schema = GetKuduTable()->partition_schema().hash_schema();
-  auto pkey = Partition::StringToPartitionKey(partition_key.ToString(),
-                                              hash_schema.size());
-  return data_->mutable_configuration()->AddLowerBoundPartitionKeyRaw(pkey);
+  return data_->mutable_configuration()->AddLowerBoundPartitionKeyRaw(
+      Partition::StringToPartitionKey(partition_key.ToString(),
+                                      hash_schema.size()));
 }
 
 Status KuduScanner::AddExclusiveUpperBoundPartitionKeyRaw(const Slice& partition_key) {
-  if (const auto& table = GetKuduTable();
-      table->data_->partition_schema_.HasCustomHashSchemas()) {
-    return Status::InvalidArgument(Substitute(
-        "$0: cannot use AddExclusiveUpperBoundPartitionKeyRaw() because "
-        "the table has custom per-range hash schemas", table->name()));
-  }
+  // The number of hash dimensions in all hash schemas of a table is an
+  // invariant and checked throughout the code. With that, the table-wide hash
+  // schema is used as a proxy to find the number of hash dimensions to separate
+  // the hash-related prefix from the rest of the encoded partition key in the
+  // code below.
+  //
+  // TODO(KUDU-2671) update this code if allowing for different number of
+  //                 dimensions in range-specific hash schemas
   const auto& hash_schema = GetKuduTable()->partition_schema().hash_schema();
-  auto pkey = Partition::StringToPartitionKey(partition_key.ToString(),
-                                              hash_schema.size());
-  return data_->mutable_configuration()->AddUpperBoundPartitionKeyRaw(pkey);
+  return data_->mutable_configuration()->AddUpperBoundPartitionKeyRaw(
+      Partition::StringToPartitionKey(partition_key.ToString(),
+                                      hash_schema.size()));
 }
 
 Status KuduScanner::SetCacheBlocks(bool cache_blocks) {
@@ -1979,7 +2135,6 @@ Status KuduScanner::NextBatch(internal::ScanBatchDataInterface* batch_data) {
   // need to do some swapping of the response objects around to avoid
   // stomping on the memory the user is looking at.
   CHECK(data_->open_);
-  CHECK(data_->proxy_);
 
   batch_data->Clear();
 
@@ -1989,6 +2144,7 @@ Status KuduScanner::NextBatch(internal::ScanBatchDataInterface* batch_data) {
 
   if (data_->data_in_open_) {
     // We have data from a previous scan.
+    CHECK(data_->proxy_);
     VLOG(2) << "Extracting data from " << data_->DebugString();
     data_->data_in_open_ = false;
     return batch_data->Reset(&data_->controller_,
@@ -2000,6 +2156,7 @@ Status KuduScanner::NextBatch(internal::ScanBatchDataInterface* batch_data) {
 
   if (data_->last_response_.has_more_results()) {
     // More data is available in this tablet.
+    CHECK(data_->proxy_);
     VLOG(2) << "Continuing " << data_->DebugString();
 
     MonoTime batch_deadline = MonoTime::Now() + data_->configuration().timeout();
@@ -2208,6 +2365,10 @@ Status KuduScanTokenBuilder::Build(vector<KuduScanToken*>* tokens) {
   return data_->Build(tokens);
 }
 
+void KuduScanTokenBuilder::SetSplitSizeBytes(uint64_t split_size_bytes) {
+  return data_->SplitSizeBytes(split_size_bytes);
+}
+
 ////////////////////////////////////////////////////////////
 // KuduReplica
 ////////////////////////////////////////////////////////////
