@@ -16,11 +16,10 @@
 // under the License.
 #pragma once
 
+#include <optional>
 #include <string>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
-
 #include "kudu/client/client.h"
 #include "kudu/client/replica_controller-internal.h"
 #include "kudu/gutil/macros.h"
@@ -40,11 +39,13 @@ class KuduClientBuilder::Data {
   MonoDelta default_rpc_timeout_;
   MonoDelta connection_negotiation_timeout_;
   std::string authn_creds_;
+  std::string jwt_;
   internal::ReplicaController::Visibility replica_visibility_;
-  boost::optional<int> num_reactors_;
+  std::optional<int> num_reactors_;
   std::string sasl_protocol_name_;
   bool require_authentication_;
   EncryptionPolicy encryption_policy_;
+  std::vector<std::string> trusted_certs_pem_;
 
   DISALLOW_COPY_AND_ASSIGN(Data);
 };
