@@ -19,32 +19,41 @@
 
 #include <functional>
 #include <mutex>
+#include <type_traits>
 #include <utility>
 
 #include <glog/logging.h>
+#include <google/protobuf/descriptor.h>
+#include <google/protobuf/message.h>
 
 #include "kudu/client/batcher.h"
 #include "kudu/client/callbacks.h"
 #include "kudu/client/error_collector.h"
+#include "kudu/client/resource_metrics-internal.h"
 #include "kudu/client/shared_ptr.h" // IWYU pragma: keep
 #include "kudu/client/write_op.h"
 #include "kudu/common/partial_row.h"
 #include "kudu/common/schema.h"
 #include "kudu/gutil/port.h"
+#include "kudu/gutil/strings/stringpiece.h"
 #include "kudu/gutil/strings/substitute.h"
-#include "kudu/rpc/messenger.h"
+#include "kudu/rpc/messenger.h" // IWYU pragma: keep
+#include "kudu/tserver/tserver.pb.h"
 #include "kudu/util/logging.h"
 
-
+using google::protobuf::FieldDescriptor;
+using google::protobuf::Reflection;
 using kudu::client::internal::Batcher;
 using kudu::client::internal::ErrorCollector;
 using kudu::client::sp::shared_ptr;
 using kudu::client::sp::weak_ptr;
 using kudu::rpc::Messenger;
+using kudu::tserver::ResourceMetricsPB;
 using std::unique_ptr;
 using strings::Substitute;
 
 namespace kudu {
+
 namespace client {
 
 KuduSession::Data::Data(shared_ptr<KuduClient> client,
@@ -335,6 +344,32 @@ Status CheckForPrimaryKey(const KuduWriteOperation& op) {
   return Status::OK();
 }
 
+// Check if the non-unique primary key is set for the write operation.
+Status CheckForNonUniquePrimaryKey(const KuduWriteOperation& op) {
+  if (PREDICT_FALSE(!op.row().IsNonUniqueKeySet())) {
+    return Status::IllegalState("Non-unique key not specified", KUDU_REDACT(op.ToString()));
+  }
+  return Status::OK();
+}
+
+// Ensure the auto-incrementing column is not set for the Insert operation.
+Status CheckAutoIncrementingColumnForInsert(const KuduWriteOperation& op) {
+  if (op.row().schema()->has_auto_incrementing() && op.row().IsAutoIncrementingColumnSet()) {
+    return Status::InvalidArgument("Auto-Incrementing column should not be "
+                                   "specified for INSERT operation");
+  }
+  return Status::OK();
+}
+
+// Ensure the auto-incrementing column is set for Non-Insert operations.
+Status CheckAutoIncrementingColumnForNonInsert(const KuduWriteOperation& op) {
+  if (op.row().schema()->has_auto_incrementing() && !op.row().IsAutoIncrementingColumnSet()) {
+    return Status::InvalidArgument("Auto-Incrementing column should be "
+                                   "specified for UPSERT/UPDATE operations");
+  }
+  return Status::OK();
+}
+
 // Check if the values for the non-nullable columns are present.
 Status CheckForNonNullableColumns(const KuduWriteOperation& op) {
   const auto& row = op.row();
@@ -343,7 +378,7 @@ Status CheckForNonNullableColumns(const KuduWriteOperation& op) {
   for (auto idx = 0; idx < num_columns; ++idx) {
     const ColumnSchema& col = schema->column(idx);
     if (!col.is_nullable() && !col.has_write_default() &&
-        !row.IsColumnSet(idx)) {
+        !row.IsColumnSet(idx) && !col.is_auto_incrementing()) {
       return Status::IllegalState(Substitute(
           "non-nullable column '$0' is not set", schema->column(idx).name()),
           KUDU_REDACT(op.ToString()));
@@ -364,11 +399,23 @@ Status CheckForNonNullableColumns(const KuduWriteOperation& op) {
   } while (false)
 
 Status KuduSession::Data::ValidateWriteOperation(KuduWriteOperation* op) const {
-  RETURN_NOT_OK_ADD_ERROR(CheckForPrimaryKey, op, error_collector_);
+  if (op->row().schema()->has_auto_incrementing()) {
+    RETURN_NOT_OK_ADD_ERROR(CheckForNonUniquePrimaryKey, op, error_collector_);
+  } else {
+    RETURN_NOT_OK_ADD_ERROR(CheckForPrimaryKey, op, error_collector_);
+  }
   switch (op->type()) {
     case KuduWriteOperation::INSERT:
+    case KuduWriteOperation::INSERT_IGNORE:
+      RETURN_NOT_OK_ADD_ERROR(CheckAutoIncrementingColumnForInsert, op, error_collector_);
+      RETURN_NOT_OK_ADD_ERROR(CheckForNonNullableColumns, op, error_collector_);
+      break;
     case KuduWriteOperation::UPSERT:
       RETURN_NOT_OK_ADD_ERROR(CheckForNonNullableColumns, op, error_collector_);
+    case KuduWriteOperation::UPSERT_IGNORE:
+    case KuduWriteOperation::UPDATE:
+    case KuduWriteOperation::UPDATE_IGNORE:
+      RETURN_NOT_OK_ADD_ERROR(CheckAutoIncrementingColumnForNonInsert, op, error_collector_);
       break;
     default:
       // Nothing else to validate for other types of write operations.
@@ -577,5 +624,18 @@ size_t KuduSession::Data::GetBatchersCountForTests() const {
   return batchers_num_;
 }
 
+void KuduSession::Data::UpdateWriteOpMetrics(const ResourceMetricsPB& resource_metrics) {
+  const auto* reflection = resource_metrics.GetReflection();
+  const auto* desc = resource_metrics.GetDescriptor();
+  for (int i = 0; i < desc->field_count(); i++) {
+    const FieldDescriptor* field = desc->field(i);
+    if (reflection->HasField(resource_metrics, field) &&
+        field->cpp_type() == FieldDescriptor::CPPTYPE_INT64) {
+      write_op_metrics_.data_->Increment(StringPiece(field->name()),
+                                         reflection->GetInt64(resource_metrics, field));
+    }
+  }
+}
+
 } // namespace client
 } // namespace kudu
