@@ -18,14 +18,15 @@
 #include "kudu/consensus/log.h"
 
 #include <algorithm>
-#include <cstddef>
 #include <cerrno>
+#include <cstddef>
 #include <cstdint>
 #include <functional>
 #include <limits>
 #include <memory>
 #include <ostream>
 #include <string>
+#include <type_traits>
 #include <vector>
 
 #include <gflags/gflags.h>
@@ -227,7 +228,7 @@ TEST_P(LogTestOptionalCompression, TestMultipleEntriesInABatch) {
     ASSERT_OK(log_->reader()->LookupOpId(2, &loaded_op));
     ASSERT_EQ("1.2", OpIdToString(loaded_op));
     Status s = log_->reader()->LookupOpId(3, &loaded_op);
-    ASSERT_TRUE(s.IsNotFound()) << "unexpected status: " << s.ToString();
+    ASSERT_TRUE(s.IsNotFound()) << s.ToString();
   }
 
   ASSERT_OK(log_->Close());
@@ -466,6 +467,7 @@ TEST_F(LogTest, TestWriteAndReadToAndFromInProgressSegment) {
   ReplicateMsg* repl = log_entry->mutable_replicate();
   repl->mutable_id()->CopyFrom(op_id);
   repl->set_op_type(NO_OP);
+  repl->mutable_noop_request();
   repl->set_timestamp(0L);
 
   // Entries are prefixed with a header.
@@ -645,7 +647,7 @@ TEST_P(LogTestOptionalCompression, TestGCOfIndexChunks) {
   ASSERT_EQ(1, num_gced_segments);
 
   Status s = log_->reader()->LookupOpId(999995, &loaded_op);
-  ASSERT_TRUE(s.IsNotFound()) << "unexpected status: " << s.ToString();
+  ASSERT_TRUE(s.IsNotFound()) << s.ToString();
 }
 
 // Tests that we can append FLUSH_MARKER messages to the log queue to make sure
@@ -1073,7 +1075,7 @@ TEST_F(LogTest, TestGetGCableDataSize) {
 
   const int kNumTotalSegments = 5;
   const int kNumOpsPerSegment = 5;
-  const int kSegmentSizeBytes = 331 + env_->GetEncryptionHeaderSize();
+  const int64_t kSegmentSizeBytes = 352 + env_->GetEncryptionHeaderSize();
   OpId op_id = MakeOpId(1, 10);
   // Create 5 segments, starting from log index 10, with 5 ops per segment.
   // [10-14], [15-19], [20-24], [25-29], [30-34]
@@ -1128,7 +1130,7 @@ TEST_F(LogTest, TestDiskSpaceCheck) {
   FLAGS_disk_reserved_bytes_free_for_testing = 0;
   options_.segment_size_mb = 1;
   Status s = BuildLog();
-  ASSERT_TRUE(s.IsIOError());
+  ASSERT_TRUE(s.IsIOError()) << s.ToString();
   ASSERT_EQ(ENOSPC, s.posix_code());
   ASSERT_STR_CONTAINS(s.ToString(), "Insufficient disk space");
 
