@@ -682,8 +682,7 @@ bool STLIncludes(const SortedSTLContainerA &a,
 // the contents of an STL map. For other sample usage, see the unittest.
 
 template<typename Pair, typename UnaryOp>
-class UnaryOperateOnFirst
-    : public std::unary_function<Pair, typename UnaryOp::result_type> {
+class UnaryOperateOnFirst {
  public:
   UnaryOperateOnFirst() {
   }
@@ -705,8 +704,7 @@ UnaryOperateOnFirst<Pair, UnaryOp> UnaryOperate1st(const UnaryOp& f) {
 }
 
 template<typename Pair, typename UnaryOp>
-class UnaryOperateOnSecond
-    : public std::unary_function<Pair, typename UnaryOp::result_type> {
+class UnaryOperateOnSecond {
  public:
   UnaryOperateOnSecond() {
   }
@@ -728,8 +726,7 @@ UnaryOperateOnSecond<Pair, UnaryOp> UnaryOperate2nd(const UnaryOp& f) {
 }
 
 template<typename Pair, typename BinaryOp>
-class BinaryOperateOnFirst
-    : public std::binary_function<Pair, Pair, typename BinaryOp::result_type> {
+class BinaryOperateOnFirst {
  public:
   BinaryOperateOnFirst() {
   }
@@ -753,8 +750,7 @@ BinaryOperateOnFirst<Pair, BinaryOp> BinaryOperate1st(const BinaryOp& f) {
 }
 
 template<typename Pair, typename BinaryOp>
-class BinaryOperateOnSecond
-    : public std::binary_function<Pair, Pair, typename BinaryOp::result_type> {
+class BinaryOperateOnSecond {
  public:
   BinaryOperateOnSecond() {
   }
@@ -790,9 +786,7 @@ BinaryOperateOnSecond<Pair, BinaryOp> BinaryOperate2nd(const BinaryOp& f) {
 // F has to be a model of AdaptableBinaryFunction.
 // G1 and G2 have to be models of AdabtableUnaryFunction.
 template<typename F, typename G1, typename G2>
-class BinaryComposeBinary : public std::binary_function<typename G1::argument_type,
-                                                        typename G2::argument_type,
-                                                        typename F::result_type> {
+class BinaryComposeBinary {
  public:
   BinaryComposeBinary(F f, G1 g1, G2 g2) : f_(f), g1_(g1), g2_(g2) { }
 
