@@ -8,11 +8,11 @@
 #include <cstring>
 
 #include <limits>
+#include <memory>
 #include <ostream>
 #include <vector>
 
 #include "kudu/gutil/charmap.h"
-#include "kudu/gutil/gscoped_ptr.h"
 #include "kudu/gutil/integral_types.h"
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/stl_util.h"
@@ -21,6 +21,7 @@
 
 using std::numeric_limits;
 using std::string;
+using std::unique_ptr;
 using std::vector;
 
 namespace strings {
@@ -232,7 +233,7 @@ int UnescapeCEscapeString(const string& src, string* dest,
 }
 
 string UnescapeCEscapeString(const string& src) {
-  gscoped_array<char> unescaped(new char[src.size() + 1]);
+  unique_ptr<char[]> unescaped(new char[src.size() + 1]);
   int len = UnescapeCEscapeSequences(src.c_str(), unescaped.get(), nullptr);
   return string(unescaped.get(), len);
 }
@@ -584,7 +585,7 @@ int Utf8SafeCHexEscapeString(const char* src, int src_len, char* dest,
 // ----------------------------------------------------------------------
 string CEscape(const StringPiece& src) {
   const int dest_length = src.size() * 4 + 1;  // Maximum possible expansion
-  gscoped_array<char> dest(new char[dest_length]);
+  unique_ptr<char[]> dest(new char[dest_length]);
   const int len = CEscapeInternal(src.data(), src.size(),
                                   dest.get(), dest_length, false, false);
   DCHECK_GE(len, 0);
@@ -593,7 +594,7 @@ string CEscape(const StringPiece& src) {
 
 string CHexEscape(const StringPiece& src) {
   const int dest_length = src.size() * 4 + 1;  // Maximum possible expansion
-  gscoped_array<char> dest(new char[dest_length]);
+  unique_ptr<char[]> dest(new char[dest_length]);
   const int len = CEscapeInternal(src.data(), src.size(),
                                   dest.get(), dest_length, true, false);
   DCHECK_GE(len, 0);
@@ -602,7 +603,7 @@ string CHexEscape(const StringPiece& src) {
 
 string Utf8SafeCEscape(const StringPiece& src) {
   const int dest_length = src.size() * 4 + 1;  // Maximum possible expansion
-  gscoped_array<char> dest(new char[dest_length]);
+  unique_ptr<char[]> dest(new char[dest_length]);
   const int len = CEscapeInternal(src.data(), src.size(),
                                   dest.get(), dest_length, false, true);
   DCHECK_GE(len, 0);
@@ -611,7 +612,7 @@ string Utf8SafeCEscape(const StringPiece& src) {
 
 string Utf8SafeCHexEscape(const StringPiece& src) {
   const int dest_length = src.size() * 4 + 1;  // Maximum possible expansion
-  gscoped_array<char> dest(new char[dest_length]);
+  unique_ptr<char[]> dest(new char[dest_length]);
   const int len = CEscapeInternal(src.data(), src.size(),
                                   dest.get(), dest_length, true, true);
   DCHECK_GE(len, 0);
