@@ -17,7 +17,6 @@
 
 #include <cstdlib>
 #include <memory>
-#include <ostream>
 #include <string>
 #include <thread>
 #include <vector>
@@ -82,6 +81,7 @@ class ClientFailoverOnNegotiationTimeoutITest : public KuduTest {
         "--leader_failure_exp_backoff_max_delta_ms=1000",
         // Decreasing TS->master heartbeat interval speeds up the test.
         "--heartbeat_interval_ms=25",
+        "--enable_txn_system_client_init=true",
     };
     cluster_opts_.extra_master_flags = {
         // Speed up Raft elections.
@@ -137,10 +137,7 @@ TEST_F(ClientFailoverOnNegotiationTimeoutITest, Kudu1580ConnectToTServer) {
   static const int kTimeoutMs = 5 * 60 * 1000;
   static const char* kTableName = "kudu1580";
 
-  if (!AllowSlowTests()) {
-    LOG(WARNING) << "test is skipped; set KUDU_ALLOW_SLOW_TESTS=1 to run";
-    return;
-  }
+  SKIP_IF_SLOW_NOT_ALLOWED();
 
   cluster_opts_.num_tablet_servers = kNumTabletServers;
   ASSERT_OK(CreateAndStartCluster());
@@ -207,7 +204,9 @@ TEST_F(ClientFailoverOnNegotiationTimeoutITest, TestTxnSystemClientRetryOnPause)
   ASSERT_OK(CreateAndStartCluster());
 
   unique_ptr<TxnSystemClient> txn_client;
-  ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(), &txn_client));
+  ASSERT_OK(TxnSystemClient::Create(cluster_->master_rpc_addrs(),
+                                    cluster_->service_principal(),
+                                    &txn_client));
   ASSERT_OK(txn_client->CreateTxnStatusTable(100, kNumTabletServers));
   ASSERT_OK(txn_client->OpenTxnStatusTable());
 
