@@ -19,11 +19,12 @@
 #include <cstdint>
 #include <functional>
 #include <memory>
+#include <optional>
 #include <ostream>
 #include <string>
+#include <type_traits>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <gflags/gflags_declare.h>
 #include <glog/logging.h>
 #include <gtest/gtest.h>
@@ -41,7 +42,7 @@
 #include "kudu/master/master.pb.h"
 #include "kudu/master/mini_master.h"
 #include "kudu/master/sys_catalog.h"
-#include "kudu/master/ts_descriptor.h"
+#include "kudu/master/ts_descriptor.h"  // IWYU pragma: keep
 #include "kudu/mini-cluster/internal_mini_cluster.h"
 #include "kudu/security/test/test_certs.h"
 #include "kudu/security/tls_context.h"
@@ -67,7 +68,6 @@ DECLARE_int32(heartbeat_interval_ms);
 METRIC_DECLARE_counter(rows_inserted);
 METRIC_DECLARE_counter(rows_updated);
 
-using boost::none;
 using kudu::cluster::InternalMiniCluster;
 using kudu::master::CatalogManager;
 using kudu::master::CreateTableRequestPB;
@@ -80,6 +80,7 @@ using kudu::master::TSDescriptor;
 using kudu::master::TabletLocationsPB;
 using kudu::pb_util::SecureShortDebugString;
 using kudu::tserver::MiniTabletServer;
+using std::nullopt;
 using std::shared_ptr;
 using std::string;
 using std::unique_ptr;
@@ -116,7 +117,7 @@ void CreateTableForTesting(MiniMaster* mini_master,
     {
       CatalogManager::ScopedLeaderSharedLock l(catalog);
       ASSERT_OK(l.first_failed_status());
-      ASSERT_OK(catalog->IsCreateTableDone(&req, &resp, /*user=*/none));
+      ASSERT_OK(catalog->IsCreateTableDone(&req, &resp, /*user=*/nullopt));
     }
     if (resp.done()) {
       is_table_created = true;
@@ -203,9 +204,10 @@ class RegistrationTest : public KuduTest {
         RETURN_NOT_OK(ls);
         s = catalog->GetTabletLocations(tablet_id,
                                         master::VOTER_REPLICA,
+                                        /*use_external_addr=*/false,
                                         &loc,
                                         &infos_dict,
-                                        /*user=*/none);
+                                        /*user=*/nullopt);
       } while (false);
       if (s.ok() && loc.interned_replicas_size() == expected_count) {
         if (locations) {
@@ -232,7 +234,7 @@ TEST_F(RegistrationTest, TestTSRegisters) {
 
   // Verify that the registration is sane.
   ServerRegistrationPB reg;
-  descs[0]->GetRegistration(&reg);
+  ASSERT_OK(descs[0]->GetRegistration(&reg));
   {
     SCOPED_TRACE(SecureShortDebugString(reg));
     ASSERT_EQ(string::npos, SecureShortDebugString(reg).find("0.0.0.0"))
@@ -266,12 +268,12 @@ TEST_F(RegistrationTest, TestTSRegisters) {
 TEST_F(RegistrationTest, TestMasterSoftwareVersion) {
   // Verify that the master's software version exists.
   ServerRegistrationPB reg;
-  cluster_->mini_master()->master()->GetMasterRegistration(&reg);
+  cluster_->mini_master()->master()->GetMasterRegistration(
+      &reg, /*use_external_addr=*/false);
   {
     SCOPED_TRACE(SecureShortDebugString(reg));
     ASSERT_TRUE(reg.has_software_version());
-    ASSERT_STR_CONTAINS(reg.software_version(),
-                        VersionInfo::GetVersionInfo());
+    ASSERT_STR_CONTAINS(reg.software_version(), VersionInfo::GetVersionInfo());
     ASSERT_LE(setup_time_, reg.start_time());
     ASSERT_LE(reg.start_time(), WallTime_Now());
   }
@@ -279,7 +281,8 @@ TEST_F(RegistrationTest, TestMasterSoftwareVersion) {
 
 TEST_F(RegistrationTest, TestServerStartWallTime) {
   ServerRegistrationPB reg;
-  cluster_->mini_master()->master()->GetMasterRegistration(&reg);
+  cluster_->mini_master()->master()->GetMasterRegistration(
+      &reg, /*use_external_addr=*/false);
   ASSERT_LE(setup_time_, reg.start_time());
   ASSERT_LE(reg.start_time(), WallTime_Now());
 
