@@ -26,6 +26,9 @@
 #include <ostream>
 #include <string>
 #include <tuple>
+#include <type_traits>
+#include <unordered_map>
+#include <utility>
 #include <vector>
 
 #include <gflags/gflags_declare.h>
@@ -49,10 +52,13 @@
 #include "kudu/master/master.proxy.h"
 #include "kudu/master/sys_catalog.h"
 #include "kudu/mini-cluster/external_mini_cluster.h"
+#include "kudu/ranger-kms/mini_ranger_kms.h"
 #include "kudu/rpc/messenger.h"
 #include "kudu/rpc/rpc_controller.h"
+#include "kudu/security/cert.h"
 #include "kudu/security/kinit_context.h"
 #include "kudu/security/test/mini_kdc.h"
+#include "kudu/security/test/test_certs.h"
 #include "kudu/security/token.pb.h"
 #include "kudu/server/server_base.pb.h"
 #include "kudu/server/server_base.proxy.h"
@@ -61,10 +67,14 @@
 #include "kudu/tserver/tserver.pb.h"
 #include "kudu/tserver/tserver_service.pb.h"
 #include "kudu/tserver/tserver_service.proxy.h"
+#include "kudu/util/curl_util.h"
 #include "kudu/util/env.h"
+#include "kudu/util/faststring.h"
+#include "kudu/util/mini_oidc.h"
 #include "kudu/util/monotime.h"
 #include "kudu/util/net/net_util.h"
 #include "kudu/util/net/sockaddr.h"
+#include "kudu/util/openssl_util.h"
 #include "kudu/util/path_util.h"
 #include "kudu/util/random.h"
 #include "kudu/util/random_util.h"
@@ -88,6 +98,8 @@ using kudu::client::sp::shared_ptr;
 using kudu::cluster::ExternalMiniCluster;
 using kudu::cluster::ExternalMiniClusterOptions;
 using kudu::rpc::Messenger;
+using kudu::security::CreateTestSSLCertWithChainSignedByRoot;
+using kudu::security::CreateTestSSLExpiredCertWithChainSignedByRoot;
 using std::get;
 using std::string;
 using std::tuple;
@@ -191,8 +203,25 @@ class SecurityITest : public KuduTest {
     return proxy.Checksum(req, &resp, &rpc);
   }
 
+  // Retrieve the cluster's IPKI certificate. Effectively, this waits until
+  // the catalog manager is initialized and has the IPKI CA ready to process
+  // requests to the /ipki-ca-cert HTTP endpoint. This also allows to have
+  // the master's TLS certificate used for RPC signed with the IPKI CA,
+  // so it will list the JWT authentication mechanism as available.
+  Status FetchClusterCACert(string* ca_cert_pem) {
+    int leader_idx;
+    RETURN_NOT_OK(cluster_->GetLeaderMasterIndex(&leader_idx));
+    const auto& http_hp = cluster_->master(leader_idx)->bound_http_hostport();
+    string url = Substitute("http://$0/ipki-ca-cert", http_hp.ToString());
+    EasyCurl curl;
+    faststring dst;
+    auto res = curl.FetchURL(url, &dst);
+    *ca_cert_pem = dst.ToString();
+    return res;
+  }
+
  private:
-  std::shared_ptr<Messenger> NewMessengerOrDie() {
+  static std::shared_ptr<Messenger> NewMessengerOrDie() {
     std::shared_ptr<Messenger> messenger;
     CHECK_OK(rpc::MessengerBuilder("test-messenger")
              .set_num_reactors(1)
@@ -507,6 +536,271 @@ void GetFullBinaryPath(string* binary) {
   (*binary) = JoinPathSegments(DirName(exe), *binary);
 }
 
+TEST_F(SecurityITest, TestJwtMiniCluster) {
+  SKIP_IF_SLOW_NOT_ALLOWED();
+
+  cluster_opts_.enable_kerberos = false;
+  cluster_opts_.num_tablet_servers = 0;
+  cluster_opts_.enable_client_jwt = true;
+
+  MiniOidcOptions oidc_opts;
+  const auto* const kValidAccount = "valid";
+  const auto* const kInvalidAccount = "invalid";
+  const uint64_t kLifetimeMs = 5000;
+  oidc_opts.account_ids = {
+    { kValidAccount, true },
+    { kInvalidAccount, false },
+  };
+  oidc_opts.lifetime_ms = kLifetimeMs;
+
+  // Set up certificates for the JWKS server
+  string ca_certificate_file;
+  string private_key_file;
+  string certificate_file;
+  ASSERT_OK(CreateTestSSLCertWithChainSignedByRoot(GetTestDataDirectory(),
+                                                   &certificate_file,
+                                                   &private_key_file,
+                                                   &ca_certificate_file));
+  // set the certs and private key for the jwks webserver
+  oidc_opts.private_key_file = private_key_file;
+  oidc_opts.server_certificate = certificate_file;
+  // set the ca_cert (jwks certificate verification is enabled by default)
+  cluster_opts_.extra_master_flags.push_back(Substitute("--trusted_certificate_file=$0",
+                                                        ca_certificate_file));
+
+  cluster_opts_.mini_oidc_options = std::move(oidc_opts);
+  ASSERT_OK(StartCluster());
+
+  string cluster_cert_pem;
+  ASSERT_EVENTUALLY([&] {
+    ASSERT_OK(FetchClusterCACert(&cluster_cert_pem));
+  });
+  ASSERT_FALSE(cluster_cert_pem.empty());
+
+  const auto* const kSubject = "kudu-user";
+  const auto configure_builder_for =
+      [&] (const string& account_id, KuduClientBuilder* b, const uint64_t delay_ms, bool is_valid) {
+    for (auto i = 0; i < cluster_->num_masters(); ++i) {
+      b->add_master_server_addr(cluster_->master(i)->bound_rpc_addr().ToString());
+    }
+    b->jwt(cluster_->oidc()->CreateJwt(account_id, kSubject, is_valid));
+    b->require_authentication(true);
+    b->trusted_certificate(cluster_cert_pem);
+    SleepFor(MonoDelta::FromMilliseconds(delay_ms));
+  };
+
+  {
+    SCOPED_TRACE("Valid JWT");
+    KuduClientBuilder valid_builder;
+    shared_ptr<KuduClient> client;
+    configure_builder_for(kValidAccount, &valid_builder, 0, true);
+    ASSERT_OK(valid_builder.Build(&client));
+    vector<string> tables;
+    ASSERT_OK(client->ListTables(&tables));
+  }
+  {
+    SCOPED_TRACE("Invalid JWT");
+    KuduClientBuilder invalid_builder;
+    shared_ptr<KuduClient> client;
+    configure_builder_for(kInvalidAccount, &invalid_builder, 0, false);
+    Status s = invalid_builder.Build(&client);
+    ASSERT_TRUE(s.IsRuntimeError()) << s.ToString();
+    ASSERT_STR_CONTAINS(s.ToString(), "FATAL_INVALID_JWT");
+  }
+  {
+    SCOPED_TRACE("Expired JWT");
+    KuduClientBuilder timeout_builder;
+    shared_ptr<KuduClient> client;
+    configure_builder_for(kValidAccount, &timeout_builder, 3 * kLifetimeMs, true);
+    Status s = timeout_builder.Build(&client);
+    ASSERT_TRUE(s.IsRuntimeError()) << s.ToString();
+    ASSERT_STR_CONTAINS(s.ToString(), "token expired");
+  }
+  {
+    SCOPED_TRACE("No JWT provided");
+    KuduClientBuilder no_jwt_builder;
+    shared_ptr<KuduClient> client;
+    for (auto i = 0; i < cluster_->num_masters(); ++i) {
+      no_jwt_builder.add_master_server_addr(cluster_->master(i)->bound_rpc_addr().ToString());
+    }
+    no_jwt_builder.require_authentication(true);
+    Status s = no_jwt_builder.Build(&client);
+    ASSERT_TRUE(s. IsNotAuthorized()) << s.ToString();
+    ASSERT_STR_CONTAINS(s.ToString(), "Not authorized");
+  }
+  {
+    SCOPED_TRACE("Valid JWT but client does not trust master's TLS cert");
+    KuduClientBuilder cb;
+    for (auto i = 0; i < cluster_->num_masters(); ++i) {
+      cb.add_master_server_addr(cluster_->master(i)->bound_rpc_addr().ToString());
+    }
+    cb.jwt(cluster_->oidc()->CreateJwt(kValidAccount, kSubject, true));
+    cb.require_authentication(true);
+
+    shared_ptr<KuduClient> client;
+    auto s = cb.Build(&client);
+    ASSERT_TRUE(s. IsNotAuthorized()) << s.ToString();
+    ASSERT_STR_CONTAINS(s.ToString(),
+        "client requires authentication, but server does not have");
+  }
+}
+
+TEST_F(SecurityITest, TestJwtMiniClusterWithInvalidCert) {
+  cluster_opts_.enable_kerberos = false;
+  cluster_opts_.num_tablet_servers = 0;
+  cluster_opts_.enable_client_jwt = true;
+  MiniOidcOptions oidc_opts;
+  const auto* const kValidAccount = "valid";
+  const uint64_t kLifetimeMs = 5000;
+  oidc_opts.account_ids = {
+    { kValidAccount, true }
+  };
+  oidc_opts.lifetime_ms = kLifetimeMs;
+  const auto* const kSubject = "kudu-user";
+
+  // Set up certificates for the JWKS server
+  string ca_certificate_file;
+  string certificate_file;
+  string private_key_file;
+  ASSERT_OK(CreateTestSSLExpiredCertWithChainSignedByRoot(
+      GetTestDataDirectory(),
+      &certificate_file,
+      &private_key_file,
+      &ca_certificate_file));
+
+  // set the certs and private key for the jwks webserver
+  oidc_opts.private_key_file = private_key_file;
+  oidc_opts.server_certificate = certificate_file;
+  // set the ca_cert (jwks certificate verification is enabled by default)
+  cluster_opts_.extra_master_flags.push_back(Substitute("--trusted_certificate_file=$0",
+                                                        ca_certificate_file));
+
+  cluster_opts_.mini_oidc_options = std::move(oidc_opts);
+
+  ASSERT_OK(StartCluster());
+
+  string cluster_cert_pem;
+  ASSERT_EVENTUALLY([&] {
+    ASSERT_OK(FetchClusterCACert(&cluster_cert_pem));
+  });
+  ASSERT_FALSE(cluster_cert_pem.empty());
+
+  KuduClientBuilder client_builder;
+  for (auto i = 0; i < cluster_->num_masters(); ++i) {
+    client_builder.add_master_server_addr(cluster_->master(i)->bound_rpc_addr().ToString());
+  }
+  client_builder.jwt(cluster_->oidc()->CreateJwt(kValidAccount, kSubject, true));
+  client_builder.trusted_certificate(cluster_cert_pem);
+  client_builder.require_authentication(true);
+
+  shared_ptr<KuduClient> client;
+  auto s = client_builder.Build(&client);
+  ASSERT_TRUE(s.IsRuntimeError()) << s.ToString();
+  ASSERT_STR_CONTAINS(s.ToString(), "Not authorized: Failed to load JWKS");
+}
+
+TEST_F(SecurityITest, TestJwtMiniClusterWithUntrustedCert) {
+  cluster_opts_.enable_kerberos = false;
+  cluster_opts_.num_tablet_servers = 0;
+  cluster_opts_.enable_client_jwt = true;
+  MiniOidcOptions oidc_opts;
+  const auto* const kValidAccount = "valid";
+  const uint64_t kLifetimeMs = 5000;
+  oidc_opts.account_ids = {
+    { kValidAccount, true }
+  };
+  oidc_opts.lifetime_ms = kLifetimeMs;
+  const auto* const kSubject = "kudu-user";
+
+  // Set up certificates for the JWKS server
+  string ca_certificate_file;
+  string private_key_file;
+  string certificate_file;
+  ASSERT_OK(CreateTestSSLCertWithChainSignedByRoot(GetTestDataDirectory(),
+                                                   &certificate_file,
+                                                   &private_key_file,
+                                                   &ca_certificate_file));
+  // set the certs and private key for the jwks webserver
+  // jwks certificate verification is enabled by default, so we won't have to set it
+  oidc_opts.private_key_file = private_key_file;
+  oidc_opts.server_certificate = certificate_file;
+
+  cluster_opts_.mini_oidc_options = std::move(oidc_opts);
+
+  ASSERT_OK(StartCluster());
+
+  string cluster_cert_pem;
+  ASSERT_EVENTUALLY([&] {
+    ASSERT_OK(FetchClusterCACert(&cluster_cert_pem));
+  });
+  ASSERT_FALSE(cluster_cert_pem.empty());
+
+  KuduClientBuilder client_builder;
+  for (auto i = 0; i < cluster_->num_masters(); ++i) {
+    client_builder.add_master_server_addr(cluster_->master(i)->bound_rpc_addr().ToString());
+  }
+  client_builder.jwt(cluster_->oidc()->CreateJwt(kValidAccount, kSubject, true));
+  client_builder.trusted_certificate(cluster_cert_pem);
+  client_builder.require_authentication(true);
+
+  shared_ptr<KuduClient> client;
+  auto s = client_builder.Build(&client);
+  ASSERT_TRUE(s.IsRuntimeError()) << s.ToString();
+  ASSERT_STR_CONTAINS(s.ToString(), "Not authorized: Failed to load JWKS");
+}
+
+TEST_F(SecurityITest, TestJwtMiniClusterWithNonWorkingJWKS) {
+  cluster_opts_.enable_kerberos = false;
+  cluster_opts_.num_tablet_servers = 0;
+  cluster_opts_.start_jwks = false;
+  cluster_opts_.enable_client_jwt = true;
+
+  MiniOidcOptions oidc_opts;
+  const auto* const kValidAccount = "valid";
+  const auto* const kSubject = "kudu-user";
+  oidc_opts.account_ids = {
+    { kValidAccount, true }
+  };
+
+  // Set up certificates for the JWKS server
+  string ca_certificate_file;
+  string private_key_file;
+  string certificate_file;
+  ASSERT_OK(CreateTestSSLCertWithChainSignedByRoot(GetTestDataDirectory(),
+                                                   &certificate_file,
+                                                   &private_key_file,
+                                                   &ca_certificate_file));
+  // set the certs and private key for the jwks webserver
+  oidc_opts.private_key_file = private_key_file;
+  oidc_opts.server_certificate = certificate_file;
+  // set the ca_cert (jwks certificate verification is enabled by default)
+  cluster_opts_.extra_master_flags.push_back(Substitute("--trusted_certificate_file=$0",
+                                                        ca_certificate_file));
+
+  cluster_opts_.mini_oidc_options = std::move(oidc_opts);
+  ASSERT_OK(StartCluster());
+
+  string cluster_cert_pem;
+  ASSERT_EVENTUALLY([&] {
+    ASSERT_OK(FetchClusterCACert(&cluster_cert_pem));
+  });
+  ASSERT_FALSE(cluster_cert_pem.empty());
+
+  KuduClientBuilder client_builder;
+  for (auto i = 0; i < cluster_->num_masters(); ++i) {
+    client_builder.add_master_server_addr(cluster_->master(i)->bound_rpc_addr().ToString());
+  }
+  client_builder.jwt(cluster_->oidc()->CreateJwt(kValidAccount, kSubject, true));
+  client_builder.trusted_certificate(cluster_cert_pem);
+  client_builder.require_authentication(true);
+
+  shared_ptr<KuduClient> client;
+  auto s = client_builder.Build(&client);
+  ASSERT_TRUE(s.IsRuntimeError()) << s.ToString();
+  ASSERT_STR_CONTAINS(s.ToString(), "Not authorized: Failed to load JWKS");
+}
+
+
 TEST_F(SecurityITest, TestWorldReadableKeytab) {
   const string credentials_name = GetTestPath("insecure.keytab");
   NO_FATALS(CreateWorldReadableFile(credentials_name));
@@ -603,7 +897,7 @@ TEST_F(SecurityITest, TestNonDefaultPrincipal) {
       builder.add_master_server_addr(cluster_->master(i)->bound_rpc_addr().ToString());
     }
     const auto s = builder.Build(&client);
-    ASSERT_TRUE(s.IsNotAuthorized());
+    ASSERT_TRUE(s.IsNotAuthorized()) << s.ToString();
     ASSERT_STR_CONTAINS(s.ToString(), "not found in Kerberos database");
 
     // Create a client with the matching SASL proto name and verify it's able to
@@ -622,8 +916,8 @@ TEST_F(SecurityITest, TestNonDefaultPrincipal) {
 TEST_F(SecurityITest, TestNonExistentPrincipal) {
   cluster_opts_.extra_master_flags.emplace_back("--principal=oryx");
   cluster_opts_.extra_tserver_flags.emplace_back("--principal=oryx");
-  Status s = StartCluster();
-  ASSERT_TRUE(s.IsRuntimeError());
+  auto s = StartCluster();
+  ASSERT_TRUE(s.IsRuntimeError()) << s.ToString();
   ASSERT_STR_CONTAINS(s.ToString(), "failed to start masters");
 }
 
@@ -640,7 +934,7 @@ TEST_F(SecurityITest, TestMismatchingPrincipals) {
   }
   cluster_->Shutdown();
   Status s = cluster_->Restart();
-  ASSERT_TRUE(s.IsTimedOut());
+  ASSERT_TRUE(s.IsTimedOut()) << s.ToString();
 }
 
 TEST_F(SecurityITest, TestRequireAuthenticationInsecureCluster) {
@@ -651,7 +945,7 @@ TEST_F(SecurityITest, TestRequireAuthenticationInsecureCluster) {
   KuduClientBuilder b;
   b.require_authentication(true);
   Status s = cluster_->CreateClient(&b, &client);
-  ASSERT_TRUE(s.IsNotAuthorized());
+  ASSERT_TRUE(s.IsNotAuthorized()) << s.ToString();
   ASSERT_STR_CONTAINS(s.ToString(),
                       "client requires authentication, but server does not have Kerberos enabled");
 }
@@ -668,7 +962,7 @@ TEST_F(SecurityITest, TestRequireEncryptionInsecureCluster) {
   KuduClientBuilder b;
   b.encryption_policy(client::KuduClientBuilder::REQUIRED);
   Status s = cluster_->CreateClient(&b, &client);
-  ASSERT_TRUE(s.IsNotAuthorized());
+  ASSERT_TRUE(s.IsNotAuthorized()) << s.ToString();
   ASSERT_STR_CONTAINS(s.ToString(), "server does not support required TLS encryption");
 }
 
@@ -682,6 +976,187 @@ TEST_F(SecurityITest, TestRequireAuthenticationSecureCluster) {
   SmokeTestCluster(client, /* transactional */ false);
 }
 
+TEST_F(SecurityITest, TestEncryptionWithKMSIntegration) {
+  cluster_opts_.enable_ranger = true;
+  cluster_opts_.enable_ranger_kms = true;
+  ASSERT_OK(StartCluster());
+
+  shared_ptr<KuduClient> client;
+  KuduClientBuilder b;
+  ASSERT_OK(cluster_->CreateClient(&b, &client));
+  SmokeTestCluster(client, /* transactional */ false);
+}
+
+TEST_F(SecurityITest, TestEncryptionWithKMSIntegrationMultipleServers) {
+  cluster_opts_.enable_ranger = true;
+  cluster_opts_.enable_ranger_kms = true;
+  ASSERT_OK(StartCluster());
+  cluster_->Shutdown();
+
+  const string& url = cluster_->ranger_kms()->url();
+  for (int i = 0; i < cluster_->num_masters(); ++i) {
+    cluster_->master(i)->mutable_flags()
+        ->emplace_back("--ranger_kms_url=invalid.host:1234/kms," + url);
+  }
+  for (int i = 0; i < cluster_->num_tablet_servers(); ++i) {
+    cluster_->tablet_server(i)->mutable_flags()
+        ->emplace_back("--ranger_kms_url=invalid.host:1234/kms," + url);
+  }
+  ASSERT_OK(cluster_->Restart());
+
+  shared_ptr<KuduClient> client;
+  KuduClientBuilder b;
+  ASSERT_OK(cluster_->CreateClient(&b, &client));
+  SmokeTestCluster(client, /*transactional=*/false);
+}
+
+TEST_F(SecurityITest, IPKICACert) {
+  SKIP_IF_SLOW_NOT_ALLOWED();
+
+  // Need to test the functionality for both leader and follower masters.
+  cluster_opts_.num_masters = 3;
+  // No need to involve tablet servers in this scenario.
+  cluster_opts_.num_tablet_servers = 0;
+
+  ASSERT_OK(StartCluster());
+
+  shared_ptr<KuduClient> client;
+  ASSERT_OK(cluster_->CreateClient(nullptr, &client));
+
+  string authn_creds;
+  ASSERT_OK(client->ExportAuthenticationCredentials(&authn_creds));
+  client::AuthenticationCredentialsPB pb;
+  ASSERT_TRUE(pb.ParseFromString(authn_creds));
+  ASSERT_EQ(1, pb.ca_cert_ders_size());
+
+  security::Cert ca_cert_client;
+  ASSERT_OK(ca_cert_client.FromString(pb.ca_cert_ders(0),
+                                      security::DataFormat::DER));
+  string ca_cert_client_pem;
+  ASSERT_OK(ca_cert_client.ToString(&ca_cert_client_pem,
+                                    security::DataFormat::PEM));
+
+  const auto fetch_ipki_ca = [c = cluster_.get()](int master_idx, string* out) {
+    const auto& http_hp = c->master(master_idx)->bound_http_hostport();
+    string url = Substitute("http://$0/ipki-ca-cert", http_hp.ToString());
+    EasyCurl curl;
+    faststring dst;
+    auto res = curl.FetchURL(url, &dst);
+    *out = dst.ToString();
+    return res;
+  };
+
+  int leader_master_idx;
+  ASSERT_OK(cluster_->GetLeaderMasterIndex(&leader_master_idx));
+  string str;
+  ASSERT_OK(fetch_ipki_ca(leader_master_idx, &str));
+  security::Cert ca_cert;
+  ASSERT_OK(ca_cert.FromString(str, security::DataFormat::PEM));
+
+  // Using (string --> security::Cert --> string) conversion chain to compare
+  // canonical representations of the CA certificates in PEM format.
+  string ca_cert_str;
+  ASSERT_OK(ca_cert.ToString(&ca_cert_str, security::DataFormat::PEM));
+  ASSERT_EQ(ca_cert_client_pem, ca_cert_str);
+
+  const auto count_valid_certs = [&](size_t* res) {
+    size_t count = 0;
+    for (auto i = 0; i < cluster_->num_masters(); ++i) {
+      string str;
+      auto s = fetch_ipki_ca(i, &str);
+      ASSERT_TRUE(s.ok() || s.IsRemoteError());
+      if (s.IsRemoteError()) {
+        // If there wasn't a CA cert in the output, there should had been
+        // an error reported.
+        ASSERT_NE(string::npos, str.find("ERROR: "));
+        continue;
+      }
+      security::Cert ca_cert;
+      ASSERT_OK(ca_cert.FromString(str, security::DataFormat::PEM));
+
+      string ca_cert_str;
+      ASSERT_OK(ca_cert.ToString(&ca_cert_str, security::DataFormat::PEM));
+      ASSERT_EQ(ca_cert_client_pem, ca_cert_str);
+      ++count;
+    }
+    *res = count;
+  };
+
+  // The IPKI has been certainly initialized at leader master since the client
+  // was able to successfully connect to the cluster (see above).
+  {
+    size_t count = 0;
+    NO_FATALS(count_valid_certs(&count));
+    ASSERT_GE(count, 1);
+  }
+
+  // At some point, all the followers should have loaded the CA information
+  // generated by the leader master upon the very first startup.
+  ASSERT_EVENTUALLY([&] {
+    size_t count = 0;
+    NO_FATALS(count_valid_certs(&count));
+    ASSERT_EQ(cluster_opts_.num_masters, count);
+  });
+}
+
+// Test the endpoints of the embedded master's webserver that output the IPKI
+// CA certificates in PEM and DER formats, /ipki-ca-cert[-pem] and
+// ipki-ca-cert-der, correspondingly. Make sure the end-points are functional
+// and their output is consistent.
+TEST_F(SecurityITest, IPKICACertWebServerEndPoints) {
+  // Need just a single master.
+  cluster_opts_.num_masters = 1;
+  // No need to involve tablet servers in this scenario.
+  cluster_opts_.num_tablet_servers = 0;
+
+  ASSERT_OK(StartCluster());
+
+  const auto fetch = [c = cluster_.get()](const string& path, string* out) {
+    const auto& http_hp = c->master()->bound_http_hostport();
+    string url = Substitute("http://$0/$1", http_hp.ToString(), path);
+    EasyCurl curl;
+    faststring dst;
+    auto res = curl.FetchURL(url, &dst);
+    *out = dst.ToString();
+    return res;
+  };
+
+  ASSERT_OK(cluster_->master()->WaitForCatalogManager());
+
+  security::Cert ca_cert_0;
+  {
+    string ca_cert_str_pem;
+    ASSERT_OK(fetch("ipki-ca-cert", &ca_cert_str_pem));
+    ASSERT_OK(ca_cert_0.FromString(ca_cert_str_pem, security::DataFormat::PEM));
+  }
+
+  security::Cert ca_cert_1;
+  {
+    string ca_cert_str_pem;
+    ASSERT_OK(fetch("ipki-ca-cert-pem", &ca_cert_str_pem));
+    ASSERT_OK(ca_cert_1.FromString(ca_cert_str_pem, security::DataFormat::PEM));
+  }
+
+  security::Cert ca_cert_2;
+  {
+    string ca_cert_str_der;
+    ASSERT_OK(fetch("ipki-ca-cert-der", &ca_cert_str_der));
+    ASSERT_OK(ca_cert_2.FromString(ca_cert_str_der, security::DataFormat::DER));
+  }
+
+  // Using (security::Cert --> string) conversion to compare canonical string
+  // representations of the captured CA certificates.
+  string ca_cert_str_0;
+  ASSERT_OK(ca_cert_0.ToString(&ca_cert_str_0, security::DataFormat::PEM));
+  string ca_cert_str_1;
+  ASSERT_OK(ca_cert_1.ToString(&ca_cert_str_1, security::DataFormat::PEM));
+  string ca_cert_str_2;
+  ASSERT_OK(ca_cert_2.ToString(&ca_cert_str_2, security::DataFormat::PEM));
+
+  ASSERT_EQ(ca_cert_str_0, ca_cert_str_1);
+  ASSERT_EQ(ca_cert_str_1, ca_cert_str_2);
+}
+
 class EncryptionPolicyTest :
     public SecurityITest,
     public ::testing::WithParamInterface<tuple<
