@@ -19,11 +19,11 @@
 #include <cstdint>
 #include <cstdlib>
 #include <memory>
+#include <optional>
 #include <string>
 #include <thread>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <gflags/gflags_declare.h>
 #include <glog/logging.h>
 #include <gtest/gtest.h>
@@ -141,8 +141,8 @@ class SecurityUnknownTskTest : public KuduTest {
                             const TokenSigningPrivateKeyPB& tsk,
                             SignedTokenPB* new_signed_token) {
     // Should be already connected to the cluster.
-    boost::optional<SignedTokenPB> authn_token = client->data_->messenger_->authn_token();
-    if (authn_token == boost::none) {
+    auto authn_token = client->data_->messenger_->authn_token();
+    if (!authn_token) {
       return Status::RuntimeError("client authn token is not set");
     }
 
