@@ -22,15 +22,16 @@
 #include <limits>
 #include <memory>
 #include <mutex>
+#include <optional>
 #include <ostream>
 #include <set>
 #include <string>
 #include <thread>
+#include <type_traits>
 #include <unordered_map>
 #include <utility>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <gflags/gflags.h>
 #include <glog/logging.h>
 #include <gtest/gtest.h>
@@ -97,7 +98,6 @@ DEFINE_int32(test_delete_leader_num_writer_threads, 1,
 DECLARE_bool(tablet_copy_download_wal_inject_latency);
 DECLARE_int32(log_segment_size_mb);
 
-using boost::none;
 using kudu::client::KuduScanner;
 using kudu::client::KuduScanBatch;
 using kudu::client::KuduSchema;
@@ -129,7 +129,7 @@ using kudu::tablet::TabletDataState;
 using kudu::tablet::TabletSuperBlockPB;
 using kudu::tserver::ListTabletsResponsePB;
 using kudu::tserver::ListTabletsResponsePB_StatusAndSchemaPB;
-using kudu::tserver::TabletCopyClient;
+using kudu::tserver::RemoteTabletCopyClient;
 using std::atomic;
 using std::lock_guard;
 using std::mutex;
@@ -513,12 +513,11 @@ TEST_F(TabletCopyITest, TestDeleteTabletDuringTabletCopy) {
 
   {
     // Start up a TabletCopyClient and open a tablet copy session.
-    TabletCopyClient tc_client(tablet_id, fs_manager.get(),
-                               cmeta_manager, cluster_->messenger(),
-                               nullptr /* no metrics */);
+    RemoteTabletCopyClient tc_client(tablet_id, fs_manager.get(),
+                                     cmeta_manager, cluster_->messenger(),
+                                     nullptr /* no metrics */);
     scoped_refptr<tablet::TabletMetadata> meta;
-    ASSERT_OK(tc_client.Start(cluster_->tablet_server(kTsIndex)->bound_rpc_hostport(),
-                              &meta));
+    ASSERT_OK(tc_client.Start(cluster_->tablet_server(kTsIndex)->bound_rpc_hostport(), &meta));
 
     // Tombstone the tablet on the remote!
     ASSERT_OK(DeleteTablet(ts, tablet_id, TABLET_DATA_TOMBSTONED, timeout));
@@ -985,6 +984,47 @@ TEST_F(TabletCopyITest, TestSlowCopyDoesntFail) {
                             workload.rows_inserted()));
 }
 
+TEST_F(TabletCopyITest, TestTabletCopyEncryptedServers) {
+  SetEncryptionFlags(true);
+  ExternalMiniClusterOptions opts;
+  opts.num_tablet_servers = 3;
+  NO_FATALS(StartClusterWithOpts(opts));
+
+  MonoDelta timeout = MonoDelta::FromSeconds(30);
+
+  TestWorkload workload(cluster_.get());
+  workload.Setup();
+  workload.Start();
+  while (workload.rows_inserted() < 1000) {
+    SleepFor(MonoDelta::FromMilliseconds(10));
+  }
+
+  // Figure out the tablet id of the created tablet.
+  vector<ListTabletsResponsePB::StatusAndSchemaPB> tablets;
+  ExternalTabletServer* replica_ets = cluster_->tablet_server(2);
+  TServerDetails* replica_ts = ts_map_[replica_ets->uuid()];
+  ASSERT_OK(WaitForNumTabletsOnTS(replica_ts, 1, timeout, &tablets));
+  const auto& tablet_id = tablets[0].tablet_status().tablet_id();
+
+  // Tombstone the follower.
+  LOG(INFO) << "Tombstoning follower tablet " << tablet_id << " on TS " << replica_ts->uuid();
+  ASSERT_OK(DeleteTablet(replica_ts, tablet_id, TABLET_DATA_TOMBSTONED, timeout));
+
+  // Wait for tablet copy to start. The copying might complete fast or there
+  // might be some scheduler anomalies, so here it's necessary to count in
+  // TABLET_DATA_COPYING --> TABLET_DATA_READY transitions as well.
+  ASSERT_OK(inspect_->WaitForTabletDataStateOnTS(
+      2, tablet_id, { tablet::TABLET_DATA_COPYING, tablet::TABLET_DATA_READY }, timeout));
+
+  workload.StopAndJoin();
+  ASSERT_OK(WaitForServersToAgree(timeout, ts_map_, tablet_id, 1));
+
+  ClusterVerifier v(cluster_.get());
+  NO_FATALS(v.CheckCluster());
+  NO_FATALS(v.CheckRowCount(workload.table_name(), ClusterVerifier::AT_LEAST,
+                            workload.rows_inserted()));
+}
+
 // Attempting to start Tablet Copy on a tablet that was deleted with
 // TABLET_DATA_DELETED should fail. This behavior helps avoid thrashing when
 // a follower tablet is deleted and the leader notices before it has processed
@@ -1248,8 +1288,11 @@ TEST_P(TabletCopyFailureITest, TestTabletCopyNewReplicaFailureCanVote) {
 
   ASSERT_OK(inspect_->WaitForReplicaCount(kNumReplicas));
   master::GetTableLocationsResponsePB table_locations;
-  ASSERT_OK(itest::GetTableLocations(cluster_->master_proxy(), TestWorkload::kDefaultTableName,
-                                     kTimeout, master::VOTER_REPLICA, /*table_id=*/none,
+  ASSERT_OK(itest::GetTableLocations(cluster_->master_proxy(),
+                                     TestWorkload::kDefaultTableName,
+                                     kTimeout,
+                                     master::VOTER_REPLICA,
+                                     /*table_id=*/std::nullopt,
                                      &table_locations));
   ASSERT_EQ(1, table_locations.tablet_locations_size());
   string tablet_id = table_locations.tablet_locations(0).tablet_id();
