@@ -20,15 +20,16 @@
 #include <atomic>
 #include <functional>
 #include <memory>
+#include <optional>
 #include <ostream>
 #include <random>
 #include <string>
+#include <type_traits>
 #include <unordered_map>
 #include <unordered_set>
 #include <utility>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <gflags/gflags.h>
 #include <glog/logging.h>
 
@@ -94,6 +95,8 @@ using kudu::rpc::MessengerBuilder;
 using kudu::rpc::RpcController;
 using strings::Substitute;
 
+using std::nullopt;
+using std::optional;
 using std::shared_ptr;
 using std::string;
 using std::unordered_map;
@@ -133,6 +136,8 @@ DEFINE_uint32(auto_rebalancing_wait_for_replica_moves_seconds, 1,
               "How long to wait before checking to see if the scheduled replica movement "
               "in this iteration of auto-rebalancing has completed.");
 
+DECLARE_bool(auto_rebalancing_enabled);
+
 namespace kudu {
 
 namespace master {
@@ -188,7 +193,12 @@ void AutoRebalancerTask::RunLoop() {
   vector<Rebalancer::ReplicaMove> replica_moves;
   while (!shutdown_.WaitFor(
       MonoDelta::FromSeconds(FLAGS_auto_rebalancing_interval_seconds))) {
-
+    if (!FLAGS_auto_rebalancing_enabled) {
+      // Toggling the auto-rebalancer on/off by changing FLAGS_auto_rebalancing_enabled,
+      // will take effect in the next loop. Already scheduled/running replica moves will
+      // be unaffected.
+      continue;
+    }
     // If catalog manager isn't initialized or isn't the leader, don't do rebalancing.
     // Putting the auto-rebalancer to sleep shouldn't affect the master's ability
     // to become the leader. When the thread wakes up and discovers it is now
@@ -207,7 +217,7 @@ void AutoRebalancerTask::RunLoop() {
     ClusterRawInfo raw_info;
     ClusterInfo cluster_info;
     TabletsPlacementInfo placement_info;
-    Status s = BuildClusterRawInfo(/*location*/boost::none, &raw_info);
+    Status s = BuildClusterRawInfo(/*location*/nullopt, &raw_info);
     if (!s.ok()) {
       LOG(WARNING) << Substitute("Could not retrieve cluster info: $0", s.ToString());
       continue;
@@ -344,7 +354,7 @@ Status AutoRebalancerTask::GetMovesUsingRebalancingAlgo(
   vector<Rebalancer::ReplicaMove> rep_moves;
   for (const auto& move : moves) {
     vector<string> tablet_ids;
-    Rebalancer::FindReplicas(move, raw_info, &tablet_ids);
+    rebalancer_.FindReplicas(move, raw_info, &tablet_ids);
     if (cross_location == CrossLocations::YES) {
       // In case of cross-location (a.k.a. inter-location) rebalancing it is
       // necessary to make sure the majority of replicas would not end up
@@ -376,9 +386,10 @@ Status AutoRebalancerTask::GetTabletLeader(
     RETURN_NOT_OK(catalog_manager_->GetTabletLocations(
         tablet_id,
         ReplicaTypeFilter::VOTER_REPLICA,
+        /*use_external_addr=*/false,
         &locs_pb,
         &ts_infos_dict,
-        boost::none));
+        nullopt));
   }
   for (const auto& r : locs_pb.interned_replicas()) {
     if (r.role() == RaftPeerPB::LEADER) {
@@ -424,7 +435,7 @@ Status AutoRebalancerTask::ExecuteMoves(
         return Status::NotFound("Could not find destination tserver");
       }
       ServerRegistrationPB dest_reg;
-      dest_desc->GetRegistration(&dest_reg);
+      RETURN_NOT_OK(dest_desc->GetRegistration(&dest_reg));
 
       auto* add_peer_change = req.add_config_changes();
       add_peer_change->set_type(ADD_PEER);
@@ -451,7 +462,7 @@ Status AutoRebalancerTask::ExecuteMoves(
 }
 
 Status AutoRebalancerTask::BuildClusterRawInfo(
-    const boost::optional<string>& location,
+    const optional<string>& location,
     ClusterRawInfo* raw_info) const {
 
   vector<ServerHealthSummary> tserver_summaries;
@@ -488,7 +499,7 @@ Status AutoRebalancerTask::BuildClusterRawInfo(
   {
     CatalogManager::ScopedLeaderSharedLock leader_lock(catalog_manager_);
     RETURN_NOT_OK(leader_lock.first_failed_status());
-    RETURN_NOT_OK(catalog_manager_->GetAllTables(&table_infos));
+    catalog_manager_->GetAllTables(&table_infos);
   }
 
   table_summaries.reserve(table_infos.size());
@@ -531,9 +542,10 @@ Status AutoRebalancerTask::BuildClusterRawInfo(
         RETURN_NOT_OK(catalog_manager_->GetTabletLocations(
             tablet_summary.id,
             ReplicaTypeFilter::VOTER_REPLICA,
+            /*use_external_addr=*/false,
             &locs_pb,
             &ts_infos_dict,
-            boost::none));
+            nullopt));
       }
 
       // Consensus state information is the same for all replicas of this tablet.
