@@ -26,6 +26,7 @@
 #include <memory>
 #include <string>
 #include <thread>
+#include <type_traits>
 #include <unordered_set>
 #include <utility>
 
@@ -34,6 +35,11 @@
 
 #include "kudu/client/client.h"
 #include "kudu/client/master_rpc.h"
+#include "kudu/fs/default_key_provider.h"
+#include "kudu/fs/fs.pb.h"
+#include "kudu/fs/key_provider.h"
+#include "kudu/postgres/mini_postgres.h"
+#include "kudu/ranger-kms/mini_ranger_kms.h"
 #include "kudu/rpc/rpc_header.pb.h"
 #if !defined(NO_CHRONY)
 #include "kudu/clock/test/mini_chronyd.h"
@@ -41,6 +47,7 @@
 #include "kudu/common/wire_protocol.h"
 #include "kudu/common/wire_protocol.pb.h"
 #include "kudu/gutil/basictypes.h"
+#include "kudu/gutil/strings/escaping.h"
 #include "kudu/gutil/strings/join.h"
 #include "kudu/gutil/strings/stringpiece.h"
 #include "kudu/gutil/strings/substitute.h"
@@ -65,6 +72,8 @@
 #include "kudu/util/env.h"
 #include "kudu/util/env_util.h"
 #include "kudu/util/fault_injection.h"
+#include "kudu/util/jwt-util.h"
+#include "kudu/util/mini_oidc.h"
 #include "kudu/util/monotime.h"
 #include "kudu/util/net/sockaddr.h"
 #include "kudu/util/net/socket.h"
@@ -75,6 +84,10 @@
 #include "kudu/util/subprocess.h"
 #include "kudu/util/test_util.h"
 
+namespace kudu {
+class JwtVerifier;
+}  // namespace kudu
+
 using kudu::client::internal::ConnectToClusterRpc;
 #if !defined(NO_CHRONY)
 using kudu::clock::MiniChronyd;
@@ -85,11 +98,13 @@ using kudu::master::MasterServiceProxy;
 using kudu::pb_util::SecureDebugString;
 using kudu::pb_util::SecureShortDebugString;
 using kudu::rpc::RpcController;
+using kudu::security::DefaultKeyProvider;
 using kudu::server::ServerStatusPB;
 using kudu::tserver::ListTabletsRequestPB;
 using kudu::tserver::ListTabletsResponsePB;
 using kudu::tserver::TabletServerAdminServiceProxy;
 using kudu::tserver::TabletServerServiceProxy;
+using std::back_inserter;
 using std::copy;
 using std::map;
 using std::pair;
@@ -97,6 +112,7 @@ using std::string;
 using std::unique_ptr;
 using std::unordered_set;
 using std::vector;
+using strings::a2b_hex;
 using strings::Substitute;
 
 typedef ListTabletsResponsePB::StatusAndSchemaPB StatusAndSchemaPB;
@@ -125,17 +141,17 @@ ExternalMiniClusterOptions::ExternalMiniClusterOptions()
       principal("kudu"),
       hms_mode(HmsMode::NONE),
       enable_ranger(false),
+      enable_ranger_kms(false),
+      ranger_cluster_key("kuduclusterkey"),
       enable_encryption(FLAGS_encrypt_data_at_rest),
       logtostderr(true),
       start_process_timeout(MonoDelta::FromSeconds(70)),
-      rpc_negotiation_timeout(MonoDelta::FromSeconds(3))
+      rpc_negotiation_timeout(MonoDelta::FromSeconds(3)),
 #if !defined(NO_CHRONY)
-      ,
       num_ntp_servers(1),
-      ntp_config_mode(BuiltinNtpConfigMode::ALL_SERVERS)
+      ntp_config_mode(BuiltinNtpConfigMode::ALL_SERVERS),
 #endif // #if !defined(NO_CHRONY) ...
-{
-}
+      enable_client_jwt(false) {}
 
 ExternalMiniCluster::ExternalMiniCluster()
   : opts_(ExternalMiniClusterOptions()) {
@@ -153,6 +169,14 @@ Env* ExternalMiniCluster::env() const {
   return Env::Default();
 }
 
+Env* ExternalMiniCluster::ts_env(int ts_idx) const {
+  return tablet_server(ts_idx)->env();
+}
+
+Env* ExternalMiniCluster::master_env(int master_idx) const {
+  return master(master_idx)->env();
+}
+
 Status ExternalMiniCluster::DeduceBinRoot(std::string* ret) {
   string exe;
   RETURN_NOT_OK(env()->GetExecutablePath(&exe));
@@ -249,12 +273,28 @@ Status ExternalMiniCluster::Start() {
   gflags::FlagSaver saver;
   FLAGS_dns_addr_resolution_override = dns_overrides_;
 
+  std::shared_ptr<JwtVerifier> jwt_verifier = nullptr;
+  if (opts_.enable_client_jwt) {
+    oidc_.reset(new MiniOidc(opts_.mini_oidc_options));
+    string jwks_url = "default.url";
+    if (opts_.start_jwks) {
+      RETURN_NOT_OK_PREPEND(oidc_->Start(), "Failed to start OIDC endpoints");
+      jwks_url = oidc_->jwks_url();
+    }
+    jwt_verifier =
+        std::make_shared<KeyBasedJwtVerifier>(jwks_url,
+                                              /* jwks_verify_server_certificate */ true,
+                                              opts_.mini_oidc_options.server_certificate);
+
+  }
+
   RETURN_NOT_OK_PREPEND(
       rpc::MessengerBuilder("minicluster-messenger")
           .set_num_reactors(1)
           .set_max_negotiation_threads(1)
           .set_rpc_negotiation_timeout_ms(opts_.rpc_negotiation_timeout.ToMilliseconds())
           .set_sasl_proto_name(opts_.principal)
+          .set_jwt_verifier(std::move(jwt_verifier))
           .Build(&messenger_),
       "Failed to start Messenger for minicluster");
 
@@ -309,9 +349,12 @@ Status ExternalMiniCluster::Start() {
   }
 #endif // #if !defined(NO_CHRONY) ...
 
-  if (opts_.enable_ranger) {
+  if (opts_.enable_ranger || opts_.enable_ranger_kms) {
+    if (!postgres_ || !postgres_->IsStarted()) {
+      postgres_.reset(new postgres::MiniPostgres(cluster_root(), GetBindIpForExternalServer(0)));
+    }
     string host = GetBindIpForExternalServer(0);
-    ranger_.reset(new ranger::MiniRanger(cluster_root(), host));
+    ranger_.reset(new ranger::MiniRanger(cluster_root(), host, postgres_));
     if (opts_.enable_kerberos) {
 
       // The SPNs match the ones defined in mini_ranger_configs.h.
@@ -343,6 +386,32 @@ Status ExternalMiniCluster::Start() {
                           "Failed to write Ranger client config");
   }
 
+  if (opts_.enable_ranger_kms) {
+    string host = GetBindIpForExternalServer(0);
+    ranger_kms_.reset(new rangerkms::MiniRangerKMS(cluster_root(), host, postgres_, ranger_));
+    if (opts_.enable_kerberos) {
+      string keytab;
+      RETURN_NOT_OK_PREPEND(kdc_->CreateServiceKeytab(
+          Substitute("rangerkms/$0@KRBTEST.COM", host),
+          &keytab),
+        "could not create ranger kms keytab");
+      string spnego_keytab;
+      RETURN_NOT_OK_PREPEND(kdc_->CreateServiceKeytab(
+          Substitute("HTTP/$0@KRBTEST.COM", host),
+          &spnego_keytab),
+        "could not create ranger kms keytab");
+
+      ranger_kms_->EnableKerberos(kdc_->GetEnvVars()["KRB5_CONFIG"], keytab, spnego_keytab);
+    }
+    RETURN_NOT_OK(kdc_->CreateUserPrincipal("keyadmin"));
+    RETURN_NOT_OK(kdc_->Kinit("keyadmin"));
+    RETURN_NOT_OK_PREPEND(ranger_kms_->Start(), "Failed to start the Ranger KMS service");
+    RETURN_NOT_OK_PREPEND(ranger_kms_->CreateClusterKey(opts_.ranger_cluster_key,
+                                                        &opts_.ranger_cluster_key_version),
+                          "Failed to create cluster key");;
+    RETURN_NOT_OK(kdc_->Kinit("test-admin"));
+  }
+
   // Start the HMS.
   if (opts_.hms_mode == HmsMode::DISABLE_HIVE_METASTORE ||
       opts_.hms_mode == HmsMode::ENABLE_HIVE_METASTORE ||
@@ -462,12 +531,12 @@ string ExternalMiniCluster::GetLogPath(const string& daemon_id) const {
 }
 
 string ExternalMiniCluster::GetDataPath(const string& daemon_id,
-                                        boost::optional<uint32_t> dir_index) const {
+                                        std::optional<uint32_t> dir_index) const {
   CHECK(!opts_.cluster_root.empty());
   string data_path = "data";
   if (dir_index) {
     CHECK_LT(*dir_index, opts_.num_data_dirs);
-    data_path = Substitute("$0-$1", data_path, dir_index.get());
+    data_path = Substitute("$0-$1", data_path, *dir_index);
   } else {
     CHECK_EQ(1, opts_.num_data_dirs);
   }
@@ -537,6 +606,7 @@ Status ExternalMiniCluster::StartMasters() {
     scoped_refptr<ExternalMaster> peer;
     RETURN_NOT_OK(CreateMaster(master_rpc_addrs, i, &peer));
     RETURN_NOT_OK_PREPEND(peer->Start(), Substitute("Unable to start Master at index $0", i));
+    RETURN_NOT_OK(peer->SetServerKey());
     masters_.emplace_back(std::move(peer));
   }
   return Status::OK();
@@ -568,6 +638,11 @@ Status ExternalMiniCluster::AddTabletServer() {
   ExternalDaemonOptions opts;
   opts.messenger = messenger_;
   opts.enable_encryption = opts_.enable_encryption;
+  opts.enable_ranger_kms = opts_.enable_ranger_kms;
+  opts.ranger_cluster_key = opts_.ranger_cluster_key;
+  if (opts.enable_ranger_kms) {
+    opts.ranger_kms_url = ranger_kms_->url();
+  }
   opts.block_manager_type = opts_.block_manager_type;
   opts.exe = GetBinaryPath(kKuduBinaryName);
   opts.wal_dir = GetWalPath(daemon_id);
@@ -580,7 +655,16 @@ Status ExternalMiniCluster::AddTabletServer() {
   vector<string> extra_flags;
   RETURN_NOT_OK(AddTimeSourceFlags(idx, &extra_flags));
   auto flags = SubstituteInFlags(opts_.extra_tserver_flags, idx);
-  copy(flags.begin(), flags.end(), std::back_inserter(extra_flags));
+  copy(flags.begin(), flags.end(), back_inserter(extra_flags));
+
+  // Add custom tablet server flags, specific to this particular tablet server
+  // instance.
+  if (!opts_.t_custom_flags.empty()) {
+    CHECK_EQ(opts_.num_tablet_servers, opts_.t_custom_flags.size());
+    const auto& custom_flags = opts_.t_custom_flags[idx];
+    copy(custom_flags.begin(), custom_flags.end(), back_inserter(extra_flags));
+  }
+
   opts.extra_flags = extra_flags;
   if (!opts_.tserver_alias_prefix.empty()) {
     opts.extra_flags.emplace_back(
@@ -599,11 +683,13 @@ Status ExternalMiniCluster::AddTabletServer() {
   }
 
   RETURN_NOT_OK(ts->Start());
+  RETURN_NOT_OK(ts->SetServerKey());
   tablet_servers_.push_back(ts);
   return Status::OK();
 }
 
-Status ExternalMiniCluster::CreateMaster(const vector<HostPort>& master_rpc_addrs, int idx,
+Status ExternalMiniCluster::CreateMaster(const vector<HostPort>& master_rpc_addrs,
+                                         size_t idx,
                                          scoped_refptr<ExternalMaster>* master) {
   DCHECK_LT(idx, master_rpc_addrs.size());
   vector<string> flags;
@@ -651,14 +737,36 @@ Status ExternalMiniCluster::CreateMaster(const vector<HostPort>& master_rpc_addr
     flags.emplace_back(Substitute("--ranger_config_path=$0",
                                   JoinPathSegments(cluster_root(),
                                                    "ranger-client")));
+    flags.emplace_back("--trusted_user_acl=test-admin");
+  }
+  if (opts_.enable_client_jwt) {
+    flags.emplace_back("--enable_jwt_token_auth=true");
+    flags.emplace_back(Substitute("--jwks_url=$0",
+                       (opts_.start_jwks) ? oidc_->jwks_url() : "default.url"));
   }
   if (!opts_.master_alias_prefix.empty()) {
     flags.emplace_back(Substitute("--host_for_tests=$0.$1",
                                   opts_.master_alias_prefix, idx));
   }
-  // Add custom master flags.
+
+  if (opts_.enable_encryption) {
+    flags.emplace_back("--encrypt_data_at_rest=true");
+    if (opts_.enable_ranger_kms) {
+      flags.emplace_back("--encryption_key_provider=ranger-kms");
+      flags.emplace_back(Substitute("--encryption_cluster_key_name=$0", opts_.ranger_cluster_key));
+      flags.emplace_back(Substitute("--ranger_kms_url=$0", ranger_kms_->url()));
+    }
+  }
+  // Add extra master flags, common for all masters in the cluster.
   copy(opts_.extra_master_flags.begin(), opts_.extra_master_flags.end(),
-       std::back_inserter(flags));
+       back_inserter(flags));
+
+  // Add custom master flags, specific to this particular master instance.
+  if (!opts_.m_custom_flags.empty()) {
+    CHECK_EQ(opts_.num_masters, opts_.m_custom_flags.size());
+    const auto& custom_flags = opts_.m_custom_flags[idx];
+    copy(custom_flags.begin(), custom_flags.end(), back_inserter(flags));
+  }
 
   string daemon_id = Substitute("master-$0", idx);
 
@@ -1077,6 +1185,7 @@ Status ExternalMiniCluster::RemoveMaster(const HostPort& hp) {
 
 ExternalDaemon::ExternalDaemon(ExternalDaemonOptions opts)
     : opts_(std::move(opts)),
+      key_provider_(new DefaultKeyProvider()),
       parent_tid_(std::this_thread::get_id()) {
   CHECK(rpc_bind_address().Initialized());
 }
@@ -1132,6 +1241,11 @@ std::vector<std::string> ExternalDaemon::GetDaemonFlags(const ExternalDaemonOpti
 
   if (opts.enable_encryption) {
     flags.emplace_back("--encrypt_data_at_rest=true");
+    if (opts.enable_ranger_kms) {
+      flags.emplace_back("--encryption_key_provider=ranger-kms");
+      flags.emplace_back(Substitute("--encryption_cluster_key_name=$0", opts.ranger_cluster_key));
+      flags.emplace_back(Substitute("--ranger_kms_url=$0", opts.ranger_kms_url));
+    }
   }
 
   // If large keys are not enabled.
@@ -1256,6 +1370,27 @@ Status ExternalDaemon::StartProcess(const vector<string>& user_flags) {
   return Status::OK();
 }
 
+Env* ExternalDaemon::env() const {
+  return Env::Default();
+}
+
+Status ExternalDaemon::SetServerKey() {
+  string path = JoinPathSegments(this->wal_dir(), "instance");;
+  LOG(INFO) << "Reading " << path;
+  InstanceMetadataPB instance;
+  RETURN_NOT_OK(pb_util::ReadPBContainerFromPath(env(), path, &instance, pb_util::NOT_SENSITIVE));
+  if (!instance.server_key().empty()) {
+    string key;
+    RETURN_NOT_OK(key_provider_->DecryptServerKey(instance.server_key(),
+                                                  instance.server_key_iv(),
+                                                  instance.server_key_version(),
+                                                  &key));
+    LOG(INFO) << "Setting key " << key;
+    env()->SetEncryptionKey(reinterpret_cast<const uint8_t*>(a2b_hex(key).c_str()), key.size() * 4);
+  }
+  return Status::OK();
+}
+
 void ExternalDaemon::SetExePath(string exe) {
   CHECK(IsShutdown()) << "Call Shutdown() before changing the executable path";
   opts_.exe = std::move(exe);
@@ -1527,7 +1662,8 @@ ScopedResumeExternalDaemon::~ScopedResumeExternalDaemon() {
 //------------------------------------------------------------
 
 ExternalMaster::ExternalMaster(ExternalDaemonOptions opts)
-    : ExternalDaemon(std::move(opts)) {
+    : ExternalDaemon(std::move(opts)),
+      env_(Env::NewEnv()) {
 }
 
 ExternalMaster::~ExternalMaster() {
@@ -1652,7 +1788,8 @@ vector<string> ExternalMaster::GetMasterFlags(const ExternalDaemonOptions& opts)
 ExternalTabletServer::ExternalTabletServer(ExternalDaemonOptions opts,
                                            vector<HostPort> master_addrs)
     : ExternalDaemon(std::move(opts)),
-      master_addrs_(std::move(master_addrs)) {
+      master_addrs_(std::move(master_addrs)),
+      env_(Env::NewEnv()) {
   DCHECK(!master_addrs_.empty());
 }
 
