@@ -19,17 +19,18 @@
 
 #include <sys/types.h>
 
+#include <cstddef>
 #include <cstdint>
 #include <functional>
 #include <map>
 #include <memory>
+#include <optional>
 #include <ostream>
 #include <string>
 #include <thread>
 #include <utility>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <glog/logging.h>
 
 #include "kudu/common/common.pb.h"
@@ -38,6 +39,7 @@
 #include "kudu/gutil/ref_counted.h"
 #include "kudu/mini-cluster/mini_cluster.h"
 #include "kudu/security/test/mini_kdc.h"
+#include "kudu/util/mini_oidc.h"
 #include "kudu/util/monotime.h"
 #include "kudu/util/net/net_util.h"
 #include "kudu/util/status.h"
@@ -49,6 +51,18 @@ class NodeInstancePB;
 class Sockaddr;
 class Subprocess;
 
+namespace ranger {
+class MiniRanger;
+}  // namespace ranger
+
+namespace rangerkms {
+class MiniRangerKMS;
+}  // namespace rangerkms
+
+namespace security {
+class KeyProvider;
+}  // namespace security
+
 namespace client {
 class KuduClient;
 class KuduClientBuilder;
@@ -72,9 +86,9 @@ namespace rpc {
 class Messenger;
 } // namespace rpc
 
-namespace ranger {
-class MiniRanger;
-} // namespace ranger
+namespace postgres {
+class MiniPostgres;
+} // namespace postgres
 
 namespace server {
 class ServerStatusPB;
@@ -129,6 +143,9 @@ enum class BuiltinNtpConfigMode {
 };
 #endif
 
+// TODO(aserbin): maybe, turn this struct into a class template based on number
+//                of master and tablet servers, changing std::vector to
+//                std::array for {masters,tservers}_custom_flags
 struct ExternalMiniClusterOptions {
   ExternalMiniClusterOptions();
 
@@ -181,6 +198,30 @@ struct ExternalMiniClusterOptions {
   // If unset, addresses are assigned automatically.
   std::vector<HostPort> master_rpc_addresses;
 
+  // Custom flags for masters in the cluster. These are per-instance,
+  // i.e. not common across all masters in the cluster:
+  // 'm_custom_flags[i]' are the flags for master at index 'i'.
+  //
+  // If not empty, must contain the same number of elements as the number
+  // of masters in the cluster. In other words, the invariant is
+  //
+  //   m_custom_flags.empty() || m_custom_flags.size() == num_masters
+  //
+  // Default: empty
+  std::vector<std::vector<std::string>> m_custom_flags;
+
+  // Custom flags for tablet servers in the cluster. These are per-instance,
+  // i.e. not common across all tablet servers in the cluster:
+  // 't_custom_flags[i]' are the flags for tablet servers at index 'i'.
+  //
+  // If not empty, must contain the same number of elements as the number
+  // of tablet servers in the cluster. In other words, the invariant is
+  //
+  //   t_custom_flags.empty() || t_custom_flags.size() == num_tablet_servers
+  //
+  // Default: empty
+  std::vector<std::vector<std::string>> t_custom_flags;
+
   // Options to configure the MiniKdc before starting it up.
   // Only used when 'enable_kerberos' is 'true'.
   MiniKdcOptions mini_kdc_options;
@@ -211,6 +252,21 @@ struct ExternalMiniClusterOptions {
   // Default: false.
   bool enable_ranger;
 
+  // If true, set up a Ranger KMS service as part of this ExternalMiniCluster.
+  //
+  // Default: false.
+  bool enable_ranger_kms;
+
+  // Cluster key in Ranger.
+  //
+  // Default: "".
+  std::string ranger_cluster_key;
+
+  // Cluster key version in Ranger.
+  //
+  // Default: "".
+  std::string ranger_cluster_key_version;
+
   // If true, enable data at rest encryption.
   //
   // Default: false.
@@ -259,6 +315,20 @@ struct ExternalMiniClusterOptions {
 
   std::string master_alias_prefix;
   std::string tserver_alias_prefix;
+
+  MiniOidcOptions mini_oidc_options;
+
+  // When set to true, servers are configured to verify JWTs via the configured
+  // OIDC server.
+  //
+  // Default: false
+  bool enable_client_jwt;
+
+  // When set to false, the jwks server is not started, so JWT verification will not be possible.
+  // Only effective if enable_client_jwt is set to true.
+  //
+  // Default: true
+  bool start_jwks = true;
 };
 
 // A mini-cluster made up of subprocesses running each of the daemons
@@ -373,10 +443,22 @@ class ExternalMiniCluster : public MiniCluster {
     return hms_.get();
   }
 
+  postgres::MiniPostgres* postgres() const {
+    return postgres_.get();
+  }
+
   ranger::MiniRanger* ranger() const {
     return ranger_.get();
   }
 
+  rangerkms::MiniRangerKMS* ranger_kms() const {
+    return ranger_kms_.get();
+  }
+
+  MiniOidc* oidc() const {
+    return oidc_.get();
+  }
+
   const std::string& cluster_root() const {
     return opts_.cluster_root;
   }
@@ -402,9 +484,17 @@ class ExternalMiniCluster : public MiniCluster {
   // Returns the UUID for the tablet server 'ts_idx'.
   virtual std::string UuidForTS(int ts_idx) const override;
 
-  // Returns the Env on which the cluster operates.
+  // Returns the Env on which the cluster operates. If encryption is enabled,
+  // the encryption key is incorrect. For reading/writing files, ts_env() and
+  // master_env() should be used instead.
   virtual Env* env() const override;
 
+  // Returns the Env on which a specific tablet server operates.
+  virtual Env* ts_env(int ts_idx) const override;
+
+  // Returns the Env on which a specific master operates.
+  virtual Env* master_env(int master_idx) const override;
+
   BindMode bind_mode() const override {
     return opts_.bind_mode;
   }
@@ -488,7 +578,7 @@ class ExternalMiniCluster : public MiniCluster {
   // 'dir_index' is an optional numeric suffix to be added to the default path.
   // If it is not specified, the cluster must be configured to use a single data dir.
   std::string GetDataPath(const std::string& daemon_id,
-                          boost::optional<uint32_t> dir_index = boost::none) const;
+                          std::optional<uint32_t> dir_index = std::nullopt) const;
 
   // Returns paths where 'daemon_id' is expected to store its data, each with a
   // numeric suffix appropriate for 'opts_.num_data_dirs'
@@ -517,7 +607,8 @@ class ExternalMiniCluster : public MiniCluster {
   //
   // It's expected that the port for the master at 'idx' is reserved, and that
   // the master can be run with the --rpc_reuseport flag.
-  Status CreateMaster(const std::vector<HostPort>& master_rpc_addrs, int idx,
+  Status CreateMaster(const std::vector<HostPort>& master_rpc_addrs,
+                      size_t idx,
                       scoped_refptr<ExternalMaster>* master);
 
  private:
@@ -539,7 +630,11 @@ class ExternalMiniCluster : public MiniCluster {
 #endif
   std::unique_ptr<MiniKdc> kdc_;
   std::unique_ptr<hms::MiniHms> hms_;
-  std::unique_ptr<ranger::MiniRanger> ranger_;
+  std::shared_ptr<postgres::MiniPostgres> postgres_;
+  std::shared_ptr<ranger::MiniRanger> ranger_;
+  std::unique_ptr<security::KeyProvider> key_provider_;
+  std::unique_ptr<rangerkms::MiniRangerKMS> ranger_kms_;
+  std::unique_ptr<MiniOidc> oidc_;
 
   std::shared_ptr<rpc::Messenger> messenger_;
 
@@ -551,11 +646,13 @@ class ExternalMiniCluster : public MiniCluster {
 struct ExternalDaemonOptions {
   ExternalDaemonOptions()
       : logtostderr(false),
-        enable_encryption(false) {
+        enable_encryption(false),
+        enable_ranger_kms(false) {
   }
 
   bool logtostderr;
   bool enable_encryption;
+  bool enable_ranger_kms;
   std::shared_ptr<rpc::Messenger> messenger;
   std::string block_manager_type;
   std::string exe;
@@ -566,6 +663,8 @@ struct ExternalDaemonOptions {
   std::string perf_record_filename;
   std::vector<std::string> extra_flags;
   MonoDelta start_process_timeout;
+  std::string ranger_kms_url;
+  std::string ranger_cluster_key;
 };
 
 class ExternalDaemon : public RefCountedThreadSafe<ExternalDaemon> {
@@ -623,6 +722,8 @@ class ExternalDaemon : public RefCountedThreadSafe<ExternalDaemon> {
                         const std::string& principal_base,
                         const std::string& bind_host);
 
+  Status SetServerKey();
+
   // Sends a SIGSTOP signal to the daemon.
   Status Pause() WARN_UNUSED_RESULT;
 
@@ -676,6 +777,8 @@ class ExternalDaemon : public RefCountedThreadSafe<ExternalDaemon> {
   // Return the options used to create the daemon.
   ExternalDaemonOptions opts() const { return opts_; }
 
+  virtual Env* env() const;
+
   void SetRpcBindAddress(HostPort rpc_hostport) {
     DCHECK(!IsProcessAlive());
     bound_rpc_ = std::move(rpc_hostport);
@@ -727,6 +830,8 @@ class ExternalDaemon : public RefCountedThreadSafe<ExternalDaemon> {
 
   std::unique_ptr<server::ServerStatusPB> status_;
 
+  std::unique_ptr<security::KeyProvider> key_provider_;
+
   // These capture the daemons parameters and running ports and
   // are used to Restart() the daemon with the same parameters.
   HostPort bound_rpc_;
@@ -767,6 +872,8 @@ class ExternalMaster : public ExternalDaemon {
   // Requires that it has previously been shutdown.
   virtual Status Restart() override WARN_UNUSED_RESULT;
 
+  Env* env() const override { return env_.get(); }
+
   // Blocks until the master's catalog manager is initialized and responding to
   // RPCs. If 'wait_mode' is WAIT_FOR_LEADERSHIP, will further block until the
   // master has been elected leader.
@@ -791,6 +898,7 @@ class ExternalMaster : public ExternalDaemon {
   // addresses in case of restart.
   static std::vector<std::string> GetCommonFlags(const HostPort& rpc_bind_addr,
                                                  const HostPort& http_addr = HostPort());
+  const std::unique_ptr<Env> env_;
   virtual ~ExternalMaster();
 };
 
@@ -805,8 +913,10 @@ class ExternalTabletServer : public ExternalDaemon {
   // Requires that it has previously been shutdown.
   virtual Status Restart() override WARN_UNUSED_RESULT;
 
+  Env* env() const override { return env_.get(); }
  private:
   const std::vector<HostPort> master_addrs_;
+  const std::unique_ptr<Env> env_;
 
   friend class RefCountedThreadSafe<ExternalTabletServer>;
   virtual ~ExternalTabletServer();
