@@ -31,7 +31,6 @@
 #include <utility>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <glog/logging.h>
 
 #include "kudu/consensus/quorum_util.h"
@@ -73,7 +72,9 @@ Rebalancer::Config::Config(vector<string> ignored_tservers_param,
                            bool run_cross_location_rebalancing,
                            bool run_intra_location_rebalancing,
                            double load_imbalance_threshold,
-                           bool force_rebalance_replicas_on_maintenance_tservers)
+                           bool force_rebalance_replicas_on_maintenance_tservers,
+                           size_t intra_location_rebalancing_concurrency,
+                           bool enable_range_rebalancing)
     : ignored_tservers(ignored_tservers_param.begin(), ignored_tservers_param.end()),
       master_addresses(std::move(master_addresses)),
       table_filters(std::move(table_filters)),
@@ -88,7 +89,10 @@ Rebalancer::Config::Config(vector<string> ignored_tservers_param,
       run_intra_location_rebalancing(run_intra_location_rebalancing),
       load_imbalance_threshold(load_imbalance_threshold),
       force_rebalance_replicas_on_maintenance_tservers(
-          force_rebalance_replicas_on_maintenance_tservers) {
+          force_rebalance_replicas_on_maintenance_tservers),
+      intra_location_rebalancing_concurrency(
+          intra_location_rebalancing_concurrency),
+      enable_range_rebalancing(enable_range_rebalancing) {
   DCHECK_GE(max_moves_per_server, 0);
 }
 
@@ -111,7 +115,9 @@ Rebalancer::Rebalancer(Config config)
 void Rebalancer::FindReplicas(const TableReplicaMove& move,
                               const ClusterRawInfo& raw_info,
                               vector<string>* tablet_ids) {
+  const bool is_range_rebalancing = config_.enable_range_rebalancing;
   const auto& table_id = move.table_id;
+  const auto& tag = move.tag;
 
   // Tablet ids of replicas on the source tserver that are non-leaders.
   vector<string> tablet_uuids_src;
@@ -124,6 +130,9 @@ void Rebalancer::FindReplicas(const TableReplicaMove& move,
     if (tablet_summary.table_id != table_id) {
       continue;
     }
+    if (is_range_rebalancing && tablet_summary.range_key_begin != tag) {
+      continue;
+    }
     if (tablet_summary.result != HealthCheckResult::HEALTHY) {
       VLOG(1) << Substitute("table $0: not considering replicas of tablet $1 "
                             "as candidates for movement since the tablet's "
@@ -280,13 +289,20 @@ Status Rebalancer::BuildClusterInfo(const ClusterRawInfo& raw_info,
   DCHECK(info);
 
   // tserver UUID --> total replica count of all table's tablets at the server
+  // (tagged context applies here)
   typedef unordered_map<string, int32_t> TableReplicasAtServer;
 
   // The result information to build.
   ClusterInfo result_info;
 
+  // tserver UUID --> total count of replicas at the server
   unordered_map<string, int32_t> tserver_replicas_count;
-  unordered_map<string, TableReplicasAtServer> table_replicas_info;
+
+  // table_id.range_key --> count of tablet replicas of the table at tservers
+  unordered_map<TableIdAndTag, TableReplicasAtServer,
+      TableIdAndTagHash, TableIdAndTagEqual> table_replicas_info;
+
+  // UUIDs of unhealthy tablet servers.
   unordered_set<string> unhealthy_tablet_servers;
 
   // Build a set of tables with RF=1 (single replica tables).
@@ -365,7 +381,10 @@ Status Rebalancer::BuildClusterInfo(const ClusterRawInfo& raw_info,
           it != tserver_replicas_count.end()) {
         it->second++;
         auto table_ins = table_replicas_info.emplace(
-            tablet.table_id, TableReplicasAtServer());
+            TableIdAndTag{ tablet.table_id,
+                           config_.enable_range_rebalancing
+                               ? tablet.range_key_begin : "" },
+            TableReplicasAtServer());
         TableReplicasAtServer& replicas_at_server = table_ins.first->second;
 
         auto replicas_ins = replicas_at_server.emplace(move_info.ts_uuid_to, 0);
@@ -401,7 +420,8 @@ Status Rebalancer::BuildClusterInfo(const ClusterRawInfo& raw_info,
       }
 
       auto table_ins = table_replicas_info.emplace(
-          tablet.table_id, TableReplicasAtServer());
+          TableIdAndTag{tablet.table_id, tablet.range_key_begin},
+          TableReplicasAtServer());
       TableReplicasAtServer& replicas_at_server = table_ins.first->second;
 
       auto replicas_ins = replicas_at_server.emplace(ri.ts_uuid, 0);
@@ -436,11 +456,13 @@ Status Rebalancer::BuildClusterInfo(const ClusterRawInfo& raw_info,
   // Populate ClusterBalanceInfo::table_info_by_skew
   auto& table_info_by_skew = result_info.balance.table_info_by_skew;
   for (const auto& elem : table_replicas_info) {
-    const auto& table_id = elem.first;
+    const auto& table_id = elem.first.table_id;
+    const auto& tag = elem.first.tag;
     int32_t max_count = numeric_limits<int32_t>::min();
     int32_t min_count = numeric_limits<int32_t>::max();
     TableBalanceInfo tbi;
     tbi.table_id = table_id;
+    tbi.tag = tag;
     for (const auto& e : elem.second) {
       const auto& ts_uuid = e.first;
       const auto replica_count = e.second;
