@@ -25,6 +25,7 @@
 #include <vector>
 
 #include <glog/logging.h>
+#include <google/protobuf/arena.h>
 
 #include "kudu/gutil/macros.h"
 #include "kudu/gutil/ref_counted.h"
@@ -35,6 +36,7 @@
 #include "kudu/rpc/transfer.h"
 #include "kudu/util/faststring.h"
 #include "kudu/util/monotime.h"
+#include "kudu/util/net/sockaddr.h"
 #include "kudu/util/slice.h"
 #include "kudu/util/status.h"
 
@@ -47,7 +49,6 @@ class MessageLite;
 namespace kudu {
 
 class Histogram;
-class Sockaddr;
 class Trace;
 
 namespace rpc {
@@ -146,10 +147,16 @@ class InboundCall {
 
   const Sockaddr& remote_address() const;
 
+  const Sockaddr& local_address() const;
+
   const scoped_refptr<Connection>& connection() const;
 
   Trace* trace();
 
+  google::protobuf::Arena* pb_arena() {
+    return &arena_;
+  }
+
   const InboundCallTiming& timing() const {
     return timing_;
   }
@@ -238,6 +245,10 @@ class InboundCall {
   // The connection on which this inbound call arrived.
   scoped_refptr<Connection> conn_;
 
+  // Local address of the connection above. This field caches the result of the
+  // Connection::GetLocalAddress() call. Set by ParseFrom().
+  Sockaddr local_addr_;
+
   // The header of the incoming call. Set by ParseFrom()
   RequestHeader header_;
 
@@ -285,6 +296,8 @@ class InboundCall {
   // client did not pass a timeout.
   MonoTime deadline_;
 
+  google::protobuf::Arena arena_;
+
   DISALLOW_COPY_AND_ASSIGN(InboundCall);
 };
 
