@@ -22,12 +22,12 @@
 
 #include <map>
 #include <memory>
+#include <optional>
 #include <set>
 #include <string>
 #include <utility>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <glog/logging.h>
 #include <google/protobuf/compiler/code_generator.h>
 #include <google/protobuf/compiler/plugin.h>
@@ -45,7 +45,6 @@
 #include "kudu/gutil/strings/util.h"
 #include "kudu/rpc/rpc_header.pb.h"
 
-using boost::optional;
 using google::protobuf::FileDescriptor;
 using google::protobuf::io::Printer;
 using google::protobuf::MethodDescriptor;
@@ -64,17 +63,17 @@ namespace rpc {
 namespace {
 
 // Return the name of the authorization method specified for this
-// RPC method, or boost::none if none is specified.
+// RPC method, or std::nullopt if none is specified.
 //
 // This handles fallback to the service-wide default.
-optional<string> GetAuthzMethod(const MethodDescriptor& method) {
+std::optional<string> GetAuthzMethod(const MethodDescriptor& method) {
   if (method.options().HasExtension(authz_method)) {
     return method.options().GetExtension(authz_method);
   }
   if (method.service()->options().HasExtension(default_authz_method)) {
     return method.service()->options().GetExtension(default_authz_method);
   }
-  return boost::none;
+  return std::nullopt;
 }
 
 } // anonymous namespace
@@ -173,7 +172,7 @@ class MethodSubstitutions : public Substituter {
     m["metric_enum_key"] = Substitute("kMetricIndex$0", method_->name());
     bool track_result = static_cast<bool>(method_->options().GetExtension(track_rpc_result));
     m["track_result"] = track_result ? " true" : "false";
-    m["authz_method"] = GetAuthzMethod(*method_).get_value_or("AuthorizeAllowAll");
+    m["authz_method"] = GetAuthzMethod(*method_).value_or("AuthorizeAllowAll");
   }
 
   // Strips the package from method arguments if they are in the same package as
@@ -370,7 +369,7 @@ class CodeGenerator : public ::google::protobuf::compiler::CodeGenerator {
             "      ::kudu::rpc::RpcContext* context) = 0;\n");
         subs->Pop(); // method
         if (auto m = GetAuthzMethod(*method)) {
-          authz_methods.insert(m.get());
+          authz_methods.insert(*m);
         }
       }
 
@@ -422,6 +421,7 @@ class CodeGenerator : public ::google::protobuf::compiler::CodeGenerator {
         "#include \"kudu/rpc/result_tracker.h\"\n"
         "#include \"kudu/rpc/service_if.h\"\n"
         "#include \"kudu/util/metrics.h\"\n"
+        "#include \"kudu/util/net/dns_resolver.h\"\n"
         "\n");
 
     // Define metric prototypes for each method in the service.
@@ -575,6 +575,10 @@ class CodeGenerator : public ::google::protobuf::compiler::CodeGenerator {
           "      std::shared_ptr<::kudu::rpc::Messenger> messenger,\n"
           "      const ::kudu::Sockaddr& sockaddr,\n"
           "      std::string hostname);\n"
+          "  $service_name$Proxy(\n"
+          "      std::shared_ptr<::kudu::rpc::Messenger> messenger,\n"
+          "      const ::kudu::HostPort& hp,\n"
+          "      ::kudu::DnsResolver* dns_resolver);\n"
           "  ~$service_name$Proxy();\n");
 
       for (int method_idx = 0; method_idx < service->method_count();
@@ -616,6 +620,7 @@ class CodeGenerator : public ::google::protobuf::compiler::CodeGenerator {
         "#include \"$path_no_extension$.proxy.h\"\n"
         "\n"
         "namespace kudu {\n"
+        "class DnsResolver;\n"
         "namespace rpc {\n"
         "class Messenger;\n"
         "class RpcController;\n"
@@ -639,6 +644,15 @@ class CodeGenerator : public ::google::protobuf::compiler::CodeGenerator {
           "            std::move(hostname),\n"
           "            \"$full_service_name$\") {\n"
           "}\n"
+          "$service_name$Proxy::$service_name$Proxy(\n"
+          "    std::shared_ptr<::kudu::rpc::Messenger> messenger,\n"
+          "    const ::kudu::HostPort& hp,\n"
+          "    ::kudu::DnsResolver* dns_resolver)\n"
+          "    : Proxy(std::move(messenger),\n"
+          "            hp,\n"
+          "            dns_resolver,\n"
+          "            \"$full_service_name$\") {\n"
+          "}\n"
           "\n"
           "$service_name$Proxy::~$service_name$Proxy() {\n"
           "}\n");
@@ -652,7 +666,8 @@ class CodeGenerator : public ::google::protobuf::compiler::CodeGenerator {
             "    const $request$& req,\n"
             "    $response$* resp,\n"
             "    ::kudu::rpc::RpcController* controller) {\n"
-            "  return SyncRequest(\"$rpc_name$\", req, resp, controller);\n"
+            "  static const std::string kRpcName = \"$rpc_name$\";\n"
+            "  return SyncRequest(kRpcName, req, resp, controller);\n"
             "}\n"
             "\n"
             "void $service_name$Proxy::$rpc_name$Async(\n"
@@ -660,7 +675,8 @@ class CodeGenerator : public ::google::protobuf::compiler::CodeGenerator {
             "    $response$* resp,\n"
             "    ::kudu::rpc::RpcController* controller,\n"
             "    const ::kudu::rpc::ResponseCallback& callback) {\n"
-            "  AsyncRequest(\"$rpc_name$\", req, resp, controller, callback);\n"
+            "  static const std::string kRpcName = \"$rpc_name$\";\n"
+            "  AsyncRequest(kRpcName, req, resp, controller, callback);\n"
             "}\n");
         subs->Pop(); // method
       }
