@@ -29,6 +29,7 @@
 #include <string>
 #include <thread>
 #include <tuple>
+#include <type_traits>
 #include <unordered_map>
 #include <utility>
 #include <vector>
@@ -42,7 +43,6 @@
 #include "kudu/gutil/ref_counted.h"
 #include "kudu/gutil/stl_util.h"
 #include "kudu/gutil/strings/substitute.h"
-#include "kudu/rpc/acceptor_pool.h"
 #include "kudu/rpc/constants.h"
 #include "kudu/rpc/messenger.h"
 #include "kudu/rpc/outbound_call.h"
@@ -73,6 +73,12 @@
 #include "kudu/util/test_macros.h"
 #include "kudu/util/test_util.h"
 
+namespace kudu {
+namespace rpc {
+class AcceptorPool;
+}  // namespace rpc
+}  // namespace kudu
+
 METRIC_DECLARE_counter(queue_overflow_rejections_kudu_rpc_test_CalculatorService_Sleep);
 METRIC_DECLARE_histogram(handler_latency_kudu_rpc_test_CalculatorService_Sleep);
 METRIC_DECLARE_histogram(rpc_incoming_queue_time);
@@ -201,7 +207,7 @@ TEST_P(TestRpc, TestNegotiationDeadlock) {
   if (enable_ssl()) mb.enable_inbound_tls();
 
   shared_ptr<Messenger> messenger;
-  CHECK_OK(mb.Build(&messenger));
+  ASSERT_OK(mb.Build(&messenger));
 
   Sockaddr server_addr = bind_addr();
   ASSERT_OK(StartTestServerWithCustomMessenger(&server_addr, messenger, enable_ssl()));
@@ -353,7 +359,7 @@ TEST_P(TestRpc, TestCallWithBadPasswordProtectedKey) {
   Sockaddr server_addr = bind_addr();
   Status s = StartTestServer(&server_addr, enable_ssl(), rpc_certificate_file, rpc_private_key_file,
       rpc_ca_certificate_file, rpc_private_key_password_cmd);
-  ASSERT_TRUE(s.IsRuntimeError());
+  ASSERT_TRUE(s.IsRuntimeError()) << s.ToString();
   ASSERT_STR_CONTAINS(s.ToString(), "failed to load private key file");
 }
 
@@ -439,7 +445,7 @@ TEST_P(TestRpc, TestHighFDs) {
   ElementDeleter d(&fake_files);
   for (int i = 0; i < kNumFakeFiles; i++) {
     unique_ptr<RandomAccessFile> f;
-    CHECK_OK(Env::Default()->NewRandomAccessFile("/dev/zero", &f));
+    ASSERT_OK(Env::Default()->NewRandomAccessFile("/dev/zero", &f));
     fake_files.push_back(f.release());
   }
 
@@ -576,7 +582,7 @@ TEST_P(TestRpc, TestClientConnectionMetrics) {
       // Attach a big sidecar so that we are less likely to be able to send the
       // whole RPC in a single write() call without queueing it.
       int junk;
-      CHECK_OK(rpc->AddOutboundSidecar(RpcSidecar::FromSlice(big_string), &junk));
+      ASSERT_OK(rpc->AddOutboundSidecar(RpcSidecar::FromSlice(big_string), &junk));
       controllers.emplace_back(std::move(rpc));
       p.AsyncRequest(GenericCalculatorService::kAddMethodName, add_req, &add_resp,
                      controllers.back().get(), [&latch]() { latch.CountDown(); });
@@ -1277,7 +1283,7 @@ TEST_P(TestRpc, TestRpcContextClientDeadline) {
   SleepResponsePB resp;
   RpcController controller;
   Status s = p.SyncRequest("Sleep", req, &resp, &controller);
-  ASSERT_TRUE(s.IsRemoteError());
+  ASSERT_TRUE(s.IsRemoteError()) << s.ToString();
   ASSERT_STR_CONTAINS(s.ToString(), "Missing required timeout");
 
   controller.Reset();
@@ -1321,7 +1327,7 @@ TEST_P(TestRpc, TestApplicationFeatureFlag) {
     controller.RequireServerFeature(99);
     Status s = p.SyncRequest("Add", req, &resp, &controller);
     SCOPED_TRACE(strings::Substitute("unsupported response: $0", s.ToString()));
-    ASSERT_TRUE(s.IsRemoteError());
+    ASSERT_TRUE(s.IsRemoteError()) << s.ToString();
   }
 }
 
@@ -1349,7 +1355,7 @@ TEST_P(TestRpc, TestApplicationFeatureFlagUnsupportedServer) {
     controller.RequireServerFeature(FeatureFlags::FOO);
     Status s = p.SyncRequest("Add", req, &resp, &controller);
     SCOPED_TRACE(strings::Substitute("supported response: $0", s.ToString()));
-    ASSERT_TRUE(s.IsNotSupported());
+    ASSERT_TRUE(s.IsNotSupported()) << s.ToString();
   }
 
   { // No required flag
@@ -1405,7 +1411,7 @@ TEST_P(TestRpc, TestCancellation) {
         controller.RequireServerFeature(FeatureFlags::FOO);
         controller.RequireServerFeature(99);
         Status s = p.SyncRequest("Add", req, &resp, &controller);
-        ASSERT_TRUE(s.IsRemoteError());
+        ASSERT_TRUE(s.IsRemoteError()) << s.ToString();
         break;
       }
       case OutboundCall::FINISHED_SUCCESS:
@@ -1464,7 +1470,7 @@ TEST_P(TestRpc, TestCancellationAsync) {
 
     int idx;
     Slice s(payload.get(), TEST_PAYLOAD_SIZE);
-    CHECK_OK(controller.AddOutboundSidecar(RpcSidecar::FromSlice(s), &idx));
+    ASSERT_OK(controller.AddOutboundSidecar(RpcSidecar::FromSlice(s), &idx));
     req.set_sidecar_idx(idx);
 
     CountDownLatch latch(1);
