@@ -20,15 +20,14 @@
 #include <cstdint>
 #include <functional>
 #include <memory>
+#include <optional>
 #include <string>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
-
 #include "kudu/gutil/port.h"
 #include "kudu/security/cert.h" // IWYU pragma: keep
-#include "kudu/security/openssl_util.h"
 #include "kudu/util/locks.h"
+#include "kudu/util/openssl_util.h"
 #include "kudu/util/rw_mutex.h"
 #include "kudu/util/status.h"
 
@@ -140,8 +139,8 @@ class TlsContext {
 
   // Returns a new certificate signing request (CSR) in DER format, if this
   // context's cert is self-signed. If the cert is already signed, returns
-  // boost::none.
-  boost::optional<CertSignRequest> GetCsrIfNecessary() const;
+  // std::nullopt.
+  std::optional<CertSignRequest> GetCsrIfNecessary() const;
 
   // Adopts the provided CA-signed certificate for this TLS context.
   //
@@ -218,7 +217,7 @@ class TlsContext {
   int32_t trusted_cert_count_;
   bool has_cert_;
   bool is_external_cert_;
-  boost::optional<CertSignRequest> csr_;
+  std::optional<CertSignRequest> csr_;
 };
 
 } // namespace security
