@@ -18,6 +18,9 @@
 #include "kudu/server/webserver.h"
 
 #include <openssl/crypto.h>
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+#include <openssl/ssl.h>
+#endif
 
 #include <cstdlib>
 #include <functional>
@@ -59,10 +62,19 @@ using std::unique_ptr;
 using strings::Substitute;
 
 DECLARE_int32(webserver_max_post_length_bytes);
+DECLARE_string(trusted_certificate_file);
 
 DEFINE_bool(test_sensitive_flag, false, "a sensitive flag");
 TAG_FLAG(test_sensitive_flag, sensitive);
 
+DECLARE_bool(webserver_enable_csp);
+
+#if OPENSSL_VERSION_NUMBER < 0x30000000L
+int fips_mode = FIPS_mode();
+#else
+int fips_mode = EVP_default_properties_is_fips_enabled(NULL);
+#endif
+
 namespace kudu {
 
 namespace {
@@ -96,14 +108,17 @@ class WebserverTest : public KuduTest {
     opts.port = 0;
     opts.doc_root = static_dir_;
     opts.enable_doc_root = enable_doc_root();
-    if (use_ssl()) SetSslOptions(&opts);
+    if (use_ssl()) {
+      SetSslOptions(&opts);
+      cert_path_ = opts.certificate_file;
+    }
     if (use_htpasswd()) SetHTPasswdOptions(&opts);
     MaybeSetupSpnego(&opts);
     server_.reset(new Webserver(opts));
 
     AddPreInitializedDefaultPathHandlers(server_.get());
     AddPostInitializedDefaultPathHandlers(server_.get());
-    if (!use_htpasswd() || !FIPS_mode()) {
+    if (!use_htpasswd() || !fips_mode) {
       ASSERT_OK(server_->Start());
 
       vector<Sockaddr> addrs;
@@ -111,7 +126,7 @@ class WebserverTest : public KuduTest {
       ASSERT_EQ(addrs.size(), 1);
       ASSERT_TRUE(addrs[0].IsWildcard());
       ASSERT_OK(addr_.ParseString("127.0.0.1", addrs[0].port()));
-      url_ = Substitute("http://$0", addr_.ToString());
+      url_ = Substitute(use_ssl() ? "https://$0/" : "http://$0", addr_.ToString());
       // For testing purposes, we assume the server has been initialized. Typically this
       // is set to true after the rpc server is started in the server startup process.
       server_->SetStartupComplete(true);
@@ -140,6 +155,7 @@ class WebserverTest : public KuduTest {
   Sockaddr addr_;
   string url_;
   string static_dir_;
+  string cert_path_;
 };
 
 class SslWebserverTest : public WebserverTest {
@@ -155,7 +171,7 @@ class PasswdWebserverTest : public WebserverTest {
 // Send a HTTP request with no username and password. It should reject
 // the request as the .htpasswd is presented to webserver.
 TEST_F(PasswdWebserverTest, TestPasswdMissing) {
-  if (FIPS_mode()) {
+  if (fips_mode) {
     return;
   }
   Status status = curl_.FetchURL(url_, &buf_);
@@ -163,7 +179,7 @@ TEST_F(PasswdWebserverTest, TestPasswdMissing) {
 }
 
 TEST_F(PasswdWebserverTest, TestPasswdPresent) {
-  if (FIPS_mode()) {
+  if (fips_mode) {
     return;
   }
   ASSERT_OK(curl_.set_auth(CurlAuthType::DIGEST, security::kTestAuthUsername,
@@ -172,12 +188,12 @@ TEST_F(PasswdWebserverTest, TestPasswdPresent) {
 }
 
 TEST_F(PasswdWebserverTest, TestCrashInFIPSMode) {
-  if (!FIPS_mode()) {
+  if (!fips_mode) {
     return;
   }
 
   Status s = server_->Start();
-  ASSERT_TRUE(s.IsIllegalState());
+  ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
   ASSERT_STR_CONTAINS(s.ToString(), "Digest authentication in FIPS approved mode");
 }
 
@@ -411,10 +427,10 @@ TEST_F(WebserverTest, TestHttpCompression) {
 }
 
 TEST_F(SslWebserverTest, TestSSL) {
-  // We use a self-signed cert, so we need to disable cert verification in curl.
-  curl_.set_verify_peer(false);
+  // We use a self-signed cert, so we have to trust it manually.
+  FLAGS_trusted_certificate_file = cert_path_;
 
-  ASSERT_OK(curl_.FetchURL(Substitute("https://$0/", addr_.ToString()), &buf_));
+  ASSERT_OK(curl_.FetchURL(url_, &buf_));
   // Should have expected title.
   ASSERT_STR_CONTAINS(buf_.ToString(), "Kudu");
 }
@@ -453,6 +469,24 @@ TEST_F(WebserverTest, TestRedactFlagsDump) {
                                                   kRedactionMessage));
 }
 
+TEST_F(WebserverTest, TestCSPHeader) {
+  constexpr const char* kCspHeader = "Content-Security-Policy";
+
+  curl_.set_return_headers(true);
+  ASSERT_OK(curl_.FetchURL(url_, &buf_));
+  // Basic sanity check: the page should have the expected title.
+  ASSERT_STR_CONTAINS(buf_.ToString(), "Kudu");
+
+  // The CSP policy is enabled by default for the embedded Kudu webserver.
+  ASSERT_OK(curl_.FetchURL(url_, &buf_));
+  ASSERT_STR_CONTAINS(buf_.ToString(), kCspHeader);
+
+  // Check if response doesn't contain CSP header when disabled.
+  FLAGS_webserver_enable_csp = false;
+  ASSERT_OK(curl_.FetchURL(url_, &buf_));
+  ASSERT_STR_NOT_CONTAINS(buf_.ToString(), kCspHeader);
+}
+
 // Used in symbolization test below.
 void SomeMethodForSymbolTest1() {}
 // Used in symbolization test below.
