@@ -19,6 +19,9 @@
 
 #include <netinet/in.h>
 #include <openssl/crypto.h>
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+#include <openssl/ssl.h>
+#endif
 #include <sys/socket.h>
 
 #include <algorithm>
@@ -90,6 +93,11 @@ DEFINE_string(webserver_x_frame_options, "DENY",
               "to all responses. This can help prevent clickjacking attacks.");
 TAG_FLAG(webserver_x_frame_options, advanced);
 
+DEFINE_bool(webserver_enable_csp, true,
+            "The webserver adds the Content-Security-Policy header to response when enabled.");
+TAG_FLAG(webserver_enable_csp, advanced);
+TAG_FLAG(webserver_enable_csp, runtime);
+
 
 namespace kudu {
 
@@ -286,7 +294,12 @@ Status Webserver::Start() {
   }
 
   if (!opts_.password_file.empty()) {
-    if (FIPS_mode()) {
+#if OPENSSL_VERSION_NUMBER < 0x30000000L
+  int fips_mode = FIPS_mode();
+#else
+  int fips_mode = EVP_default_properties_is_fips_enabled(NULL);
+#endif
+    if (fips_mode) {
       return Status::IllegalState(
           "Webserver cannot be started with Digest authentication in FIPS approved mode");
     }
@@ -652,7 +665,7 @@ void Webserver::SendResponse(struct sq_connection* connection,
 
       ostringstream oss;
       Status s = zlib::CompressLevel(uncompressed, 1, &oss);
-      if (s.ok()) {
+      if (PREDICT_TRUE(s.ok())) {
         resp->output.str(oss.str());
         is_compressed = true;
       } else {
@@ -677,19 +690,47 @@ void Webserver::SendResponse(struct sq_connection* connection,
 
   // Write the headers to the buffer first, then write the body.
   oss << Substitute("HTTP/1.1 $0\r\n", HttpStatusCodeToString(resp->status_code));
-  oss << Substitute("Content-Type: $0\r\n",
-                    mode == StyleMode::STYLED ? "text/html" : "text/plain");
+
+  // The "Content-Type" is defined by the value of the 'mode' parameter.
+  const char* content_type = nullptr;
+  switch (mode) {
+    case StyleMode::STYLED:
+      content_type = "text/html";
+      break;
+    case StyleMode::UNSTYLED:
+      content_type = "text/plain";
+      break;
+    case StyleMode::BINARY:
+      content_type = "application/octet-stream";
+      break;
+    default:
+      LOG(FATAL) << "unexpected style mode: " << static_cast<uint32_t>(mode);
+      break;  // unreachable
+  }
+  oss << Substitute("Content-Type: $0\r\n", content_type);
+
   oss << Substitute("Content-Length: $0\r\n", body.length());
-  if (is_compressed) oss << "Content-Encoding: gzip\r\n";
+  if (is_compressed) {
+    oss << "Content-Encoding: gzip\r\n";
+  }
+  if (PREDICT_TRUE(FLAGS_webserver_enable_csp)) {
+    // TODO(aserbin): add information on when to update the SHA hash and
+    //                how to do so (ideally, the exact command line)
+    oss << "Content-Security-Policy: default-src 'self';"
+        << "style-src 'self' 'unsafe-hashes' 'sha256-47DEQpj8HBSa+/TImW+5JCeuQeRkm5NMpJWZG3hSuFU=';"
+        << "img-src 'self' data:;\r\n";
+  }
   oss << Substitute("X-Frame-Options: $0\r\n", FLAGS_webserver_x_frame_options);
   static const unordered_set<string> kInvalidHeaders = {
+    "Content-Encoding",
     "Content-Length",
     "Content-Type",
-    "X-Frame-Options"
+    "Content-Security-Policy",
+    "X-Frame-Options",
   };
   for (const auto& entry : resp->response_headers) {
     // It's forbidden to override the above headers.
-    if (ContainsKey(kInvalidHeaders, entry.first)) {
+    if (PREDICT_FALSE(ContainsKey(kInvalidHeaders, entry.first))) {
       LOG(FATAL) << Substitute("Reserved header $0 was overridden by handler",
                                entry.first);
     }
@@ -714,6 +755,10 @@ void Webserver::AddKnoxVariables(const WebRequest& req, EasyJson* json) {
   }
 }
 
+string Webserver::MustachePartialTag(const string& path) {
+  return Substitute("{{> $0.mustache}}", path);
+}
+
 void Webserver::RegisterPathHandler(const string& path, const string& alias,
     const PathHandlerCallback& callback, bool is_styled, bool is_on_nav_bar) {
   string render_path = (path == "/") ? "/home" : path;
@@ -740,8 +785,15 @@ void Webserver::RegisterPrerenderedPathHandler(const string& path, const string&
   InsertOrDie(&path_handlers_, path, new PathHandler(is_styled, is_on_nav_bar, alias, callback));
 }
 
-string Webserver::MustachePartialTag(const string& path) const {
-  return Substitute("{{> $0.mustache}}", path);
+void Webserver::RegisterBinaryDataPathHandler(
+    const string& path,
+    const string& alias,
+    const PrerenderedPathHandlerCallback& callback) {
+  std::lock_guard<RWMutex> l(lock_);
+  InsertOrDie(&path_handlers_, path, new PathHandler(false /*is_styled*/,
+                                                     false /*is_on_nav_bar*/,
+                                                     alias,
+                                                     callback));
 }
 
 bool Webserver::MustacheTemplateAvailable(const string& path) const {
@@ -771,7 +823,7 @@ static const char* const kMainTemplate = R"(
     <nav class="navbar navbar-default">
       <div class="container-fluid">
         <div class="navbar-header">
-          <a class="navbar-brand" style="padding-top: 5px;" href="{{base_url}}/">
+          <a class="navbar-brand" href="{{base_url}}/">
             <img src="{{base_url}}/logo.png" width='61' height='45' alt="Kudu"/>
           </a>
         </div>
