@@ -21,6 +21,7 @@
 #include <cstdlib>
 #include <cstring>
 #include <ostream>
+#include <type_traits>
 
 #include <glog/logging.h>
 
@@ -46,14 +47,15 @@
 #include "kudu/util/faststring.h"
 #include "kudu/util/memory/arena.h"
 
-namespace kudu {
-namespace tablet {
-
-using std::shared_ptr;
+using std::nullopt;
+using std::optional;
 using std::string;
 using std::vector;
 using strings::Substitute;
 
+namespace kudu {
+namespace tablet {
+
 string DeltaKeyAndUpdate::Stringify(DeltaType type, const Schema& schema, bool pad_key) const {
   return StrCat(Substitute("($0 delta key=$2, change_list=$1)",
                            DeltaType_Name(type),
@@ -149,7 +151,7 @@ void SelectedDeltas::ProcessDelta(rowid_t row_idx, Delta new_delta) {
 
 string SelectedDeltas::ToString() const {
   rowid_t idx = 0;
-  return JoinMapped(rows_, [&idx](const boost::optional<DeltaPair>& dp) {
+  return JoinMapped(rows_, [&idx](const optional<DeltaPair>& dp) {
       if (!dp) {
         return Substitute("$0: UNSELECTED", idx++);
       }
@@ -232,7 +234,7 @@ void DeltaPreparer<Traits>::Start(size_t nrows, int prepare_flags) {
 
 template<class Traits>
 void DeltaPreparer<Traits>::Finish(size_t nrows) {
-  MaybeProcessPreviousRowChange(boost::none);
+  MaybeProcessPreviousRowChange(nullopt);
   prev_prepared_idx_ = cur_prepared_idx_;
   cur_prepared_idx_ += nrows;
 
@@ -338,7 +340,7 @@ Status DeltaPreparer<Traits>::AddDelta(const DeltaKey& key, Slice val, bool* fin
     PreparedDelta d;
     d.key = key;
     d.val = val;
-    prepared_deltas_.emplace_back(d);
+    prepared_deltas_.emplace_back(std::move(d));
   }
 
   if (finished_row_for_apply_or_collect &&
@@ -435,9 +437,10 @@ template<class Traits>
 Status DeltaPreparer<Traits>::CollectMutations(vector<Mutation*>* dst, Arena* arena) {
   DCHECK(prepared_flags_ & DeltaIterator::PREPARE_FOR_COLLECT);
   DCHECK_LE(cur_prepared_idx_ - prev_prepared_idx_, dst->size());
-  for (const PreparedDelta& src : prepared_deltas_) {
-    DeltaKey key = src.key;
+  for (const auto& src : prepared_deltas_) {
+    const auto& key = src.key;
     RowChangeList changelist(src.val);
+    DCHECK_GE(key.row_idx(), prev_prepared_idx_);
     uint32_t rel_idx = key.row_idx() - prev_prepared_idx_;
 
     Mutation *mutation = Mutation::CreateInArena(arena, key.timestamp(), changelist);
@@ -507,7 +510,7 @@ Status DeltaPreparer<Traits>::InitDecoderIfNecessary(RowChangeListDecoder* decod
 }
 
 template<class Traits>
-void DeltaPreparer<Traits>::MaybeProcessPreviousRowChange(boost::optional<rowid_t> cur_row_idx) {
+void DeltaPreparer<Traits>::MaybeProcessPreviousRowChange(optional<rowid_t> cur_row_idx) {
   if (prepared_flags_ & DeltaIterator::PREPARE_FOR_APPLY &&
       last_added_idx_ &&
       (!cur_row_idx || cur_row_idx != *last_added_idx_)) {
