@@ -96,17 +96,17 @@ Status AlterSchemaOp::Prepare() {
   TRACE("PREPARE ALTER-SCHEMA: Starting");
 
   // Decode schema
-  unique_ptr<Schema> schema(new Schema);
-  Status s = SchemaFromPB(state_->request()->schema(), schema.get());
+  SchemaPtr schema_ptr = std::make_shared<Schema>();
+  Status s = SchemaFromPB(state_->request()->schema(), schema_ptr.get());
   if (!s.ok()) {
     state_->completion_callback()->set_error(s, TabletServerErrorPB::INVALID_SCHEMA);
     return s;
   }
 
   Tablet* tablet = state_->tablet_replica()->tablet();
-  RETURN_NOT_OK(tablet->CreatePreparedAlterSchema(state(), schema.get()));
+  RETURN_NOT_OK(tablet->CreatePreparedAlterSchema(state(), schema_ptr));
 
-  state_->AddToAutoReleasePool(std::move(schema));
+  state_->AddToAutoReleasePool(std::move(schema_ptr));
 
   TRACE("PREPARE ALTER-SCHEMA: finished");
   return s;
@@ -138,7 +138,7 @@ Status AlterSchemaOp::Apply(CommitMsg** commit_msg) {
   }
 
   state_->tablet_replica()->log()
-    ->SetSchemaForNextLogSegment(*DCHECK_NOTNULL(state_->schema()),
+    ->SetSchemaForNextLogSegment(*DCHECK_NOTNULL(state_->schema().get()),
                                  state_->schema_version());
 
   // Altered tablets should be included in the next tserver heartbeat so that
