@@ -19,11 +19,11 @@
 #include <cstdint>
 #include <memory>
 #include <mutex>
+#include <optional>
 #include <string>
 
-#include <boost/optional/optional.hpp>
-
 #include "kudu/common/common.pb.h"
+#include "kudu/common/schema.h"
 #include "kudu/consensus/consensus.pb.h"
 #include "kudu/gutil/macros.h"
 #include "kudu/tablet/ops/op.h"
@@ -32,7 +32,6 @@
 
 namespace kudu {
 
-class Schema;
 class rw_semaphore;
 
 namespace tablet {
@@ -59,8 +58,8 @@ class AlterSchemaOpState : public OpState {
   const tserver::AlterSchemaRequestPB* request() const override { return request_; }
   tserver::AlterSchemaResponsePB* response() const override { return response_; }
 
-  void set_schema(const Schema* schema) { schema_ = schema; }
-  const Schema* schema() const { return schema_; }
+  void set_schema(const SchemaPtr& schema) { schema_ = schema; }
+  const SchemaPtr schema() const { return schema_; }
 
   std::string new_table_name() const {
     return request_->new_table_name();
@@ -99,7 +98,7 @@ class AlterSchemaOpState : public OpState {
   // Sets the fact that the alter had an error.
   void SetError(const Status& s);
 
-  boost::optional<OperationResultPB> error() const {
+  std::optional<OperationResultPB> error() const {
     return error_;
   }
 
@@ -109,7 +108,7 @@ class AlterSchemaOpState : public OpState {
   DISALLOW_COPY_AND_ASSIGN(AlterSchemaOpState);
 
   // The new (target) Schema.
-  const Schema* schema_;
+  SchemaPtr schema_;
 
   // The original RPC request and response.
   const tserver::AlterSchemaRequestPB *request_;
@@ -120,7 +119,7 @@ class AlterSchemaOpState : public OpState {
 
   // The error result of this alter schema op. May be empty if the
   // op hasn't been applied or if the alter succeeded.
-  boost::optional<OperationResultPB> error_;
+  std::optional<OperationResultPB> error_;
 };
 
 // Executes the alter schema op.
