@@ -41,12 +41,14 @@ RowOp::RowOp(google::protobuf::Arena* pb_arena, DecodedRowOperation op)
   if (!decoded_op.result.ok()) {
     SetFailed(decoded_op.result);
   }
+  error_ignored = op.error_ignored;
 }
 
 void RowOp::SetFailed(const Status& s) {
   DCHECK(!result) << SecureDebugString(*result);
   result = google::protobuf::Arena::CreateMessage<OperationResultPB>(pb_arena_);
   StatusToPB(s, result->mutable_failed_status());
+  failed = true;
 }
 
 void RowOp::SetInsertSucceeded(int mrs_id) {
@@ -67,6 +69,7 @@ void RowOp::SetErrorIgnored() {
   DCHECK(!result) << SecureDebugString(*result);
   result = google::protobuf::Arena::CreateMessage<OperationResultPB>(pb_arena_);
   error_ignored = true;
+  failed = true;
 }
 
 void RowOp::SetMutateSucceeded(OperationResultPB* result) {
