@@ -78,8 +78,8 @@ void RowSetMetadata::LoadFromPB(const RowSetDataPB& pb) {
   id_ = pb.id();
 
   // Load the min/max keys.
-  min_encoded_key_ = boost::none;
-  max_encoded_key_ = boost::none;
+  min_encoded_key_.reset();
+  max_encoded_key_.reset();
   DCHECK_EQ(pb.has_min_encoded_key(), pb.has_max_encoded_key());
   if (pb.has_min_encoded_key() && pb.has_max_encoded_key()) {
     min_encoded_key_ = pb.min_encoded_key();
@@ -186,20 +186,18 @@ void RowSetMetadata::SetColumnDataBlocks(const std::map<ColumnId, BlockId>& bloc
   blocks_by_col_id_ = std::move(new_map);
 }
 
-Status RowSetMetadata::CommitRedoDeltaDataBlock(int64_t dms_id,
-                                                int64_t num_deleted_rows,
-                                                const BlockId& block_id) {
+void RowSetMetadata::CommitRedoDeltaDataBlock(int64_t dms_id,
+                                              int64_t num_deleted_rows,
+                                              const BlockId& block_id) {
   std::lock_guard<LockType> l(lock_);
   last_durable_redo_dms_id_ = dms_id;
   redo_delta_blocks_.push_back(block_id);
   IncrementLiveRowsUnlocked(-num_deleted_rows);
-  return Status::OK();
 }
 
-Status RowSetMetadata::CommitUndoDeltaDataBlock(const BlockId& block_id) {
+void RowSetMetadata::CommitUndoDeltaDataBlock(const BlockId& block_id) {
   std::lock_guard<LockType> l(lock_);
   undo_delta_blocks_.push_back(block_id);
-  return Status::OK();
 }
 
 void RowSetMetadata::CommitUpdate(const RowSetMetadataUpdate& update,
@@ -298,8 +296,12 @@ int64_t RowSetMetadata::live_row_count() const {
   return live_row_count_;
 }
 
-BlockIdContainer RowSetMetadata::GetAllBlocks() {
+BlockIdContainer RowSetMetadata::GetAllBlocks() const {
   BlockIdContainer blocks;
+  blocks.reserve(blocks_by_col_id_.size() +
+                 undo_delta_blocks_.size() +
+                 redo_delta_blocks_.size() +
+                 2);  // '2' is reserved for 'adhoc_index_block_' and 'bloom_block_'
   std::lock_guard<LockType> l(lock_);
   if (!adhoc_index_block_.IsNull()) {
     blocks.push_back(adhoc_index_block_);
@@ -316,6 +318,27 @@ BlockIdContainer RowSetMetadata::GetAllBlocks() {
   return blocks;
 }
 
+BlockId RowSetMetadata::GetMaxLiveBlockId() const {
+  BlockId max_block_id;
+  std::lock_guard<LockType> l(lock_);
+  if (!adhoc_index_block_.IsNull()) {
+    max_block_id = std::max(max_block_id, adhoc_index_block_);
+  }
+  if (!bloom_block_.IsNull()) {
+    max_block_id = std::max(max_block_id, bloom_block_);
+  }
+  for (const auto& e : blocks_by_col_id_) {
+    max_block_id = std::max(max_block_id, e.second);
+  }
+  for (const auto& block_id : undo_delta_blocks_) {
+    max_block_id = std::max(max_block_id, block_id);
+  }
+  for (const auto& block_id : redo_delta_blocks_) {
+    max_block_id = std::max(max_block_id, block_id);
+  }
+  return max_block_id;
+}
+
 RowSetMetadataUpdate::RowSetMetadataUpdate() {
 }
 
