@@ -20,15 +20,14 @@
 #include <algorithm>
 #include <functional>
 #include <mutex>
+#include <optional>
 #include <ostream>
 #include <string>
 #include <type_traits>
 #include <unordered_map>
 #include <utility>
 
-#include <boost/optional/optional.hpp>
 #include <gflags/gflags.h>
-#include <google/protobuf/stubs/port.h>
 
 #include "kudu/common/common.pb.h"
 #include "kudu/common/schema.h"
@@ -61,7 +60,8 @@
 
 DEFINE_bool(enable_tablet_orphaned_block_deletion, true,
             "Whether to enable deletion of orphaned blocks from disk. "
-            "Note: This is only exposed for debugging purposes!");
+            "Note: This is only exposed for debugging purposes and used "
+            "in CLI tools.");
 TAG_FLAG(enable_tablet_orphaned_block_deletion, advanced);
 TAG_FLAG(enable_tablet_orphaned_block_deletion, hidden);
 TAG_FLAG(enable_tablet_orphaned_block_deletion, runtime);
@@ -80,6 +80,9 @@ using kudu::log::MinLogIndexAnchorer;
 using kudu::pb_util::SecureDebugString;
 using kudu::pb_util::SecureShortDebugString;
 using std::memory_order_relaxed;
+using std::make_optional;
+using std::nullopt;
+using std::optional;
 using std::shared_ptr;
 using std::string;
 using std::unique_ptr;
@@ -103,11 +106,11 @@ Status TabletMetadata::CreateNew(FsManager* fs_manager,
                                  const PartitionSchema& partition_schema,
                                  const Partition& partition,
                                  const TabletDataState& initial_tablet_data_state,
-                                 boost::optional<OpId> tombstone_last_logged_opid,
+                                 optional<OpId> tombstone_last_logged_opid,
                                  bool supports_live_row_count,
-                                 boost::optional<TableExtraConfigPB> extra_config,
-                                 boost::optional<string> dimension_label,
-                                 boost::optional<TableTypePB> table_type,
+                                 optional<TableExtraConfigPB> extra_config,
+                                 optional<string> dimension_label,
+                                 optional<TableTypePB> table_type,
                                  scoped_refptr<TabletMetadata>* metadata) {
 
   // Verify that no existing tablet exists with the same ID.
@@ -133,8 +136,9 @@ Status TabletMetadata::CreateNew(FsManager* fs_manager,
       supports_live_row_count,
       std::move(extra_config),
       std::move(dimension_label),
-      !table_type || *table_type == TableTypePB::DEFAULT_TABLE ?
-          boost::none : std::move(table_type)));
+      !table_type ||
+          *table_type == TableTypePB::DEFAULT_TABLE ? nullopt
+                                                    : std::move(table_type)));
   RETURN_NOT_OK(ret->Flush());
   dir_group_cleanup.cancel();
 
@@ -160,16 +164,17 @@ Status TabletMetadata::LoadOrCreate(FsManager* fs_manager,
                                     const PartitionSchema& partition_schema,
                                     const Partition& partition,
                                     const TabletDataState& initial_tablet_data_state,
-                                    boost::optional<OpId> tombstone_last_logged_opid,
-                                    boost::optional<TableExtraConfigPB> extra_config,
-                                    boost::optional<string> dimension_label,
-                                    boost::optional<TableTypePB> table_type,
+                                    optional<OpId> tombstone_last_logged_opid,
+                                    optional<TableExtraConfigPB> extra_config,
+                                    optional<string> dimension_label,
+                                    optional<TableTypePB> table_type,
                                     scoped_refptr<TabletMetadata>* metadata) {
   Status s = Load(fs_manager, tablet_id, metadata);
   if (s.ok()) {
-    if (!(*metadata)->schema().Equals(schema)) {
+    const SchemaPtr schema_ptr = (*metadata)->schema();
+    if (*schema_ptr != schema) {
       return Status::Corruption(Substitute("Schema on disk ($0) does not "
-        "match expected schema ($1)", (*metadata)->schema().ToString(),
+        "match expected schema ($1)", schema_ptr->ToString(),
         schema.ToString()));
     }
     return Status::OK();
@@ -209,7 +214,7 @@ vector<BlockIdPB> TabletMetadata::CollectBlockIdPBs(const TabletSuperBlockPB& su
   return block_ids;
 }
 
-BlockIdContainer TabletMetadata::CollectBlockIds() {
+BlockIdContainer TabletMetadata::CollectBlockIds() const {
   BlockIdContainer block_ids;
   for (const auto& r : rowsets_) {
     BlockIdContainer rowset_block_ids = r->GetAllBlocks();
@@ -220,8 +225,16 @@ BlockIdContainer TabletMetadata::CollectBlockIds() {
   return block_ids;
 }
 
+BlockId TabletMetadata::GetMaxLiveBlockId() const {
+  BlockId max_block_id;
+  for (const auto& r : rowsets_) {
+    max_block_id = std::max(max_block_id, r->GetMaxLiveBlockId());
+  }
+  return max_block_id;
+}
+
 Status TabletMetadata::DeleteTabletData(TabletDataState delete_type,
-                                        const boost::optional<OpId>& last_logged_opid) {
+                                        const optional<OpId>& last_logged_opid) {
   DCHECK(!last_logged_opid || last_logged_opid->IsInitialized());
   CHECK(delete_type == TABLET_DATA_DELETED ||
         delete_type == TABLET_DATA_TOMBSTONED ||
@@ -299,11 +312,11 @@ TabletMetadata::TabletMetadata(FsManager* fs_manager, string tablet_id,
                                const Schema& schema, PartitionSchema partition_schema,
                                Partition partition,
                                const TabletDataState& tablet_data_state,
-                               boost::optional<OpId> tombstone_last_logged_opid,
+                               optional<OpId> tombstone_last_logged_opid,
                                bool supports_live_row_count,
-                               boost::optional<TableExtraConfigPB> extra_config,
-                               boost::optional<string> dimension_label,
-                               boost::optional<TableTypePB> table_type)
+                               optional<TableExtraConfigPB> extra_config,
+                               optional<string> dimension_label,
+                               optional<TableTypePB> table_type)
     : state_(kNotWrittenYet),
       tablet_id_(std::move(tablet_id)),
       table_id_(std::move(table_id)),
@@ -312,8 +325,7 @@ TabletMetadata::TabletMetadata(FsManager* fs_manager, string tablet_id,
       log_prefix_(Substitute("T $0 P $1: ", tablet_id_, fs_manager_->uuid())),
       next_rowset_idx_(0),
       last_durable_mrs_id_(kNoDurableMemStore),
-      schema_(new Schema(schema)),
-      schema_ptr_(schema_.get()),
+      schema_(std::make_shared<Schema>(schema)),
       schema_version_(0),
       table_name_(std::move(table_name)),
       partition_schema_(std::move(partition_schema)),
@@ -339,7 +351,6 @@ TabletMetadata::TabletMetadata(FsManager* fs_manager, string tablet_id)
       tablet_id_(std::move(tablet_id)),
       fs_manager_(fs_manager),
       next_rowset_idx_(0),
-      schema_ptr_(nullptr),
       num_flush_pins_(0),
       needs_flush_(false),
       flush_count_for_tests_(0),
@@ -390,11 +401,14 @@ Status TabletMetadata::LoadFromSuperBlock(const TabletSuperBlockPB& superblock)
     table_name_ = superblock.table_name();
 
     uint32_t schema_version = superblock.schema_version();
-    unique_ptr<Schema> schema(new Schema());
+    SchemaPtr schema = std::make_shared<Schema>();
     RETURN_NOT_OK_PREPEND(SchemaFromPB(superblock.schema(), schema.get()),
                           "Failed to parse Schema from superblock " +
                           SecureShortDebugString(superblock));
-    SetSchemaUnlocked(std::move(schema), schema_version);
+    {
+      SchemaPtr old_schema;
+      SwapSchemaUnlocked(schema, schema_version, &old_schema);
+    }
 
     if (!superblock.has_partition()) {
       // KUDU-818: Possible backward compatibility issue with tables created
@@ -418,7 +432,7 @@ Status TabletMetadata::LoadFromSuperBlock(const TabletSuperBlockPB& superblock)
       CHECK_EQ(table_id_, superblock.table_id());
       PartitionSchema partition_schema;
       RETURN_NOT_OK(PartitionSchema::FromPB(superblock.partition_schema(),
-                                            *schema_ptr_, &partition_schema));
+                                            *schema_, &partition_schema));
       CHECK(partition_schema_ == partition_schema);
 
       Partition partition;
@@ -442,11 +456,7 @@ Status TabletMetadata::LoadFromSuperBlock(const TabletSuperBlockPB& superblock)
     // Determine the largest block ID known to the tablet metadata so we can
     // notify the block manager of blocks it may have missed (e.g. if a data
     // directory failed and the blocks on it were not read).
-    BlockId max_block_id;
-    const auto& block_ids = CollectBlockIds();
-    for (BlockId block_id : block_ids) {
-      max_block_id = std::max(max_block_id, block_id);
-    }
+    BlockId max_block_id = GetMaxLiveBlockId();
 
     for (const BlockIdPB& block_pb : superblock.orphaned_blocks()) {
       BlockId orphaned_block_id = BlockId::FromPB(block_pb);
@@ -484,19 +494,19 @@ Status TabletMetadata::LoadFromSuperBlock(const TabletSuperBlockPB& superblock)
         !OpIdEquals(MinimumOpId(), superblock.tombstone_last_logged_opid())) {
       tombstone_last_logged_opid_ = superblock.tombstone_last_logged_opid();
     } else {
-      tombstone_last_logged_opid_ = boost::none;
+      tombstone_last_logged_opid_.reset();
     }
 
     if (superblock.has_extra_config()) {
       extra_config_ = superblock.extra_config();
     } else {
-      extra_config_ = boost::none;
+      extra_config_.reset();
     }
 
     if (superblock.has_dimension_label()) {
       dimension_label_ = superblock.dimension_label();
     } else {
-      dimension_label_ = boost::none;
+      dimension_label_.reset();
     }
 
     if (superblock.has_table_type() && superblock.table_type() != TableTypePB::DEFAULT_TABLE) {
@@ -510,11 +520,11 @@ Status TabletMetadata::LoadFromSuperBlock(const TabletSuperBlockPB& superblock)
           new TxnMetadata(
               txn_meta.has_aborted() && txn_meta.aborted(),
               txn_meta.has_commit_mvcc_op_timestamp() ?
-                  boost::make_optional(Timestamp(txn_meta.commit_mvcc_op_timestamp())) :
-                  boost::none,
+                  make_optional(Timestamp(txn_meta.commit_mvcc_op_timestamp())) :
+                  nullopt,
               txn_meta.has_commit_timestamp() ?
-                  boost::make_optional(Timestamp(txn_meta.commit_timestamp())) :
-                  boost::none,
+                  make_optional(Timestamp(txn_meta.commit_timestamp())) :
+                  nullopt,
               txn_meta.has_flushed_committed_mrs() && txn_meta.flushed_committed_mrs()
           ));
     }
@@ -714,7 +724,7 @@ void TabletMetadata::SetPreFlushCallback(StatusClosure callback) {
   pre_flush_callback_ = std::move(callback);
 }
 
-boost::optional<consensus::OpId> TabletMetadata::tombstone_last_logged_opid() const {
+std::optional<consensus::OpId> TabletMetadata::tombstone_last_logged_opid() const {
   std::lock_guard<LockType> l(data_lock_);
   return tombstone_last_logged_opid_;
 }
@@ -743,7 +753,7 @@ Status TabletMetadata::ToSuperBlockUnlocked(TabletSuperBlockPB* super_block,
   partition_.ToPB(pb.mutable_partition());
   pb.set_last_durable_mrs_id(last_durable_mrs_id_);
   pb.set_schema_version(schema_version_);
-  RETURN_NOT_OK(partition_schema_.ToPB(*schema_ptr_, pb.mutable_partition_schema()));
+  RETURN_NOT_OK(partition_schema_.ToPB(*schema_, pb.mutable_partition_schema()));
   pb.set_table_name(table_name_);
 
   for (const shared_ptr<RowSetMetadata>& meta : rowsets) {
@@ -756,8 +766,8 @@ Status TabletMetadata::ToSuperBlockUnlocked(TabletSuperBlockPB* super_block,
     InsertOrDie(pb.mutable_txn_metadata(), txn_id_and_metadata.first, meta_pb);
   }
 
-  DCHECK(schema_ptr_->has_column_ids());
-  RETURN_NOT_OK_PREPEND(SchemaToPB(*schema_ptr_, pb.mutable_schema()),
+  DCHECK(schema_->has_column_ids());
+  RETURN_NOT_OK_PREPEND(SchemaToPB(*schema_, pb.mutable_schema()),
                         "Couldn't serialize schema into superblock");
 
   pb.set_tablet_data_state(tablet_data_state_);
@@ -937,22 +947,21 @@ RowSetMetadata *TabletMetadata::GetRowSetForTests(int64_t id) {
   return nullptr;
 }
 
-void TabletMetadata::SetSchema(const Schema& schema, uint32_t version) {
-  unique_ptr<Schema> new_schema(new Schema(schema));
-  std::lock_guard<LockType> l(data_lock_);
-  SetSchemaUnlocked(std::move(new_schema), version);
+void TabletMetadata::SetSchema(const SchemaPtr& schema, uint32_t version) {
+  // In case this is the last reference to the schema, destruct the pointer
+  // outside the lock.
+  SchemaPtr old_schema;
+  {
+    std::lock_guard<LockType> l(data_lock_);
+    SwapSchemaUnlocked(schema, version, &old_schema);
+  }
 }
 
-void TabletMetadata::SetSchemaUnlocked(
-    unique_ptr<Schema> new_schema, uint32_t version) {
-  DCHECK(new_schema->has_column_ids());
-
-  // "Release" barrier ensures that, when we publish the new Schema object,
-  // all of its initialization is also visible.
-  base::subtle::Release_Store(reinterpret_cast<AtomicWord*>(&schema_ptr_),
-                              reinterpret_cast<AtomicWord>(new_schema.get()));
-  std::swap(schema_, new_schema);
-  old_schemas_.emplace_back(std::move(new_schema));
+void TabletMetadata::SwapSchemaUnlocked(SchemaPtr schema, uint32_t version,
+                                        SchemaPtr* old_schema) {
+  DCHECK(schema->has_column_ids());
+  *old_schema = std::move(schema_);
+  schema_ = std::move(schema);
   schema_version_ = version;
 }
 
@@ -976,7 +985,7 @@ uint32_t TabletMetadata::schema_version() const {
 void TabletMetadata::set_tablet_data_state(TabletDataState state) {
   std::lock_guard<LockType> l(data_lock_);
   if (state == TABLET_DATA_READY) {
-    tombstone_last_logged_opid_ = boost::none;
+    tombstone_last_logged_opid_.reset();
   }
   tablet_data_state_ = state;
 }
@@ -991,17 +1000,17 @@ void TabletMetadata::SetExtraConfig(TableExtraConfigPB extra_config) {
   extra_config_ = std::move(extra_config);
 }
 
-boost::optional<TableExtraConfigPB> TabletMetadata::extra_config() const {
+optional<TableExtraConfigPB> TabletMetadata::extra_config() const {
   std::lock_guard<LockType> l(data_lock_);
   return extra_config_;
 }
 
-boost::optional<string> TabletMetadata::dimension_label() const {
+optional<string> TabletMetadata::dimension_label() const {
   std::lock_guard<LockType> l(data_lock_);
   return dimension_label_;
 }
 
-const boost::optional<TableTypePB>& TabletMetadata::table_type() const {
+const optional<TableTypePB>& TabletMetadata::table_type() const {
   std::lock_guard<LockType> l(data_lock_);
   return table_type_;
 }
