@@ -49,6 +49,7 @@ struct TabletMetrics {
   scoped_refptr<Counter> rows_updated;
   scoped_refptr<Counter> rows_deleted;
   scoped_refptr<Counter> insert_ignore_errors;
+  scoped_refptr<Counter> upsert_ignore_errors;
   scoped_refptr<Counter> update_ignore_errors;
   scoped_refptr<Counter> delete_ignore_errors;
   scoped_refptr<Counter> insertions_failed_dup_key;
@@ -75,6 +76,7 @@ struct TabletMetrics {
   scoped_refptr<Counter> bytes_flushed;
   scoped_refptr<Counter> deleted_rowset_gc_bytes_deleted;
   scoped_refptr<Counter> undo_delta_block_gc_bytes_deleted;
+  scoped_refptr<Counter> ops_timed_out_in_prepare_queue;
 
   scoped_refptr<Histogram> bloom_lookups_per_op;
   scoped_refptr<Histogram> key_file_lookups_per_op;
@@ -105,6 +107,10 @@ struct TabletMetrics {
   scoped_refptr<Histogram> undo_delta_block_gc_delete_duration;
   scoped_refptr<Histogram> undo_delta_block_gc_perform_duration;
 
+  // Metrics specific to rowset merge compaction.
+  scoped_refptr<Histogram> compact_rs_mem_usage;
+  scoped_refptr<Histogram> compact_rs_mem_usage_to_deltas_size_ratio;
+
   scoped_refptr<Counter> leader_memory_pressure_rejections;
 
   // Compaction metrics.
