@@ -22,11 +22,11 @@
 #include <iostream>
 #include <iterator>
 #include <memory>
+#include <optional>
 #include <string>
 #include <tuple>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <gflags/gflags.h>
 #include <glog/logging.h>
 
@@ -91,6 +91,15 @@ DEFINE_uint32(max_staleness_interval_sec, 300,
               "cluster or when some unexpected concurrent activity is "
               "present (such as automatic recovery of failed replicas, etc.)");
 
+DEFINE_uint32(intra_location_rebalancing_concurrency, 0,
+              "How many independent intra-location rebalancing sessions can be "
+              "run in parallel. Since the location assignment naturally provides "
+              "non-intersecting sets of servers, it's possible to "
+              "independently move tablet replicas within different locations "
+              "in parallel. Value of 0 means 'the number of CPU cores'. "
+              "The actual number of concurrent sessions is the minimum of two "
+              "values: this setting and the number of locations in a cluster.");
+
 DEFINE_int64(max_run_time_sec, 0,
              "Maximum time to run the rebalancing, in seconds. Specifying 0 "
              "means not imposing any limit on the rebalancing run time.");
@@ -129,6 +138,9 @@ DEFINE_bool(disable_intra_location_rebalancing, false,
             "replica distribution within each location. "
             "This setting is applicable to multi-location clusters only.");
 
+DEFINE_bool(enable_range_rebalancing, false,
+            "Whether to enable table range rebalancing");
+
 DEFINE_bool(move_replicas_from_ignored_tservers, false,
             "Whether to move replicas from the specified 'ignored_tservers' to other "
             "servers when the source tablet server is healthy. "
@@ -278,7 +290,7 @@ Status EvaluateMoveSingleReplicasFlag(const vector<string>& master_addresses,
   // destination replica to catch up. During that time the tablet would not be
   // available. The idea is to reduce the risk of unintended unavailability
   // unless it's explicitly requested by the operator.
-  boost::optional<string> tid;
+  std::optional<string> tid;
   if (!ksck_results.cluster_status.tablet_summaries.empty()) {
     tid = ksck_results.cluster_status.tablet_summaries.front().id;
   }
@@ -311,6 +323,12 @@ Status RunRebalance(const RunnerContext& context) {
   const vector<string> table_filters =
       Split(FLAGS_tables, ",", strings::SkipEmpty());
 
+  if (FLAGS_enable_range_rebalancing && table_filters.size() != 1) {
+    return Status::NotSupported(
+        "range rebalancing is currently implemented for a single table only: "
+        "use '--tables' to specify a table for range rebalancing");
+  }
+
   // Evaluate --move_single_replicas flag: decide whether enable to disable
   // moving of single-replica tablets based on the reported version of the
   // Kudu components.
@@ -331,7 +349,9 @@ Status RunRebalance(const RunnerContext& context) {
       !FLAGS_disable_cross_location_rebalancing,
       !FLAGS_disable_intra_location_rebalancing,
       FLAGS_load_imbalance_threshold,
-      FLAGS_force_rebalance_replicas_on_maintenance_tservers));
+      FLAGS_force_rebalance_replicas_on_maintenance_tservers,
+      FLAGS_intra_location_rebalancing_concurrency,
+      FLAGS_enable_range_rebalancing));
 
   // Print info on pre-rebalance distribution of replicas.
   RETURN_NOT_OK(rebalancer.PrintStats(cout));
@@ -425,6 +445,7 @@ unique_ptr<Mode> BuildClusterMode() {
         .AddOptionalParameter("fetch_info_concurrency")
         .AddOptionalParameter("force_rebalance_replicas_on_maintenance_tservers")
         .AddOptionalParameter("ignored_tservers")
+        .AddOptionalParameter("intra_location_rebalancing_concurrency")
         .AddOptionalParameter("load_imbalance_threshold")
         .AddOptionalParameter("max_moves_per_server")
         .AddOptionalParameter("max_run_time_sec")
@@ -432,6 +453,7 @@ unique_ptr<Mode> BuildClusterMode() {
         .AddOptionalParameter("move_replicas_from_ignored_tservers")
         .AddOptionalParameter("move_single_replicas")
         .AddOptionalParameter("output_replica_distribution_details")
+        .AddOptionalParameter("enable_range_rebalancing")
         .AddOptionalParameter("report_only")
         .AddOptionalParameter("tables")
         .Build();
