@@ -42,7 +42,6 @@ TransactionEntry::TransactionEntry(int64_t txn_id, std::string user)
 scoped_refptr<ParticipantEntry> TransactionEntry::GetOrCreateParticipant(
     const string& tablet_id) {
   DCHECK(metadata_.IsReadLocked());
-  DCHECK_EQ(TxnStatePB::OPEN, metadata_.state().pb.state());
 
   // In the expected case, this participant hasn't been added; add it.
   std::lock_guard<simple_spinlock> l(lock_);
@@ -66,5 +65,11 @@ TxnStatePB TransactionEntry::state() const {
   return l.data().pb.state();
 }
 
+int64_t TransactionEntry::commit_timestamp() const {
+  CowLock<PersistentTransactionEntry> l(&metadata_, LockMode::READ);
+  DCHECK(l.data().pb.has_commit_timestamp());
+  return l.data().pb.commit_timestamp();
+}
+
 } // namespace transactions
 } // namespace kudu
