@@ -86,7 +86,10 @@ TEST(ScannerTest, TestExpire) {
   mgr.NewScanner(null_replica, RemoteUser(), RowFormatFlags::NO_FLAGS, &s1);
   mgr.NewScanner(null_replica, RemoteUser(), RowFormatFlags::NO_FLAGS, &s2);
   SleepFor(MonoDelta::FromMilliseconds(200));
-  s2->UpdateAccessTime();
+  {
+    // Update the access time by locking and unlocking.
+    auto access = s2->LockForAccess();
+  }
   mgr.RemoveExpiredScanners();
   ASSERT_EQ(1, mgr.CountActiveScanners());
   ASSERT_EQ(1, mgr.metrics_->scanners_expired->value());
