@@ -25,6 +25,7 @@
 #include <memory>
 #include <sstream>
 #include <string>
+#include <type_traits>
 #include <unordered_map>
 #include <utility>
 #include <vector>
@@ -35,6 +36,7 @@
 #include "kudu/common/common.pb.h"
 #include "kudu/common/iterator_stats.h"
 #include "kudu/common/partition.h"
+#include "kudu/common/schema.h"
 #include "kudu/common/wire_protocol.pb.h"
 #include "kudu/consensus/consensus.pb.h"
 #include "kudu/consensus/log_anchor_registry.h"
@@ -53,7 +55,6 @@
 #include "kudu/server/webui_util.h"
 #include "kudu/tablet/metadata.pb.h"
 #include "kudu/tablet/ops/op.h"
-#include "kudu/tablet/tablet.h"
 #include "kudu/tablet/tablet.pb.h"
 #include "kudu/tablet/tablet_metadata.h"
 #include "kudu/tablet/tablet_replica.h"
@@ -69,6 +70,12 @@
 #include "kudu/util/url-coding.h"
 #include "kudu/util/web_callback_registry.h"
 
+namespace kudu {
+namespace tablet {
+class Tablet;
+}  // namespace tablet
+}  // namespace kudu
+
 using kudu::MaintenanceManagerStatusPB;
 using kudu::consensus::ConsensusStatePB;
 using kudu::consensus::GetConsensusRole;
@@ -89,12 +96,10 @@ using std::string;
 using std::vector;
 using strings::Substitute;
 
-DECLARE_int32(scan_history_count);
-
+DECLARE_int32(completed_scan_history_count);
+DECLARE_int32(slow_scan_history_count);
 namespace kudu {
 
-class Schema;
-
 namespace tserver {
 
 namespace {
@@ -356,6 +361,7 @@ void TabletServerPathHandlers::HandleTabletsPage(const Webserver::WebRequest& /*
     for (const scoped_refptr<TabletReplica>& replica : replicas) {
       EasyJson replica_json = details_json.PushBack(EasyJson::kObject);
       const auto& tmeta = replica->tablet_metadata();
+      const SchemaPtr schema_ptr = tmeta->schema();
       TabletStatusPB status;
       replica->GetTabletStatusPB(&status);
       replica_json["table_name"] = status.table_name();
@@ -366,7 +372,7 @@ void TabletServerPathHandlers::HandleTabletsPage(const Webserver::WebRequest& /*
       }
       replica_json["partition"] =
           tmeta->partition_schema().PartitionDebugString(tmeta->partition(),
-                                                         tmeta->schema());
+                                                         *schema_ptr);
       replica_json["state"] = replica->HumanReadableState();
       if (status.has_estimated_on_disk_size()) {
         replica_json["n_bytes"] =
@@ -425,9 +431,9 @@ void TabletServerPathHandlers::HandleTabletPage(const Webserver::WebRequest& req
   output->Set("table_name", table_name);
 
   const auto& tmeta = replica->tablet_metadata();
-  const Schema& schema = tmeta->schema();
+  const SchemaPtr schema_ptr = tmeta->schema();
   output->Set("partition",
-              tmeta->partition_schema().PartitionDebugString(tmeta->partition(), schema));
+              tmeta->partition_schema().PartitionDebugString(tmeta->partition(), *schema_ptr));
   output->Set("on_disk_size", HumanReadableNumBytes::ToString(replica->OnDiskSize()));
   uint64_t live_row_count;
   Status s = replica->CountLiveRows(&live_row_count);
@@ -437,7 +443,7 @@ void TabletServerPathHandlers::HandleTabletPage(const Webserver::WebRequest& req
     output->Set("tablet_live_row_count", "N/A");
   }
 
-  SchemaToJson(schema, output);
+  SchemaToJson(*schema_ptr, output);
 }
 
 void TabletServerPathHandlers::HandleTabletSVGPage(const Webserver::WebRequest& req,
@@ -498,29 +504,29 @@ const char* ScanStateToString(const ScanState& scan_state) {
 }
 
 // Formats the scan descriptor's pseudo-SQL query string as HTML.
-string ScanQueryHtml(const ScanDescriptor& scan) {
+string ScanQueryHtml(const SharedScanDescriptor& scan) {
   string query = "<b>SELECT</b> ";
-  if (scan.projected_columns.empty()) {
+  if (scan->projected_columns.empty()) {
     query.append("COUNT(*)");
   } else {
-    query.append(JoinMapped(scan.projected_columns, EscapeForHtmlToString, ",<br>       "));
+    query.append(JoinMapped(scan->projected_columns, EscapeForHtmlToString, ",<br>       "));
   }
   query.append("<br>  <b>FROM</b> ");
-  if (scan.table_name.empty()) {
+  if (scan->table_name.empty()) {
     query.append("&lt;unknown&gt;");
   } else {
-    query.append(EscapeForHtmlToString(scan.table_name));
+    query.append(EscapeForHtmlToString(scan->table_name));
   }
 
-  if (!scan.predicates.empty()) {
+  if (!scan->predicates.empty()) {
     query.append("<br> <b>WHERE</b> ");
-    query.append(JoinMapped(scan.predicates, EscapeForHtmlToString, "<br>   <b>AND</b> "));
+    query.append(JoinMapped(scan->predicates, EscapeForHtmlToString, "<br>   <b>AND</b> "));
   }
 
   return query;
 }
 
-void IteratorStatsToJson(const ScanDescriptor& scan, EasyJson* json) {
+void IteratorStatsToJson(const SharedScanDescriptor& scan, EasyJson* json) {
 
   auto fill_stats = [] (EasyJson& row, const string& column, const IteratorStats& stats) {
     row["column"] = column;
@@ -535,7 +541,7 @@ void IteratorStatsToJson(const ScanDescriptor& scan, EasyJson* json) {
   };
 
   IteratorStats total_stats;
-  for (const auto& column : scan.iterator_stats) {
+  for (const auto& column : scan->iterator_stats) {
     EasyJson row = json->PushBack(EasyJson::kObject);
     fill_stats(row, column.first, column.second);
     total_stats += column.second;
@@ -545,28 +551,28 @@ void IteratorStatsToJson(const ScanDescriptor& scan, EasyJson* json) {
   fill_stats(total_row, "total", total_stats);
 }
 
-void ScanToJson(const ScanDescriptor& scan, EasyJson* json) {
+void ScanToJson(const SharedScanDescriptor& scan, EasyJson* json) {
   MonoTime now = MonoTime::Now();
   MonoDelta duration;
-  if (scan.state == ScanState::kActive) {
-    duration = now - scan.start_time;
+  if (scan->state == ScanState::kActive) {
+    duration = now - scan->start_time;
   } else {
-    duration = scan.last_access_time - scan.start_time;
+    duration = scan->last_access_time - scan->start_time;
   }
-  MonoDelta time_since_start = now - scan.start_time;
+  MonoDelta time_since_start = now - scan->start_time;
 
-  json->Set("tablet_id", scan.tablet_id);
-  json->Set("scanner_id", scan.scanner_id);
-  json->Set("state", ScanStateToString(scan.state));
+  json->Set("tablet_id", scan->tablet_id);
+  json->Set("scanner_id", scan->scanner_id);
+  json->Set("state", ScanStateToString(scan->state));
   json->Set("query", ScanQueryHtml(scan));
-  json->Set("requestor", scan.remote_user.username());
+  json->Set("requestor", scan->remote_user.username());
 
   json->Set("duration", HumanReadableElapsedTime::ToShortString(duration.ToSeconds()));
-  json->Set("num_round_trips", scan.last_call_seq_id);
+  json->Set("num_round_trips", scan->last_call_seq_id);
   json->Set("time_since_start",
             HumanReadableElapsedTime::ToShortString(time_since_start.ToSeconds()));
 
-  const auto& cpu_times = scan.cpu_times;
+  const auto& cpu_times = scan->cpu_times;
   json->Set("wall_secs",
             HumanReadableElapsedTime::ToShortString(cpu_times.wall_seconds()));
   json->Set("user_secs",
@@ -592,14 +598,24 @@ const char* kLongTimingTitle = "wall time, user cpu time, and system cpu time "
 void TabletServerPathHandlers::HandleScansPage(const Webserver::WebRequest& /*req*/,
                                                Webserver::WebResponse* resp) {
   EasyJson* output = &resp->output;
-  (*output)["scan_history_count"] = FLAGS_scan_history_count;
+  (*output)["completed_scan_history_count"] = FLAGS_completed_scan_history_count;
   output->Set("timing_title", kLongTimingTitle);
-  EasyJson scans = output->Set("scans", EasyJson::kArray);
-  vector<ScanDescriptor> descriptors = tserver_->scanner_manager()->ListScans();
+  EasyJson completed_scans = output->Set("completed_scans", EasyJson::kArray);
+  vector<SharedScanDescriptor> descriptors = tserver_->scanner_manager()->ListScans();
 
   for (const auto& descriptor : descriptors) {
-    EasyJson scan = scans.PushBack(EasyJson::kObject);
-    ScanToJson(descriptor, &scan);
+    EasyJson completed_scan = completed_scans.PushBack(EasyJson::kObject);
+    ScanToJson(descriptor, &completed_scan);
+  }
+
+  (*output)["slow_scan_history_count"] = FLAGS_slow_scan_history_count;
+  EasyJson slow_scans = output->Set("slow_scans", EasyJson::kArray);
+  vector<SharedScanDescriptor> slow_descriptors =
+      tserver_->scanner_manager()->ListSlowScans();
+
+  for (const auto& slow_descriptor : slow_descriptors) {
+    EasyJson slow_scan = slow_scans.PushBack(EasyJson::kObject);
+    ScanToJson(slow_descriptor, &slow_scan);
   }
 }
 
@@ -646,6 +662,7 @@ void TabletServerPathHandlers::HandleMaintenanceManagerPage(const Webserver::Web
     registered_op["logs_retained"] = HumanReadableNumBytes::ToString(op_pb.logs_retained_bytes());
     registered_op["perf"] = op_pb.perf_improvement();
     registered_op["workload_score"] = op_pb.workload_score();
+    registered_op["data_retained"] = HumanReadableNumBytes::ToString(op_pb.data_retained_bytes());
   }
 }
 
