@@ -56,6 +56,14 @@ class EasyCurl {
                   faststring* dst,
                   const std::vector<std::string>& headers = {});
 
+  // Same as above, but with failover support. If the connection to the first
+  // URL in the vector fails or times out, it attempts to connect to the next
+  // one in the list. It stops at the first successful attempt and returns the
+  // result.
+  Status FetchURL(const std::vector<std::string>& urls,
+                  faststring* dst,
+                  const std::vector<std::string>& headers = {});
+
   // Issue an HTTP POST to the given URL with the given data.
   // Returns results in 'dst' as above.
   // The optional param 'headers' holds additional headers.
@@ -65,6 +73,15 @@ class EasyCurl {
                    faststring* dst,
                    const std::vector<std::string>& headers = {});
 
+  // Same as above, but with failover support. If the connection to the first
+  // URL in the vector fails or times out, it attempts to connect to the next
+  // one in the list. It stops at the first successful attempt and returns the
+  // result.
+  Status PostToURL(const std::vector<std::string>& urls,
+                   const std::string& post_data,
+                   faststring* dst,
+                   const std::vector<std::string>& headers = {});
+
   // Set whether to verify the server's SSL certificate in the case of an HTTPS
   // connection.
   void set_verify_peer(bool verify) {
@@ -79,6 +96,13 @@ class EasyCurl {
     timeout_ = t;
   }
 
+  // Set the list of DNS servers to be used instead of the system default.
+  // The format of the dns servers option is:
+  //   host[:port][,host[:port]]...
+  void set_dns_servers(std::string dns_servers) {
+    dns_servers_ = std::move(dns_servers);
+  }
+
   Status set_auth(CurlAuthType auth_type, std::string username = "", std::string password = "") {
     auth_type_ = std::move(auth_type);
     username_ = std::move(username);
@@ -134,6 +158,15 @@ class EasyCurl {
                    faststring* dst,
                    const std::vector<std::string>& headers = {});
 
+  // Same as above, but with failover support. If the connection to the first
+  // URL in the vector fails or times out, it attempts to connect to the next
+  // one in the list. It stops at the first successful attempt and returns the
+  // result.
+  Status DoRequest(const std::vector<std::string>& url,
+                   const std::string* post_data,
+                   faststring* dst,
+                   const std::vector<std::string>& headers = {});
+
   CURL* curl_;
 
   std::string custom_method_;
@@ -153,6 +186,8 @@ class EasyCurl {
 
   MonoDelta timeout_;
 
+  std::string dns_servers_;
+
   int num_connects_ = 0;
 
   char errbuf_[kErrBufSize];
