@@ -37,6 +37,7 @@ class WritableFile;
 struct RandomAccessFileOptions;
 struct RWFileOptions;
 struct WritableFileOptions;
+struct SequentialFileOptions;
 
 template <typename T>
 class ArrayView;
@@ -45,6 +46,7 @@ class ArrayView;
 struct SpaceInfo {
   int64_t capacity_bytes; // Capacity of a filesystem, in bytes.
   int64_t free_bytes;     // Bytes available to non-privileged processes.
+  uint64_t filesystem_id; // FilesystemID returned by statvfs()
 };
 
 class Env {
@@ -74,6 +76,11 @@ class Env {
   // The result of Default() belongs to kudu and must never be deleted.
   static Env* Default();
 
+  // Return a new Env instance of the same type as the default
+  // environment.  Unlike the default env, this is not owned by Kudu, and
+  // must be destroyed when not used anymore.
+  static std::unique_ptr<Env> NewEnv();
+
   // Create a brand new sequentially-readable file with the specified name.
   // On success, stores a pointer to the new file in *result and returns OK.
   // On failure stores NULL in *result and returns non-OK.  If the file does
@@ -83,6 +90,11 @@ class Env {
   virtual Status NewSequentialFile(const std::string& fname,
                                    std::unique_ptr<SequentialFile>* result) = 0;
 
+  // Like the previous NewSequentialFile, but allows options to be specified.
+  virtual Status NewSequentialFile(const SequentialFileOptions& opts,
+                                   const std::string& fname,
+                                   std::unique_ptr<SequentialFile>* result) = 0;
+
   // Create a brand new random access read-only file with the
   // specified name.  On success, stores a pointer to the new file in
   // *result and returns OK.  On failure stores NULL in *result and
@@ -183,7 +195,7 @@ class Env {
   virtual Status DeleteRecursively(const std::string &dirname) = 0;
 
   // Store the logical size of fname in *file_size.
-  virtual Status GetFileSize(const std::string& fname, uint64_t* file_size) = 0;
+  virtual Status GetFileSize(const std::string& fname, uint64_t* file_size)= 0;
 
   // Store the physical size of fname in *file_size.
   //
@@ -367,12 +379,20 @@ class Env {
   // Creates symlink 'dst' that points to 'source'.
   virtual Status CreateSymLink(const std::string& src, const std::string& dst) = 0;
 
+  // Returns the encryption header size.
+  virtual size_t GetEncryptionHeaderSize() const = 0;
+
   // Special string injected into file-growing operations' random failures
   // (if enabled).
   //
   // Only useful for tests.
   static const char* const kInjectedFailureStatusMsg;
 
+  virtual bool IsEncryptionEnabled() const = 0;
+
+  // Set the raw server encryption key. The key size is in bits.
+  virtual void SetEncryptionKey(const uint8_t* key, size_t key_size) = 0;
+
  private:
   DISALLOW_COPY_AND_ASSIGN(Env);
 };
@@ -382,6 +402,9 @@ class File {
  public:
   virtual ~File();
 
+  // Returns the encryption header size.
+  virtual size_t GetEncryptionHeaderSize() const = 0;
+
   // Returns the filename provided at construction time.
   virtual const std::string& filename() const = 0;
 };
@@ -401,15 +424,27 @@ class Fifo : public File {
   // opened for reads before calling.
   virtual int read_fd() const = 0;
 
+  // Initializes the default environment with encryption enabled using the
+  // given AES key.
+  static Status InitializeEncryptedEnv(int key_size, uint8_t* server_key);
+
   // Returns the write fd, set when opened for writes. The fifo must have been
   // opened for writes before calling.
   virtual int write_fd() const = 0;
 };
 
+struct SequentialFileOptions {
+  // Whether the file contains sensitive information. If true, the file is
+  // encrypted if encryption is enabled.
+  bool is_sensitive;
+
+  SequentialFileOptions() : is_sensitive(false) {}
+};
+
 // A file abstraction for reading sequentially through a file
 class SequentialFile : public File {
  public:
-  SequentialFile() { }
+  SequentialFile() {}
 
   // Read up to "result.size" bytes from the file.
   // Sets "result.data" to the data that was read.
@@ -481,14 +516,21 @@ struct WritableFileOptions {
   // See CreateMode for details.
   Env::OpenMode mode;
 
+  // Whether the file contains sensitive information. If true, the file is
+  // encrypted if encryption is enabled.
+  bool is_sensitive;
+
   WritableFileOptions()
-    : sync_on_close(false),
-      mode(Env::CREATE_OR_OPEN_WITH_TRUNCATE) { }
+      : sync_on_close(false), mode(Env::CREATE_OR_OPEN_WITH_TRUNCATE), is_sensitive(false) {}
 };
 
 // Options specified when a file is opened for random access.
 struct RandomAccessFileOptions {
-  RandomAccessFileOptions() {}
+  // Whether the file contains sensitive information. If true, the file is
+  // encrypted if encryption is enabled.
+  bool is_sensitive;
+
+  RandomAccessFileOptions() : is_sensitive(false) {}
 };
 
 // A file abstraction for sequential writing.  The implementation
@@ -548,9 +590,12 @@ struct RWFileOptions {
   // See CreateMode for details.
   Env::OpenMode mode;
 
+  // Whether the file contains sensitive information. If true, the file is
+  // encrypted if encryption is enabled.
+  bool is_sensitive;
+
   RWFileOptions()
-    : sync_on_close(false),
-      mode(Env::CREATE_OR_OPEN_WITH_TRUNCATE) { }
+      : sync_on_close(false), mode(Env::CREATE_OR_OPEN_WITH_TRUNCATE), is_sensitive(false) {}
 };
 
 // A file abstraction for both reading and writing. No notion of a built-in
@@ -662,6 +707,8 @@ class RWFile : public File {
   // Retrieves the file's size.
   virtual Status Size(uint64_t* size) const = 0;
 
+  virtual bool IsEncrypted() const = 0;
+
   // Retrieve a map of the file's live extents.
   //
   // Each map entry is an offset and size representing a section of live file
@@ -695,10 +742,21 @@ extern Status WriteStringToFile(Env* env, const Slice& data,
 extern Status WriteStringToFileSync(Env* env, const Slice& data,
                                     const std::string& fname);
 
+// A utility routine: write "data" to the named encrypted file.
+extern Status WriteStringToFileEncrypted(Env* env, const Slice& data,
+                                         const std::string& fname);
+// Like above but also fsyncs the new file.
+extern Status WriteStringToFileEncryptedSync(Env* env, const Slice& data,
+                                             const std::string& fname);
+
 // A utility routine: read contents of named file into *data
 extern Status ReadFileToString(Env* env, const std::string& fname,
                                faststring* data);
 
+// A utility routine: read contents of named encrypted file into *data
+extern Status ReadFileToStringEncrypted(Env* env, const std::string& fname,
+                                        faststring* data);
+
 // Overloaded operator for printing Env::ResourceLimitType.
 std::ostream& operator<<(std::ostream& o, Env::ResourceLimitType t);
 
