@@ -88,6 +88,7 @@
 
 using base::subtle::Atomic64;
 using base::subtle::Barrier_AtomicIncrement;
+using kudu::security::ssl_make_unique;
 using std::accumulate;
 using std::string;
 using std::unique_ptr;
@@ -222,11 +223,16 @@ const uint8_t kEncryptionHeaderSize = 64;
 
 const char* const kEncryptionHeaderMagic = "kuduenc";
 
-using evp_ctx_unique_ptr = std::unique_ptr<EVP_CIPHER_CTX, decltype(&EVP_CIPHER_CTX_free)>;
+namespace security {
+
+template<> struct SslTypeTraits<EVP_CIPHER_CTX> {
+  static constexpr auto kFreeFunc = &EVP_CIPHER_CTX_free;
+};
+
+} // namespace security
 
 namespace {
 
-
 struct FreeDeleter {
   inline void operator()(void* ptr) const {
     free(ptr);
@@ -452,10 +458,14 @@ Status DoEncryptV(const EncryptionHeader* eh,
   InlineBigEndianEncodeFixed64(&iv[0], 0);
   InlineBigEndianEncodeFixed64(&iv[8], offset / kEncryptionBlockSize);
 
-  evp_ctx_unique_ptr ctx(EVP_CIPHER_CTX_new(), EVP_CIPHER_CTX_free);
-
-  OPENSSL_RET_NOT_OK(EVP_EncryptInit_ex(ctx.get(), GetEVPCipher(eh->algorithm),
-                                        nullptr, eh->key, iv),
+  const auto* cipher = GetEVPCipher(eh->algorithm);
+  if (!cipher) {
+    return Status::RuntimeError(
+        StringPrintf("no cipher for algorithm 0x%02x", eh->algorithm));
+  }
+  auto ctx = ssl_make_unique(EVP_CIPHER_CTX_new());
+  OPENSSL_RET_IF_NULL(ctx, "failed to create cipher context");
+  OPENSSL_RET_NOT_OK(EVP_EncryptInit_ex(ctx.get(), cipher, nullptr, eh->key, iv),
                      "Failed to initialize encryption");
   OPENSSL_RET_NOT_OK(EVP_CIPHER_CTX_set_padding(ctx.get(), 0),
                      "failed to disable padding");
@@ -497,9 +507,14 @@ Status DoDecryptV(const EncryptionHeader* eh, uint64_t offset, ArrayView<Slice>
   InlineBigEndianEncodeFixed64(&iv[0], 0);
   InlineBigEndianEncodeFixed64(&iv[8], offset / kEncryptionBlockSize);
 
-  evp_ctx_unique_ptr ctx(EVP_CIPHER_CTX_new(), EVP_CIPHER_CTX_free);
-  OPENSSL_RET_NOT_OK(EVP_DecryptInit_ex(ctx.get(), GetEVPCipher(eh->algorithm),
-                                        nullptr, eh->key, iv),
+  const auto* cipher = GetEVPCipher(eh->algorithm);
+  if (!cipher) {
+    return Status::RuntimeError(
+        StringPrintf("no cipher for algorithm 0x%02x", eh->algorithm));
+  }
+  auto ctx = ssl_make_unique(EVP_CIPHER_CTX_new());
+  OPENSSL_RET_IF_NULL(ctx, "failed to create cipher context");
+  OPENSSL_RET_NOT_OK(EVP_DecryptInit_ex(ctx.get(), cipher, nullptr, eh->key, iv),
                      "Failed to initialize decryption");
   OPENSSL_RET_NOT_OK(EVP_CIPHER_CTX_set_padding(ctx.get(), 0),
                      "failed to disable padding");
@@ -1541,13 +1556,25 @@ class PosixEnv : public Env {
     bool encrypted = opts.is_sensitive && IsEncryptionEnabled();
     EncryptionHeader header;
     if (encrypted) {
+      auto cleanup = MakeScopedCleanup([&]() {
+        fclose(f);
+      });
+
       DCHECK(server_key_);
       int fd;
       RETURN_NOT_OK(DoOpen(fname, OpenMode::MUST_EXIST, &fd));
+
+      auto fd_cleanup = MakeScopedCleanup([&]() {
+          DoClose(fd);
+      });
+
       RETURN_NOT_OK(ReadEncryptionHeader(fd, fname, *server_key_, &header));
       if (fseek(f, kEncryptionHeaderSize, SEEK_CUR)) {
         return IOError(fname, errno);
       }
+
+      cleanup.cancel();
+      fd_cleanup.cancel();
     }
     result->reset(new PosixSequentialFile(fname, encrypted, f, header));
     return Status::OK();
@@ -1573,7 +1600,7 @@ class PosixEnv : public Env {
     bool encrypted = opts.is_sensitive && IsEncryptionEnabled();
     if (encrypted) {
       DCHECK(server_key_);
-      RETURN_NOT_OK(ReadEncryptionHeader(fd, fname, *server_key_, &header));
+      RETURN_NOT_OK_EVAL(ReadEncryptionHeader(fd, fname, *server_key_, &header), DoClose(fd));
     }
     result->reset(new PosixRandomAccessFile(fname, fd,
                   encrypted, header));
@@ -1591,7 +1618,12 @@ class PosixEnv : public Env {
     TRACE_EVENT1("io", "PosixEnv::NewWritableFile", "path", fname);
     int fd;
     RETURN_NOT_OK(DoOpen(fname, opts.mode, &fd));
-    return InstantiateNewWritableFile(fname, fd, opts, result);
+    auto cleanup = MakeScopedCleanup([&]() {
+      DoClose(fd);
+    });
+    RETURN_NOT_OK(InstantiateNewWritableFile(fname, fd, opts, result));
+    cleanup.cancel();
+    return Status::OK();
   }
 
   virtual Status NewTempWritableFile(const WritableFileOptions& opts,
@@ -1602,8 +1634,12 @@ class PosixEnv : public Env {
     int fd = 0;
     string tmp_filename;
     RETURN_NOT_OK(MkTmpFile(name_template, &fd, &tmp_filename));
+    auto cleanup = MakeScopedCleanup([&]() {
+      DoClose(fd);
+    });
     RETURN_NOT_OK(InstantiateNewWritableFile(tmp_filename, fd, opts, result));
     created_filename->swap(tmp_filename);
+    cleanup.cancel();
     return Status::OK();
   }
 
@@ -1628,6 +1664,9 @@ class PosixEnv : public Env {
     RETURN_NOT_OK(DoOpen(fname, opts.mode, &fd));
     EncryptionHeader eh;
     if (encrypt) {
+      auto cleanup = MakeScopedCleanup([&]() {
+        DoClose(fd);
+      });
       DCHECK(server_key_);
       if (size >= kEncryptionHeaderSize) {
         RETURN_NOT_OK(ReadEncryptionHeader(fd, fname, *server_key_, &eh));
@@ -1635,6 +1674,7 @@ class PosixEnv : public Env {
         RETURN_NOT_OK(GenerateHeader(&eh));
         RETURN_NOT_OK(WriteEncryptionHeader(fd, fname, *server_key_, eh));
       }
+      cleanup.cancel();
     }
     result->reset(new PosixRWFile(fname, fd, opts.sync_on_close,
                                   encrypt, eh));
@@ -1649,9 +1689,13 @@ class PosixEnv : public Env {
     bool encrypt = opts.is_sensitive && IsEncryptionEnabled();
     EncryptionHeader eh;
     if (encrypt) {
+      auto cleanup = MakeScopedCleanup([&]() {
+        DoClose(fd);
+      });
       DCHECK(server_key_);
       RETURN_NOT_OK(GenerateHeader(&eh));
       RETURN_NOT_OK(WriteEncryptionHeader(fd, *created_filename, *server_key_, eh));
+      cleanup.cancel();
     }
     res->reset(new PosixRWFile(*created_filename, fd, opts.sync_on_close,
                                encrypt, eh));
