@@ -127,8 +127,8 @@ TEST_F(FlagTagsTest, TestSensitiveFlags) {
   // Setting a sensitive flag should return a redacted value.
   {
     kudu::g_should_redact = kudu::RedactContext::LOG;
-    ASSERT_STR_CONTAINS(CommandlineFlagsIntoString(EscapeMode::NONE), strings::Substitute(
-                        "--test_sensitive_flag=$0", kRedactionMessage));
+    ASSERT_STR_CONTAINS(CommandlineFlagsIntoString(EscapeMode::NONE, Selection::ALL),
+                        strings::Substitute("--test_sensitive_flag=$0", kRedactionMessage));
   }
 }
 
