@@ -246,7 +246,7 @@ DECLARE_bool(logtostderr);
 TAG_FLAG(logtostderr, stable);
 TAG_FLAG(logtostderr, runtime);
 
-DECLARE_int32(max_log_size);
+DECLARE_uint32(max_log_size);
 TAG_FLAG(max_log_size, stable);
 TAG_FLAG(max_log_size, runtime);
 
@@ -437,6 +437,16 @@ void CheckFlagsAllowed() {
   }
 }
 
+bool CheckCustomValidators() {
+  const auto& validators(GetFlagValidators());
+  for (const auto& e : validators) {
+    if (!e.second()) {
+      return true;
+    }
+  }
+  return false;
+}
+
 // Run 'late phase' custom validators: these can be run only when all flags are
 // already parsed and individually validated.
 void RunCustomValidators() {
@@ -543,25 +553,20 @@ void ValidateFlags() {
   RunCustomValidators();
 }
 
-string CommandlineFlagsIntoString(EscapeMode mode) {
-  string ret_value;
-  vector<CommandLineFlagInfo> flags;
-  GetAllFlags(&flags);
-
-  for (const auto& f : flags) {
-    ret_value += "--";
-    if (mode == EscapeMode::HTML) {
-      ret_value += EscapeForHtmlToString(f.name);
-    } else if (mode == EscapeMode::NONE) {
-      ret_value += f.name;
-    }
-    ret_value += "=";
-    ret_value += CheckFlagAndRedact(f, mode);
-    ret_value += "\n";
+bool AreFlagsConsistent() {
+  if (CheckFlagsAndWarn("unsafe", FLAGS_unlock_unsafe_flags)) {
+    return false;
   }
-  return ret_value;
+  if (CheckFlagsAndWarn("experimental", FLAGS_unlock_experimental_flags)) {
+    return false;
+  }
+  if (CheckCustomValidators()) {
+    return false;
+  }
+  return true;
 }
 
+
 vector<CommandLineFlagInfo> GetNonDefaultFlagsHelper() {
   vector<CommandLineFlagInfo> all_flags;
   GetAllFlags(&all_flags);
@@ -643,4 +648,30 @@ bool GetBooleanEnvironmentVariable(const char* env_var_name) {
   return false;  // unreachable
 }
 
+string CommandlineFlagsIntoString(EscapeMode mode, Selection selection) {
+  string ret_value;
+  vector<CommandLineFlagInfo> flags;
+  switch (selection) {
+    case Selection::ALL:
+      GetAllFlags(&flags);
+      break;
+    case Selection::NONDEFAULT:
+      flags = GetNonDefaultFlagsHelper();
+      break;
+  }
+
+  for (const auto& f : flags) {
+    ret_value += "--";
+    if (mode == EscapeMode::HTML) {
+      ret_value += EscapeForHtmlToString(f.name);
+    } else if (mode == EscapeMode::NONE) {
+      ret_value += f.name;
+    }
+    ret_value += "=";
+    ret_value += CheckFlagAndRedact(f, mode);
+    ret_value += "\n";
+  }
+  return ret_value;
+}
+
 } // namespace kudu
