@@ -190,7 +190,7 @@ void FlushCoverageOnExit() {
 // the first time it's called.
 //
 // NOTE: this is only used in coverage builds!
-void FailureWriterWithCoverage(const char* data, int size) {
+void FailureWriterWithCoverage(const char* data, size_t size) {
   FlushCoverageOnExit();
 
   // Original implementation from glog:
@@ -205,7 +205,7 @@ void FailureWriterWithCoverage(const char* data, int size) {
 // NOTE: this is only used in coverage builds!
 void FlushCoverageAndAbort() {
   FlushCoverageOnExit();
-  abort();
+  exit(1);
 }
 } // anonymous namespace
 
@@ -256,7 +256,8 @@ void InitGoogleLoggingSafe(const char* arg) {
     // This allows us to handle both LOG(FATAL) and unintended crashes like
     // SEGVs.
     google::InstallFailureWriter(FailureWriterWithCoverage);
-    google::InstallFailureFunction(FlushCoverageAndAbort);
+    google::InstallFailureFunction(
+            reinterpret_cast<google::logging_fail_func_t>(FlushCoverageAndAbort));
   }
 
   // Needs to be done after InitGoogleLogging
@@ -349,11 +350,11 @@ void GetFullLogFilename(google::LogSeverity severity, string* filename) {
 
 std::string FormatTimestampForLog(MicrosecondsInt64 micros_since_epoch) {
   time_t secs_since_epoch = micros_since_epoch / 1000000;
-  int usecs = micros_since_epoch % 1000000;
+  size_t usecs = micros_since_epoch % 1000000;
   struct tm tm_time;
   localtime_r(&secs_since_epoch, &tm_time);
 
-  return StringPrintf("%02d%02d %02d:%02d:%02d.%06d",
+  return StringPrintf("%02d%02d %02d:%02d:%02d.%06ld",
                       1 + tm_time.tm_mon,
                       tm_time.tm_mday,
                       tm_time.tm_hour,
@@ -407,7 +408,7 @@ ostream& operator<<(ostream &os, const PRIVATE_ThrottleMsg& /*unused*/) {
 #endif
   CHECK(log && log == log->self())
       << "You must not use COUNTER with non-glog ostream";
-  int ctr = log->ctr();
+  size_t ctr = log->ctr();
   if (ctr > 0) {
     os << " [suppressed " << ctr << " similar messages]";
   }
