@@ -17,6 +17,7 @@
 #include "kudu/util/metrics.h"
 
 #include <iostream>
+#include <tuple>
 #include <utility>
 
 #include <gflags/gflags.h>
@@ -69,13 +70,13 @@ void WriteMetricsToJson(JsonWriter* writer,
   writer->EndArray();
 }
 
-template<typename Collection>
-void WriteMetricsToPrometheus(PrometheusWriter* writer,
-                              const Collection& metrics,
-                              const std::string& prefix) {
+void WriteMetricsPrometheus(PrometheusWriter* writer,
+                            const MetricEntity::MetricMap& metrics,
+                            const string& prefix) {
   for (const auto& [name, val] : metrics) {
     WARN_NOT_OK(val->WriteAsPrometheus(writer, prefix),
-                Substitute("Failed to write $0 as Prometheus", name));
+                Substitute("unable to write '$0' ($1) in Prometheus format",
+                           name, val->prototype()->description()));
   }
 }
 
@@ -404,15 +405,25 @@ Status MetricEntity::WriteAsJson(JsonWriter* writer, const MetricJsonOptions& op
 }
 
 Status MetricEntity::WriteAsPrometheus(PrometheusWriter* writer) const {
-  MetricMap metrics;
-  AttributeMap attrs;
+  static const string kIdMaster = "kudu.master";
+  static const string kIdTabletServer = "kudu.tabletserver";
+
+  if (strcmp(prototype_->name(), "server") != 0) {
+    // Only server-level metrics are emitted in Prometheus format as of now,
+    // non-server metric entities are currently silently skipped.
+    //
+    // TODO(KUDU-3563): output tablet-level metrics in Prometheus format as well
+    return Status::OK();
+  }
+
+  // Empty filters result in getting all the metrics for this MetricEntity.
+  //
+  // TODO(aserbin): instead of hard-coding, pass MetricFilters as a parameter
   MetricFilters filters;
   filters.entity_level = "debug";
-  const string master_prefix = "kudu_master_";
-  const string tserver_prefix = "kudu_tserver_";
-  const string master_server = "kudu.master";
-  const string tablet_server = "kudu.tabletserver";
-  // Empty filters results in getting all the metrics for this MetricEntity.
+
+  MetricMap metrics;
+  AttributeMap attrs;
   const auto s = GetMetricsAndAttrs(filters, &metrics, &attrs);
   if (s.IsNotFound()) {
     // Status::NotFound is returned when this entity has been filtered, treat it
@@ -420,21 +431,22 @@ Status MetricEntity::WriteAsPrometheus(PrometheusWriter* writer) const {
     return Status::OK();
   }
   RETURN_NOT_OK(s);
-  // Only emit server level metrics
-  if (strcmp(prototype_->name(), "server") == 0) {
-    if (id_ == master_server) {
-      // attach kudu_master_ as prefix to metrics
-      WriteMetricsToPrometheus(writer, metrics, master_prefix);
-      return Status::OK();
-    }
-    if (id_ == tablet_server) {
-      // attach kudu_tserver_ as prefix to metrics
-      WriteMetricsToPrometheus(writer, metrics, tserver_prefix);
-      return Status::OK();
-    }
+
+  if (id_ == kIdMaster) {
+    // Prefix all master metrics with 'kudu_master_'.
+    static const string kMasterPrefix = "kudu_master_";
+    WriteMetricsPrometheus(writer, metrics, kMasterPrefix);
+    return Status::OK();
+  }
+  if (id_ == kIdTabletServer) {
+    // Prefix all tablet server metrics with 'kudu_tserver_'.
+    static const string kTabletServerPrefix = "kudu_tserver_";
+    WriteMetricsPrometheus(writer, metrics, kTabletServerPrefix);
+    return Status::OK();
   }
 
-  return Status::NotFound("Entity is not relevant to Prometheus");
+  return Status::NotSupported(
+      Substitute("$0: unexpected server-level metric entity", id_));
 }
 
 Status MetricEntity::CollectTo(MergedEntityMetrics* collections,
@@ -735,10 +747,20 @@ void MetricPrototype::WriteFields(JsonWriter* writer,
   }
 }
 
-void MetricPrototype::WriteFields(PrometheusWriter* writer, const string &prefix) const {
+void MetricPrototype::WriteHelpAndType(PrometheusWriter* writer,
+                                       const string& prefix) const {
+  static constexpr const char* const kSummary = "summary";
+
+  // The way how HdrHistogram-backed stats are presented in Kudu metrics
+  // corresponds to a 'summary' metric in Prometheus, not a 'histogram' one [1].
+  //
+  // [1] https://prometheus.io/docs/concepts/metric_types/#summary
+  const auto m_type = type();
+  const char* const metric_type_str =
+      m_type == MetricType::kHistogram ? kSummary : MetricType::Name(m_type);
   writer->WriteEntry(Substitute("# HELP $0$1 $2\n# TYPE $3$4 $5\n",
                                 prefix, name(), description(),
-                                prefix, name(), MetricType::Name(type())));
+                                prefix, name(), metric_type_str));
 }
 
 //
@@ -818,9 +840,8 @@ Status Gauge::WriteAsJson(JsonWriter* writer,
   return Status::OK();
 }
 
-Status Gauge::WriteAsPrometheus(PrometheusWriter* writer, const std::string& prefix) const {
-  prototype_->WriteFields(writer, prefix);
-
+Status Gauge::WriteAsPrometheus(PrometheusWriter* writer, const string& prefix) const {
+  prototype_->WriteHelpAndType(writer, prefix);
   WriteValue(writer, prefix);
 
   return Status::OK();
@@ -976,19 +997,20 @@ void MeanGauge::WriteValue(JsonWriter* writer) const {
   writer->Double(total_count());
 }
 
-void MeanGauge::WriteValue(PrometheusWriter* writer, const std::string& prefix) const {
-  auto output = Substitute("$0$1{unit_type=\"$2\"} $3\n", prefix, prototype_->name(),
-                           MetricUnit::Name(prototype_->unit()), value());
+void MeanGauge::WriteValue(PrometheusWriter* writer, const string& prefix) const {
+  static constexpr const char* const kFmt = "$0$1$2{unit_type=\"$3\"} $4\n";
 
-  SubstituteAndAppend(&output, "$0$1$2{unit_type=\"$3\"} $4\n",
-                       prefix, prototype_->name(), "_count",
-                       MetricUnit::Name(prototype_->unit()), total_count());
+  const char* const name = prototype_->name();
+  DCHECK(name);
+  const char* const unit = MetricUnit::Name(prototype_->unit());
+  DCHECK(unit);
 
-  SubstituteAndAppend(&output, "$0$1$2{unit_type=\"$3\"} $4\n",
-                       prefix, prototype_->name(), "_sum",
-                       MetricUnit::Name(prototype_->unit()), total_sum());
+  string out;
+  SubstituteAndAppend(&out, kFmt, prefix, name, "", unit, value());
+  SubstituteAndAppend(&out, kFmt, prefix, name, "_count", unit, total_count());
+  SubstituteAndAppend(&out, kFmt, prefix, name, "_sum", unit, total_sum());
 
-  writer->WriteEntry(output);
+  writer->WriteEntry(out);
 }
 
 //
@@ -1029,12 +1051,10 @@ Status Counter::WriteAsJson(JsonWriter* writer,
   return Status::OK();
 }
 
-Status Counter::WriteAsPrometheus(PrometheusWriter* writer, const std::string& prefix) const {
-  prototype_->WriteFields(writer, prefix);
-
+Status Counter::WriteAsPrometheus(PrometheusWriter* writer, const string& prefix) const {
+  prototype_->WriteHelpAndType(writer, prefix);
   writer->WriteEntry(Substitute("$0$1{unit_type=\"$2\"} $3\n", prefix, prototype_->name(),
                                 MetricUnit::Name(prototype_->unit()), value()));
-
   return Status::OK();
 }
 
@@ -1094,57 +1114,41 @@ Status Histogram::WriteAsJson(JsonWriter* writer,
   return Status::OK();
 }
 
-Status Histogram::WriteAsPrometheus(PrometheusWriter* writer, const std::string& prefix) const {
-  prototype_->WriteFields(writer, prefix);
-  std::string output = "";
-  MetricJsonOptions opts;
-  // Snapshot is taken to preserve the consistency across metrics and
-  // requirements given by Prometheus. The value for the _bucket in +Inf
-  // quantile needs to be equal to the total _count
-  HistogramSnapshotPB snapshot;
-  RETURN_NOT_OK(GetHistogramSnapshotPB(&snapshot, opts));
+Status Histogram::WriteAsPrometheus(PrometheusWriter* writer,
+                                    const string& prefix) const {
+  static constexpr struct QuantileInfo {
+    const char* const tag;
+    const double quantile;
+  } kQuantiles[] = {
+    { "0.75",   75.0  },
+    { "0.95",   95.0  },
+    { "0.99",   99.0  },
+    { "0.999",  99.9  },
+    { "0.9999", 99.99 },
+  };
+  static constexpr const char* const kFmt =
+      "$0$1{unit_type=\"$2\", quantile=\"$3\"} $4\n";
 
-  output = Substitute("$0$1$2{unit_type=\"$3\", le=\"0.75\"} $4\n",
-                       prefix, prototype_->name(), "_bucket",
-                       MetricUnit::Name(prototype_->unit()),
-                       snapshot.percentile_75());
+  const char* const name = prototype_->name();
+  DCHECK(name);
+  const char* const unit = MetricUnit::Name(prototype_->unit());
+  DCHECK(unit);
 
-  SubstituteAndAppend(&output, "$0$1$2{unit_type=\"$3\", le=\"0.95\"} $4\n",
-                       prefix, prototype_->name(), "_bucket",
-                       MetricUnit::Name(prototype_->unit()),
-                       snapshot.percentile_95());
+  // A snapshot is taken to have more consistent statistics while generating
+  // the output.
+  const HdrHistogram h(*histogram_);
+  string out;
+  SubstituteAndAppend(&out, kFmt, prefix, name, unit, "0", h.MinValue());
+  for (const auto& [tag, q] : kQuantiles) {
+    SubstituteAndAppend(&out, kFmt, prefix, name, unit, tag, h.ValueAtPercentile(q));
+  }
+  SubstituteAndAppend(&out, kFmt, prefix, name, unit, "1", h.MaxValue());
 
-  SubstituteAndAppend(&output, "$0$1$2{unit_type=\"$3\", le=\"0.99\"} $4\n",
-                       prefix, prototype_->name(), "_bucket",
-                       MetricUnit::Name(prototype_->unit()),
-                       snapshot.percentile_99());
+  SubstituteAndAppend(&out, "$0$1_sum $2\n", prefix, name, h.TotalSum());
+  SubstituteAndAppend(&out, "$0$1_count $2\n", prefix, name, h.TotalCount());
 
-  SubstituteAndAppend(&output, "$0$1$2{unit_type=\"$3\", le=\"0.999\"} $4\n",
-                       prefix, prototype_->name(), "_bucket",
-                       MetricUnit::Name(prototype_->unit()),
-                       snapshot.percentile_99_9());
-
-  SubstituteAndAppend(&output, "$0$1$2{unit_type=\"$3\", le=\"0.9999\"} $4\n",
-                       prefix, prototype_->name(), "_bucket",
-                       MetricUnit::Name(prototype_->unit()),
-                       snapshot.percentile_99_99());
-
-  SubstituteAndAppend(&output, "$0$1$2{unit_type=\"$3\", le=\"+Inf\"} $4\n",
-                       prefix, prototype_->name(), "_bucket",
-                       MetricUnit::Name(prototype_->unit()),
-                       snapshot.total_count());
-
-  SubstituteAndAppend(&output, "$0$1$2{unit_type=\"$3\"} $4\n",
-                       prefix, prototype_->name(), "_sum",
-                       MetricUnit::Name(prototype_->unit()),
-                       snapshot.total_sum());
-
-  SubstituteAndAppend(&output, "$0$1$2{unit_type=\"$3\"} $4\n",
-                       prefix, prototype_->name(), "_count",
-                       MetricUnit::Name(prototype_->unit()),
-                       snapshot.total_count());
-
-  writer->WriteEntry(output);
+  prototype_->WriteHelpAndType(writer, prefix);
+  writer->WriteEntry(out);
 
   return Status::OK();
 }
