@@ -21,6 +21,7 @@
 
 #include <ctime>
 #include <limits>
+#include <ostream>
 
 #include <glog/logging.h>
 
@@ -215,6 +216,7 @@ MonoDelta MonoTime::GetDeltaSince(const MonoTime &rhs) const {
 
 void MonoTime::AddDelta(const MonoDelta &delta) {
   DCHECK(Initialized());
+  DCHECK(delta.Initialized());
   nanos_ += delta.nano_delta_;
 }
 
@@ -348,4 +350,11 @@ MonoDelta operator-(const MonoTime& t_end, const MonoTime& t_beg) {
   return MonoDelta(delta);
 }
 
+std::ostream& operator<<(std::ostream& os, const kudu::MonoTime& time) {
+  struct timespec ts;
+  time.ToTimeSpec(&ts);
+  os << ts.tv_nsec << "ns";
+  return os;
+}
+
 } // namespace kudu
