@@ -19,7 +19,6 @@
 
 #include <dirent.h>
 #include <fcntl.h>
-#include <signal.h>
 #if defined(__linux__)
 #include <sys/prctl.h>
 #endif
@@ -52,7 +51,6 @@
 #include "kudu/util/errno.h"
 #include "kudu/util/faststring.h"
 #include "kudu/util/monotime.h"
-#include "kudu/util/path_util.h"
 #include "kudu/util/signal.h"
 #include "kudu/util/status.h"
 #include "kudu/util/stopwatch.h"
@@ -266,8 +264,6 @@ Subprocess::Subprocess(vector<string> argv, int sig_on_destruct)
       fd_state_(),
       child_fds_(),
       sig_on_destruct_(sig_on_destruct) {
-  // By convention, the first argument in argv is the base name of the program.
-  argv_[0] = BaseName(argv_[0]);
 
   fd_state_[STDIN_FILENO]   = PIPED;
   fd_state_[STDOUT_FILENO]  = SHARED;
@@ -281,7 +277,7 @@ Subprocess::~Subprocess() {
   if (state_ == kRunning) {
     LOG(WARNING) << Substitute(
         "Child process $0 ($1) was orphaned. Sending signal $2...",
-        child_pid_, JoinStrings(argv_, " "), sig_on_destruct_);
+        child_pid_.load(), JoinStrings(argv_, " "), sig_on_destruct_);
     WARN_NOT_OK(KillAndWait(sig_on_destruct_),
                 Substitute("Failed to KillAndWait() with signal $0",
                            sig_on_destruct_));
@@ -324,7 +320,7 @@ static int pipe2(int pipefd[2], int flags) {
 Status Subprocess::Start() {
   VLOG(2) << "Invoking command: " << argv_;
   if (state_ != kNotStarted) {
-    const string err_str = Substitute("$0: illegal sub-process state", state_);
+    const string err_str = Substitute("$0: illegal sub-process state", state_.load());
     LOG(DFATAL) << err_str;
     return Status::IllegalState(err_str);
   }
@@ -553,6 +549,20 @@ Status Subprocess::WaitNoBlock(int* wait_status) {
   return DoWait(wait_status, NON_BLOCKING);
 }
 
+Status Subprocess::WaitAndCheckExitCode() {
+  int wait_status;
+  RETURN_NOT_OK(DoWait(&wait_status, BLOCKING));
+  int exit_status;
+  string info_str;
+
+  RETURN_NOT_OK(GetExitStatus(&exit_status, &info_str));
+
+  return exit_status == 0
+    ? Status::OK()
+    : Status::RuntimeError(Substitute("Exit code: $0 ($1)",
+                                      exit_status, info_str));
+}
+
 Status Subprocess::GetProcfsState(int pid, ProcfsState* state) {
   faststring data;
   string filename = Substitute("/proc/$0/stat", pid);
@@ -592,11 +602,15 @@ Status Subprocess::Kill(int signal) {
     LOG(DFATAL) << err_str;
     return Status::IllegalState(err_str);
   }
-  if (kill(child_pid_, signal) != 0) {
+  int ret = kill(child_pid_, signal);
+  if (ret != 0 && ret != ESRCH) {
     return Status::RuntimeError("Unable to kill",
                                 ErrnoToString(errno),
                                 errno);
   }
+  if (ret == ESRCH) {
+    LOG(WARNING) << Substitute("Process $0 ($1) has already exited", program_, child_pid_.load());
+  }
 
   // Signal delivery is often asynchronous. For some signals, we try to wait
   // for the process to actually change state, using /proc/<pid>/stat as a
@@ -630,6 +644,11 @@ Status Subprocess::Kill(int signal) {
 }
 
 Status Subprocess::KillAndWait(int signal) {
+  if (state_ != kRunning) {
+    const string err_str = "Sub-process is not running";
+    LOG(DFATAL) << err_str;
+    return Status::IllegalState(err_str);
+  }
   string procname = Substitute("$0 (pid $1)", argv0(), pid());
 
   // This is a fatal error because all errors in Kill() are signal-independent,
@@ -788,7 +807,7 @@ Status Subprocess::DoWait(int* wait_status, WaitMode mode) {
     return Status::OK();
   }
   if (state_ != kRunning) {
-    const string err_str = Substitute("$0: illegal sub-process state", state_);
+    const string err_str = Substitute("$0: illegal sub-process state", state_.load());
     LOG(DFATAL) << err_str;
     return Status::IllegalState(err_str);
   }
@@ -856,4 +875,8 @@ int Subprocess::ReleaseChildFd(int stdfd) {
   return ret;
 }
 
+bool Subprocess::IsStarted() {
+  return state_ != kNotStarted;
+}
+
 } // namespace kudu
