@@ -39,6 +39,7 @@
 #include <glog/logging.h>
 #include <gtest/gtest-spi.h>
 
+#include "kudu/gutil/strings/escaping.h"
 #include "kudu/gutil/strings/numbers.h"
 #include "kudu/gutil/strings/split.h"
 #include "kudu/gutil/strings/strcat.h"
@@ -66,16 +67,28 @@ DEFINE_string(test_leave_files, "on_failure",
 DEFINE_int32(test_random_seed, 0, "Random seed to use for randomized tests");
 
 DECLARE_string(time_source);
+DECLARE_bool(encrypt_data_at_rest);
 
 using std::string;
 using std::vector;
 using strings::Substitute;
 
+namespace {
+int testIteration = 0;
+} // namespace
+
 namespace kudu {
 
 const char* kInvalidPath = "/dev/invalid-path-for-kudu-tests";
 static const char* const kSlowTestsEnvVar = "KUDU_ALLOW_SLOW_TESTS";
 static const char* const kLargeKeysEnvVar = "KUDU_USE_LARGE_KEYS_IN_TESTS";
+static const char* const kEncryptDataInTests = "KUDU_ENCRYPT_DATA_IN_TESTS";
+static const int kEncryptionKeySize = 16;
+static const uint8_t kEncryptionKey[kEncryptionKeySize] =
+  {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 42};
+static const uint8_t kEncryptionKeyIv[kEncryptionKeySize] =
+  {42, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0};
+static const char* const kEncryptionKeyVersion = "kuduclusterkey@0";
 
 static const uint64_t kTestBeganAtMicros = Env::Default()->NowMicros();
 
@@ -86,6 +99,11 @@ static const uint64_t kTestBeganAtMicros = Env::Default()->NowMicros();
 // This can be checked using the 'IsGTest()' function from test_util_prod.cc.
 bool g_is_gtest = true;
 
+void KuduTestEventListener::OnTestIterationStart(const testing::UnitTest& /*unit_test*/,
+                                                 int iteration) {
+  testIteration = iteration;
+}
+
 ///////////////////////////////////////////////////
 // KuduTest
 ///////////////////////////////////////////////////
@@ -127,10 +145,15 @@ KuduTest::KuduTest()
     // reports an error since the flag is unknown to the gflags runtime.
     google::SetCommandLineOptionWithMode(e.first, e.second, google::SET_FLAGS_DEFAULT);
   }
+
+  if (EnableEncryption()) {
+    SetEncryptionFlags(true);
+  }
+
   // If the TEST_TMPDIR variable has been set, then glog will automatically use that
   // as its default log directory. We would prefer that the default log directory
   // instead be the test-case-specific subdirectory.
-  FLAGS_log_dir = GetTestDataDirectory();
+  FLAGS_log_dir = test_dir_;
 }
 
 KuduTest::~KuduTest() {
@@ -178,6 +201,25 @@ void KuduTest::OverrideKrb5Environment() {
   setenv("KRB5CCNAME", kInvalidPath, 1);
 }
 
+void KuduTest::SetEncryptionFlags(bool enable_encryption) {
+  FLAGS_encrypt_data_at_rest = enable_encryption;
+  if (enable_encryption) {
+    Env::Default()->SetEncryptionKey(kEncryptionKey, kEncryptionKeySize * 8);
+  }
+}
+
+void KuduTest::GetEncryptionKey(string* key, string* iv, string* version) {
+  if (FLAGS_encrypt_data_at_rest) {
+    strings::b2a_hex(kEncryptionKey, key, kEncryptionKeySize);
+    strings::b2a_hex(kEncryptionKeyIv, iv, kEncryptionKeySize);
+    *version = kEncryptionKeyVersion;
+  } else {
+    *key = "";
+    *iv = "";
+    *version = "";
+  }
+}
+
 ///////////////////////////////////////////////////
 // Test utility functions
 ///////////////////////////////////////////////////
@@ -186,6 +228,8 @@ bool AllowSlowTests() { return GetBooleanEnvironmentVariable(kSlowTestsEnvVar);
 
 bool UseLargeKeys() { return GetBooleanEnvironmentVariable(kLargeKeysEnvVar); }
 
+bool EnableEncryption() { return GetBooleanEnvironmentVariable(kEncryptDataInTests); }
+
 void OverrideFlagForSlowTests(const std::string& flag_name,
                               const std::string& new_value) {
   // Ensure that the flag is valid.
@@ -223,8 +267,9 @@ string GetTestDataDirectory() {
   // The directory name includes some strings for specific reasons:
   // - program name: identifies the directory to the test invoker
   // - timestamp and pid: disambiguates with prior runs of the same test
+  // - iteration: identifies the iteration when using --gtest_repeat
   //
-  // e.g. "env-test.TestEnv.TestReadFully.1409169025392361-23600"
+  // e.g. "env-test.TestEnv.TestReadFully.1409169025392361-23600-0"
   //
   // If the test is sharded, the shard index is also included so that the test
   // invoker can more easily identify all directories belonging to each shard.
@@ -233,13 +278,14 @@ string GetTestDataDirectory() {
   if (shard_index && shard_index[0] != '\0') {
     shard_index_infix = Substitute("$0.", shard_index);
   }
-  dir += Substitute("/$0.$1$2.$3.$4-$5",
+  dir += Substitute("/$0.$1$2.$3.$4-$5-$6",
     StringReplace(google::ProgramInvocationShortName(), "/", "_", true),
     shard_index_infix,
     StringReplace(test_info->test_case_name(), "/", "_", true),
     StringReplace(test_info->name(), "/", "_", true),
     kTestBeganAtMicros,
-    getpid());
+    getpid(),
+    testIteration);
   Status s = Env::Default()->CreateDir(dir);
   CHECK(s.IsAlreadyPresent() || s.ok())
     << "Could not create directory " << dir << ": " << s.ToString();
@@ -392,12 +438,12 @@ int CountOpenFds(Env* env, const string& path_pattern) {
 }
 
 namespace {
+const vector<string> kWildcard = { "0.0.0.0" };
+
 Status WaitForBind(pid_t pid, uint16_t* port,
                    const vector<string>& addresses,
                    const char* kind,
                    MonoDelta timeout) {
-  static const vector<string> kWildcard = { "0.0.0.0" };
-
   // In general, processes do not expose the port they bind to, and
   // reimplementing lsof involves parsing a lot of files in /proc/. So,
   // requiring lsof for tests and parsing its output seems more
@@ -477,6 +523,70 @@ Status WaitForBind(pid_t pid, uint16_t* port,
   LOG(FATAL) << "could not determine bound port the process";
   __builtin_unreachable();
 }
+
+Status WaitForBindAtPort(const vector<string>& addresses,
+                         uint16_t port,
+                         const char* kind,
+                         MonoDelta timeout) {
+  string lsof;
+  RETURN_NOT_OK(FindExecutable("lsof", {"/sbin", "/usr/sbin"}, &lsof));
+  const vector<string> cmd = { lsof, "-wbnP", "-Fpfn", "-a", "-i", kind };
+
+  // The '-Fpfn' flag gets lsof to output something like:
+  //   p2133
+  //   f549
+  //   n*:8038
+  //   f550
+  //   n*:8088
+  //   p5801
+  //   f548
+  //   n127.0.0.1:43954->127.0.0.1:43617
+  //   p95857
+  //   f3
+  //   n127.0.0.1:63337
+  //
+  // The first line is the pid for every process of the user. We ignore it.
+  // Subsequent lines come in pairs. In each pair, the first half of the pair
+  // is file descriptor number, we ignore it.
+  // The second half has the bind address and port.
+  const MonoTime deadline = MonoTime::Now() + timeout;
+  const auto& addresses_to_check = addresses.empty() ? kWildcard : addresses;
+  for (int64_t i = 1; ; ++i) {
+    string lsof_out;
+    RETURN_NOT_OK(Subprocess::Call(cmd, "", &lsof_out));
+    StripTrailingNewline(&lsof_out);
+    const vector<string> lines = strings::Split(lsof_out, "\n");
+
+    for (const auto& addr : addresses_to_check) {
+      const string addr_pattern = Substitute(
+          "n$0:$1", addr == "0.0.0.0" ? "*" : addr, port);
+      for (const auto& l : lines) {
+        if (l.empty()) {
+          return Status::RuntimeError("unexpected lsof output", lsof_out);
+        }
+        if (l[0] == 'p' || l[0] == 'f') {
+          continue;
+        }
+        if (l[0] == 'n') {
+          if (l == addr_pattern) {
+            return Status::OK();
+          }
+          continue;
+        }
+        return Status::RuntimeError("unexpected lsof output", lsof_out);
+      }
+    }
+
+    if (deadline < MonoTime::Now()) {
+      break;
+    }
+    SleepFor(MonoDelta::FromMilliseconds(i * 10));
+  }
+
+  return Status::TimedOut(
+      Substitute("time out waiting for port $0 to be bound", port));
+}
+
 } // anonymous namespace
 
 Status WaitForTcpBind(pid_t pid, uint16_t* port,
@@ -491,6 +601,18 @@ Status WaitForUdpBind(pid_t pid, uint16_t* port,
   return WaitForBind(pid, port, addresses, "4UDP", timeout);
 }
 
+Status WaitForTcpBindAtPort(const vector<string>& addresses,
+                            uint16_t port,
+                            MonoDelta timeout) {
+  return WaitForBindAtPort(addresses, port, "4TCP", timeout);
+}
+
+Status WaitForUdpBindAtPort(const vector<string>& addresses,
+                            uint16_t port,
+                            MonoDelta timeout) {
+  return WaitForBindAtPort(addresses, port, "4UDP", timeout);
+}
+
 Status FindHomeDir(const string& name, const string& bin_dir, string* home_dir) {
   string name_upper;
   ToUpperCase(name, &name_upper);
